;; bookmark-menu.el --- setup a menu of bookmarks

;; Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.

;; Author: Max R. Andersen <max@cs.auc.dk>
;; Created: July 28 2001
;; Keywords: customization, bookmarks

;; This file is *NOT* part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; This package maintains a menu of bookmarks directly in the menu
;; bar.  I was getting tired of having to enter the Search/Bookmarks
;; menu and then choose a bookmark from a popup-menu.
;;
;; I found it peculiar that I could not find anything like this
;; for Emacs and its bookmarks/registers, so I created this.
;;
;; The code is highly inspired by recentf.el for the management of the menu.
;;
;; Installation/Usage:
;; (require 'bookmark-menu)
;; (bookmark-menu-mode 1)

;;; TODO:
;;  * Add a Create bookmark functionallity (or simply just embed the
Search/Bookmarks menu ?)
;;  * Include annotations in the menu (a simple extension - just
;;    update bookmark-menu-make-menu-item and
;;    bookmark-menu-make-menu-items)
;;  * [WISH] Hierachial bookmarks (just like bookmarks/hotlist/favorites in
browsers)

;;; History:

;;  30-Jul-2001: First official release, just a simple
;;  version which lists the current bookmarks nothing more - nothing
;;  less.
;;


;;; Code:


(require 'bookmark)
(require 'easymenu)

(defvar bookmark-menu-initialized-p nil
  "Non-nil if bookmark-menu already initialized.")

;; IMPORTANT: This function must be defined before the following defcustoms
;; because it is used in their :set clause. To avoid byte-compiler warnings
;; the `symbol-value' function is used to access the `bookmark-menu-path'
;; and `bookmark-menu-title' values.
(defun bookmark-menu-customization-changed (sym val)
  "Function called when menu customization has changed.
It removes the bookmark menu and forces its complete redrawing.  SYM is
the variable customized and VAL its new value."
  (when bookmark-menu-initialized-p
    (easy-menu-remove-item nil
                           (symbol-value 'bookmark-menu-path)
                           (symbol-value 'bookmark-menu-title))
    (setq bookmark-update-menu-p t))
  (set-default sym val))

(defgroup bookmark-menu nil
  "Maintain a menu of bookmarks."
;  :version "21.1" ; what does this one do ?
  :group 'bookmark)

(defcustom bookmark-menu-action 'bookmark-jump
  "*Function to invoke with a bookmark name item of the bookmarks menu.
The default action `bookmark-jump' jump directly to the selected bookmark."
  :group 'bookmark-menu
  :type 'function
  :set 'bookmark-menu-customization-changed)

(defcustom bookmark-menu-title "Bookmarks"
  "*Name of the bookmark menu."
  :group 'bookmark-menu
  :type 'string
  :set 'bookmark-menu-customization-changed)

(defcustom bookmark-menu-mode nil
  "Toggle bookmark menu mode.
When bookmark menu mode is enabled, it maintains a menu for bookmarks.

Setting this variable directly does not take effect;
use either \\[customize] or the function `bookmark-menu-mode'."
  :set (lambda (symbol value)
         (bookmark-menu-mode (or value 0)))
  :initialize 'custom-initialize-default
  :type 'boolean
  :group 'bookmark-menu
  :require 'bookmark-menu)

(defcustom bookmark-menu-path nil
  "*Path where to add the bookmark menu.
If nil add it at top level (see also `easy-menu-change')."
  :group 'bookmark-menu
  :type '(choice (const :tag "Top Level" nil)
                 (sexp :tag "Menu Path"))
  :set 'bookmark-menu-customization-changed)

(defvar bookmark-menu-update-menu-p t
  "Non-nil if the bookmark menu must be updated.")


(defcustom bookmark-menu-before nil
  "*Name of the menu before which the bookmark menu will be added.
If nil add it at end of menu (see also `easy-menu-change')."
  :group 'bookmark-menu
  :type '(choice (string :tag "Name")
                 (const :tag "Last" nil))
  :set 'bookmark-menu-customization-changed)

(defun bookmark-menu-update-menu-hook ()
  "Update the bookmark from the current 'bookmark-alist'."
  (when bookmark-menu-update-menu-p
;    (condition-case nil ;; eat any possible errors
        (progn
          (setq bookmark-menu-update-menu-p nil)
          (easy-menu-change bookmark-menu-path
                            bookmark-menu-title
                            (bookmark-menu-make-menu-items)
                            bookmark-menu-before)
          (message "Bookmark menu updated...%s" (setq
bookmark-menu-update-count (+ bookmark-menu-update-count 1)))
          t)
 ;     (error nil))
        ))

(defvar bookmark-menu-update-count 1)

;;;###autoload
(defun bookmark-menu-mode (&optional arg)
  "Toggle bookmark-menu mode.
With prefix ARG, turn bookmark-menu mode on if and only if ARG is positve.
Returns the new status of bookmark-menu mode (non-nil means on).

When bookmark menu mode is enabled, it maintains a menu for visiting
files that were operated on recently."
  (interactive "P")
  (let ((on-p (if arg
                  (> (prefix-numeric-value arg) 0)
                (not bookmark-menu-mode))))
    (cond
     ;; bookmark-menu-mode enabled
     (on-p
      (unless bookmark-menu-initialized-p
        (setq bookmark-menu-initialized-p t)
        (setq bookmark-menu-update-menu-p t)
        ;; TODO: addhook each time a bookmark is added/deleted ?
        (add-hook (if (featurep 'xemacs)
                      'activate-menubar-hook
                    'menu-bar-update-hook)
                  'bookmark-menu-update-menu-hook)))
      ;; bookmark-menu disabled.
      (bookmark-menu-initialized-p
       (setq bookmark-initialized-p nil)
       (easy-menu-remove-item bookmark-menu-path bookmark-menu-title)
       (remove-hook (if (featurep 'xemacs)
                        'activate-menubar-hook
                      'menu-bar-update-hook)
                   'bookmark-update-menu-hook)))
      (setq boomark-mode on-p)))

(defun bookmark-menu-make-menu-items ()
  "Make menu items from 'bookmark-alist'."
  (let ((bookmark-items
         (mapcar 'bookmark-menu-make-menu-item
                 (bookmark-all-names))
        ))
    (append (or bookmark-items (list ["No bookmarks" t nil])))
    ))


(defun bookmark-menu-make-menu-item (menu-element)
  "Make a menu item from MENU-ELEMENT (see 'bookmark-make-menu-element')."
  (vector menu-element
          (list bookmark-menu-action menu-element)
          )
  )

(provide 'bookmark-menu)


