;;; chmod.el --- Make the current file executable, if necessary.

;; Copyright (C) 2000 by Free Software Foundation, Inc.

;; Author: Steve Kemp <skx@tardis.ed.ac.uk>
;; Keywords: Convienence

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;;  This is a simple package that will automatically give 
;; execute permissions to a local file that begins with a
;; sheban line.
;;

;;; Usage

;;  Place this file somewhere on your `load-path' - optionally
;; byte-compiling it, then place the following line in your
;; .emacs file.
;;
;;    (require 'chmod)


;;; Notes

;;  The shebang line is not tested for a valid interpretter...
;;
;;  Under a Windows system, or when you are editting a remote file
;; via ange-ftp, no changes are made.


;;; History:

;;  Initial Revision.


;;; Code:

(defvar chmod-remote-file-regexp "^/[^/:]*:"
  "Regexp to match for remote filename.")

(defun chmod-is-remote-file (filename)
  "Return t if FILENAME is a valid remote file name for ange-ftp."
  (cond ((and filename
              (string-match chmod-remote-file-regexp filename)) t)))

(defun chmod-is-ange-ftp ()
  "Return t if the current buffer contain a remote ange-ftp file."
  (interactive)
  (chmod-is-remote-file (buffer-file-name)))


(defun chmod-if-script ()
  "Make the file in the current buffer executable if it is not
already.
If file already has any execute bits set at all, do not change
existing
file modes.
The file changing modes will not attempt to change the mode of a
remote
file, or a file on a Windows machine."
  (if' (or (chmod-is-ange-ftp)
	  (eq 'windows-nt system-type))
      nil
    (progn
      (and (save-excursion
	     (save-restriction
	       (widen)
	       (goto-char (point-min))
	       (save-match-data
		 (looking-at "^#!")
		 )))
 	   (let* ((current-mode (file-modes (buffer-file-name)))
		  (add-mode (logand ?\111 (default-file-modes))))
	     (or (/= (logand ?\111 current-mode) 0)
		 (zerop add-mode)
		 (message "Changing mode to executable")
		 (set-file-modes (buffer-file-name)
				 (logior current-mode add-mode))))))))


;; Install the hook.
(add-hook 'after-save-hook 'chmod-if-script)

(provide 'chmod)
;;; chmod.el ends here


