;;; dekeys.el --- disable and enable keys withouth modifying keymaps
;;
;; Author    : Juanma Barranquero <lektu@uol.com.br>
;; Created On: <2001-03-12 12:50:27 lektu>
;; Time-stamp: <2001-03-13 16:29:58 lektu>
;; Version   : 1.0
;;
;; Copyright (C) 2001 Juanma Barranquero
;;
;; Keywords: convenience keyboard
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Description:

;; This package define commands `dek-enable-key' and `dek-disable-key'
;; (and aliases `enable-key' and `disable-key', if possible) to allow
;; dynamically disabling and re-enabling keys without the need to
;; search and modify the right keymap.

;; Keys can be disabled and enabled at any time, but the disabling
;; only takes effect when `dek-mode' is active. This option can be set
;; through customize, or by using the function `dek-mode' with a
;; positive argument. Setting the variable to t in .emacs before
;; loading dekeys is also supported (though not recommended).

;; Please note that dekeys disables keys, not commands. That means
;; that if you have `end-of-buffer' both in M-> and C-end, disabling
;; M-> will not disable C-end, although you can disable that key too
;; (each command "knows" in which keys has been disabled).

;; NOTE: A current limitation is that dekeys can only disable keys
;; bound to a symbol naming an interactively-callable function. No
;; intent has been made to support any other kind of key/command
;; disabling.

;; The disabling of keys with dekeys is neither global nor local. It
;; can be thought of as associating a disable flag with a pair (key,
;; command). Whenever a dekeys-disabled command is invoked by the user
;; (not from a lisp function), it checks if that key is disabled for
;; the command. If so, the command is not executed. But the binding
;; between a key and a command depends of course of the current
;; keymaps. So in other words, if you disable "q" in a dired buffer,
;; you'll disable `quit-window' in dired buffers and also in any other
;; mode that has the same binding, but "q" will still be a
;; self-inserting key in text buffers, for example, and `quit-window'
;; will work if invoked from any other key.

;; dekeys works by "perverting" the standard use of the 'disabled
;; attribute and the `disabled-command-hook' hook and function. If you
;; modify this hook in your setup, it is *much* better if you do it
;; before loading dekeys. If that's not practical you can try setting
;; the option `dek-watch-hook-modifications', that advises `add-hook'
;; and `remove-hook' to "carry" any modification to that hook to
;; `dek-original-hook'. It is of course still possible to modify the
;; hook through `setq', but you shouldn't setq a hook, should you?

;; dekeys tries to err on the safe side, and even if you disable and
;; reenable a key associated to a previously disabled command (like
;; `erase-buffer', for example, supposing you've bound it to a key),
;; it should do the right thing. The "right" thing, by the way, is, in
;; that order (the first option executed finishes the processing):

;; - If the command is not a symbol bound to a function, call the
;;   default hook.

;; - If the command is disabled by dekeys, silently do nothing. The
;;   normal hook `dek-disabled-hook' is run.

;; - If the command is disabled in the standard way and the value of
;;   `disabled-command-hook' (now on `dek-original-hook') is nil,
;;   execute the command.

;; - If the command is disabled in the standard way and the value of
;;   `disabled-command-hook' is not nil, run the hook. Supposing it
;;   has its default value, that calls the `disabled-command-hook'
;;   function to ask the user what to do.

;; - If the command is not disabled in any way, execute the command.

;; All this sounds very complicated, but the gist of it is: either
;; process the command through dekeys, or if not appropriate, pass it
;; along to the default processing. Still, the whole matter *is*
;; complex and I cannot be sure that it works 100% right, so

;; *** I ACCEPT NO RESPONSABILITY FOR THE USE OF THIS MODULE AND ***
;; *** I DON'T MAKE ANY WARRANTY ABOUT IT OR ITS WORKING.        ***

;; Use it at your own risk. It can fail, it can work differently that
;; stated, it can eat your work and set fire to your computer. YOU
;; HAVE BEEN WARNED. That said, I think it works as expected, but you
;; shouldn't experiment with it while editing valuable information.

;; Please send comments, suggestions, bug reports, etc. to
;; Juanma Barranquero <lektu@uol.com.br>

;;; Code:

;; Customization info is at the end of the file (courtesy of
;; `custom-initialize-reset') :(

(eval-when-compile (require 'cl))

;;; Internal

;; To keep value of disabled-command-hook
(defvar dek-original-hook disabled-command-hook)

(setq disabled-command-hook 'dek-disabled-command-func)

;; Check that the binding of the key is a commandp symbol
(defun dek-get-command (key)
  (let ((binding (key-binding key)))
    (and (symbolp binding)
         (commandp binding)
         binding)))

;; Default processing. Depends on the original value of
;; disabled-command-hook.
(defun dek-do-default-processing ()
  (if (not dek-original-hook)
      (call-interactively this-command)
    (run-hooks 'dek-original-hook)))

;; Function to be run from the hook disabled-command-hook.
;; If necessary, it runs disabled-command-hook for disabled
;; commands that aren't dek-disabled.
(defun dek-disabled-command-func (&rest ignore)
  (let* ((key (this-command-keys))
         (command (dek-get-command key))
         (keys-disabled (get command 'dek-keys-disabled))
         (disabled (get command 'dek-original-disabled)))
    (cond ((not (and command keys-disabled))            ; not procesed by dekeys
           (dek-do-default-processing))
          ((and dek-mode (member key keys-disabled))    ; dek-disabled
           (run-hooks 'dek-disabled-hook))
          ((not (eq disabled 'not))                     ; initially disabled
           (dek-do-default-processing))
          (t (call-interactively this-command)))))      ; initially enabled

;; To watch for additions to disable-command-hook
(defadvice add-hook (before dek-advice activate compile disable)
  (when (and dek-mode (eq (ad-get-arg 0) 'disabled-command-hook))
    (ad-set-arg 0 'dek-original-hook)))

;; To watch for deletions from disable-command-hook
(defadvice remove-hook (before dek-advice activate compile disable)
  (when (and dek-mode (eq (ad-get-arg 0) 'disabled-command-hook))
    (ad-set-arg 0 'dek-original-hook)))

;; Activate/deactivate the advice of hook functions.
(defun dek-toggle-advice (value)
  (let ((ad-func (if value 'ad-enable-advice 'ad-disable-advice))
        (params '(before dek-advice)))
    (when dek-mode
      (ignore-errors
        (apply ad-func 'add-hook params)
        (apply ad-func 'remove-hook params))
      (ad-activate 'add-hook)
      (ad-activate 'remove-hook))))

;;; User interface

(defun dek-mode (arg)
  "Toggle disabling of keys.
With arg, activate disabling of keys iff ARG is positive.
The functions `dek-enable-key' and `dek-disable-key' can be
used to enable/disable keys without modifying their bindings.
Disabled keys are marked as such even if `dek-mode' is nil,
but there is no effect until `dek-mode' is activated again."
  (interactive "P")
  (let ((mode (if arg
                  (> (prefix-numeric-value arg) 0)
                (not dek-mode))))
    (unless (eq mode dek-mode)
      (dek-toggle-advice mode)
      (setq dek-mode mode))))

(defun dek-disable-key (key)
  "*Disable KEY without removing its binding.
Use `dek-enable-key' to re-enable it."
  (let* ((command (dek-get-command key))
         (keys-disabled (get command 'dek-keys-disabled))
         (disabled (get command 'disabled)))
    (unless (get command 'dek-original-disabled)
      (put command 'dek-original-disabled (or disabled 'not)))
    (put command 'disabled (or disabled t))
    (unless (member key keys-disabled)
      (put command 'dek-keys-disabled (cons key keys-disabled)))))

(defun dek-enable-key (key)
  "*Enable KEY, that was previously disabled with `dek-disable-key'."
  (let* ((command (dek-get-command key))
         (keys-disabled (delete key (get command 'dek-keys-disabled))))
    (put command 'dek-keys-disabled keys-disabled)
    (unless keys-disabled
      (put command 'disabled (let ((disabled (get command 'dek-original-disabled)))
                               (if (eq disabled 'not) nil disabled)))
      (put command 'dek-original-disabled nil))))

(unless (fboundp 'enable-key) (defalias 'enable-key 'dek-enable-key))
(unless (fboundp 'disable-key) (defalias 'disable-key 'dek-disable-key))

;;; Customization

;; Help function for customization
(defun dek-set-variable (symbol value)
  (cond ((eq symbol 'dek-watch-hook-modifications)
         (dek-toggle-advice value)
         (set symbol value))
        ((eq symbol 'dek-mode)
         (dek-mode value))))

(defgroup dekeys nil
  "Disabling of keys without modifying any keymap."
  :group 'keyboard)

(defcustom dek-mode nil
  "*Activate disabling of keys.
Setting this variable directly does not take effect;
use either M-x customize or the function `dek-mode'."
  :type 'boolean
  :initialize 'custom-initialize-reset
  :set 'dek-set-variable
  :require 'advice
  :group 'dekeys)

(defcustom dek-watch-hook-modifications nil
  "*Watch for modifications to `disable-command-hook'.
When non-nil, modifications of `disable-command-hook' by
hook functions `add-hook' and `remove-hook' are carried
over to `dek-original-hook'."
  :type 'boolean
  :initialize 'custom-initialize-reset
  :set 'dek-set-variable
  :group 'dekeys)

(defcustom dek-disabled-hook nil
  "*Normal hook run instead of a dek-disabled command."
  :type 'hook
  :group 'dekeys)

(provide 'dekeys)

;; dekeys.el ends here


