;;; eebash.el - Transparently/selectively export elisp code to the bash console.
;;
;; $Id: eebash.el,v 1.5 2001/04/25 03:19:30 burton Exp $
;;
;; Copyright (C) 2001-2004 Free Software Foundation, Inc.
;; Copyright (C) 2001-2004 Kevin A. Burton (burton@openprivacy.org)
;;
;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 0.1.0
;;
;; This file is [not yet] part of GNU Emacs.
;;
;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; The eebash (Emacs Execution under Bash) package provides a mechanism to
;; selectively export lisp functions into bash.  This has a lot of potential to
;; bridge the gap between a quick console and the full development environment
;; of Emacs.
;;
;; Technically it is setup so that you alias all the elisp function you need to
;; an 'eebash' bash function.  This will then perform everything necessary to
;; 'bootup' Emacs, load the necessary libraries, and execute your command.
;;  
;; Advantages:
;;
;;   - startup emacs fast (without site-lisp and without user startup file) 
;;
;;   - provide --help option to all exported command (uses `describe-function'
;;     style output.
;;
;;   - check all function parameters to make sure everything is correct, if you
;;     don't provide enough parameters you will get an error message explaining
;;     the syntax of the exported Lisp function.
;;

;;; Install:
;;
;; This is not your typical Emacs List install.  You need to set the variable
;; EEBASH_LISP_SOURCE to point to this script.
;;
;; Then in your .bashrc you should 'source' the eebash.sh file.  Then you should
;; setup aliases to all your commands.
;;
;; Example .bashrc --
;;
;; #tell eebash.sh where to obtain the eebash.el script
;; export EEBASH_LISP_SOURCE="~/el/eebash.el"
;;
;; #include/source the eebash.sh script
;; source ~/bin/eebash.sh
;;
;; #setup all my eebash aliases
;; alias byte-recompile-directory="eebash byte-recompile-directory"
;; alias byte-compile-file="eebash byte-compile-file"
;; alias eebash-hello-world="eebash eebash-hello-world"

;;;
;; TODO:
;;
;; - Since I am already printing out a lisp tmp file and loading it.  Maybe we
;;   don't need the eebash.el file... maybe I can include this as header info in
;;   the temp file..??
;;
;; - How do we autoload lisp files that are in other directories???  Maybe an
;;   environment variable?
;;
;; - remove temp files when emacs is done with them.
;;
;; - dump the stack if anything breaks under emacs.  
;;
;; - What would be necessary to run this under XEmacs???
;;
;; - rewrite the eebash.sh script so that it is 'eebash' which is executed
;;   directly.  This will allow us to avoid the functions needing to be in the
;;   environment.  When you execute 'set' you should not see these functions. 
;;
;;
;; - ability to pass parameters to emacs:
;;     -load
;;
;; - Some functions pay attention to when they are called interactively.  If
;;   these functions are not called interactively then they may not output text
;;   via (message) IE emacs-version.  Before we call a function make sure we
;;   tell it it is being called interactively...
;;
;; - If a function definition is in a lisp file we need to load the function
;;   should be  function:library and we need to call (load-library library)



;;;
;; History:
;;
;; - Sat Apr 21 01:14:57 2001 (burton@relativity.yi.org): enable the --help 
;;   option...
;;
;; - Sat Apr 21 04:40:52 2001 (burton@relativity.yi.org): if we don't give a 
;;   function the right command arguments, make sure we give a NICE message 
;;   (maybe --help) and don't just return :(.  For some reason Emacs will screw 
;;   up the console :(
;;
;; - Sun Apr 22 14:38:44 2001 (burton@relativity.yi.org): echo the load path
;;   when we want help.
;;

(defvar eebash-version "0.1.0" "eebash version info.")

(defun eebash-funcall(function &optional parameters)
  "Go over the given function and all parameters and make sure these are valid.  
If we don't have the correct number of arguments, then we should output help 
via the `eebash-describe-function'.  After this execute the function with the
given paramters and output the result."

  (eebash-init)
  
  (eebash-load-function function)

  (let(arglist parameter-length min max)

    ;;make sure we have the minimum required parameters.
    (setq arglist (eebash-get-arglist function))
    
    (setq parameter-length (length parameters))

    (setq min (eebash-get-arglist-min arglist))

    (setq max (eebash-get-arglist-max arglist))

    ;;if the parameter length is not between min and max the user has either
    ;;entered too many params or too few.

    (if (and (>= parameter-length min)
             (<= parameter-length max))
        (progn

          ;;OK.  we can now call the function

          (apply function parameters))

      (message "%s: invalid syntax:\n" (symbol-name function))

      (eebash-describe-function function))))

(defun eebash-get-arglist-min(arglist)
  "Get the minimum number of arguments required from the given arglist."

  (let((i 0)
       (count 0)
       (optional-found nil)
       arg)

    (while (and (< i (length arglist))
                (not optional-found))

      (setq arg (elt arglist i))

      (if (string-equal arg "&optional")
          (setq optional-found t)
        (setq count (1+ count)))
      
      (setq i (1+ i)))
    
    count))

(defun eebash-get-arglist-max(arglist)
  "Get the maximum number of arguments required from the given arglist."

  (let((i 0)
       (count 0)
       arg)

    (while (< i (length arglist))

      (setq arg (elt arglist i))

      (if (not (string-equal arg "&optional"))
          (setq count (1+ count)))
      
      (setq i (1+ i)))
    
    count))

(defun eebash-load-function(function)
  "If the given function is not loaded, because it is set to autoload, then we
should load it.  This will only happen once"

  (let(sf)

    (setq sf (symbol-function function))

    ;;FIXME: destroy the "loading..." message on the console..  Is this possible
    ;;at all?? Can we disable the 'message' function???
    (if (and (sequencep sf)
             (equal (elt sf 0)
                    'autoload))
        (progn
          (load-library (elt sf 1))
          (message "")))))

(defun eebash-get-arglist(def)
  "Get the argument list of the given function.  The variable `def' must be the
function definition you want to find.  If passed a function a function
definition will be used.  "

  (if (symbolp def)
      (setq def (indirect-function (symbol-function function))))

  (cond ((byte-code-function-p def)
         (car (append def nil)))
        ((eq (car-safe def) 'lambda)
         (nth 1 def))
        (t t)))

(defun eebash-describe-function (function)
  "Display the full documentation and syntax of FUNCTION (a symbol).  See
`describe-function'."

  ;;init eebash...
  (eebash-init)
  
  ;;make sure we load the function library...  if we don't load it we have no
  ;;way to figure out what the function definition should look like
  (eebash-load-function function)

  (eebash-show-help)
  
  (message "%s is an exported GNU Emacs Lisp function." function)
  
  ;;(message "snarfing documentation...")
  ;;(Snarf-documentation (concat doc-directory "DOC-" emacs-version))
  
  (message
   (with-output-to-string 
     
     (eebash-describe-function-1 function nil (interactive-p))
     (save-excursion
       (set-buffer standard-output)
       ;; Return the text we displayed.
       (buffer-string)))))

(defun eebash-describe-function-1 (function parens interactive-p)

  
  (let* ((def (if (symbolp function)
                  (symbol-function function)
                function))
         file-name string need-close
         (beg
          (if (commandp def)
              "an interactive " "a ")))
    
    ;; Handle symbols aliased to other symbols.
    (setq def (indirect-function def))

    ;; If definition is a macro, find the function inside it.
    (if (eq (car-safe def) 'macro)
        (setq def (cdr def)))

    (let ((arglist (eebash-get-arglist def)))

      (if (listp arglist)
          (progn
            (princ (cons
                    (if (symbolp function)
                        function "anonymous")
                    (mapcar (lambda (arg)
                              (if (memq arg '(&optional &rest))
                                  arg
                                (intern (upcase (symbol-name arg)))))
                            arglist)))
            (terpri))))
    (let ((doc (documentation function)))
      (if doc
          (progn (terpri)
                 (princ doc))
        (princ "not documented")))))

(defun eebash-init()

  (add-to-list 'load-path (expand-file-name "~/el"))
  (add-to-list 'load-path (expand-file-name "~/elisp"))

  ;;FIXME: need to load the library this function is in.
  ;;(load-library "my-commands")
  
  )

(defun eebash-show-help()
  "Show eebash generic eebash help.  This is meant to be called from the command
line."

    (message (concat "eebash version "
                     eebash-version
                     " - "
                     "Transparently export GNU Emacs elisp code to the bash console."))

    (message "")

    (message "Using load-path:")

    (message "")
    
     (let(load-path-entry (i 0))
      
      (while (< i (length load-path))
        
        (setq load-path-entry (elt load-path i))
        
        (message load-path-entry)
        
        (setq i (1+ i))))

    
    (message ""))

(defun eebash-hello-world()
  "Test function which outputs hello world to the console."
  
  (message "Hello World"))

(provide 'eebash)
