;; emoney.el --- A home finance package  -*-Emacs-Lisp-*-

;; Copyright (C) 2003 Steve Youngs

;; RCS: $Id: emoney.el,v 1.1.1.1 2003/06/03 17:00:07 youngs Exp $
;; Author:        Steve Youngs <youngs@xemacs.org>
;; Maintainer:    Steve Youngs <youngs@xemacs.org>
;; Created:       <2003-06-04>
;; Last-Modified: <2003-06-04 02:59:02 (steve)>
;; Homepage:      none yet, email the maintainer for latest version.
;; Keywords:      money finance banking cash

;; This file is part of eMoney.

;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions
;; are met:
;;
;; 1. Redistributions of source code must retain the above copyright
;;    notice, this list of conditions and the following disclaimer.
;;
;; 2. Redistributions in binary form must reproduce the above copyright
;;    notice, this list of conditions and the following disclaimer in the
;;    documentation and/or other materials provided with the distribution.
;;
;; 3. Neither the name of the author nor the names of any contributors
;;    may be used to endorse or promote products derived from this
;;    software without specific prior written permission.
;;
;; THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
;; IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;; DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

;;; Commentary:
;;
;;  eMoney is based on balance.el, originally by Jason Baietto
;;  <jason@ssd.csd.harris.com> and later maintained by Bob Newell
;;  <bnewell@alum.mit.edu>.
;;
;;  eMoney tries to give the user a reasonably self-contained finance
;;  solution.  The target audience is the home user and possibly the
;;  small (very small) business operator.  It probably will never be
;;  compatible with any commercial finance packages available.
;;
;;  Right now, eMoney is nothing more than a very simple cash book
;;  program.  Use it to balance your cheque book.  See the "Todo:"
;;  section for a partial list of possible upcoming features.
;;
;;  Installation:
;;
;;    Byte-compile this and put it in your load-path.  And...
;;
;;    (require 'emoney)
;;    (add-to-list 'auto-mode-alist '("\\.emy$" . emoney-mode))
;;
;;  If this came to you as an XEmacs package, or if you have set it up
;;  that way, those 2 lines are not needed.
;;
;;    The last step is simply to do a find-file on foo.emy.
;;
;; Please note that I use XEmacs exclusively, I have no idea whether or
;; not this will run or even byte-compile with GNU/Emacs.  Also, I have
;; no desire or intentions of making this package portable between
;; XEmacs and GNU/Emacs.  Harsh words?  No, not really, I simply don't
;; have the time or resources to invest in the extra work involved.
;; And also there is no way that I'd be able to support GNU/Emacs users
;; because I'm not familiar with that flavour of Emacs.  If you want to
;; port it, be my guest, this is an Open Source project.
;;
;; Another note: This package uses correct English spelling wherever
;; possible.  For example, "cheque" instead of "check", "summarise"
;; instead of "summarize".  Although I welcome and encourage _ALL_
;; contributions, patches that s/que/ch/ or s/ise/ize/ will go straight
;; to /dev/null.

;;; Todo:
;;
;;     o Manage multiple accounts.
;;     o Transfers between accounts.
;;     o Recurring transactions.
;;     o Bank reconciliation.
;;     o Simple reporting (P/L, balance sheet etc).
;;     o Currency conversion.
;;     o Stock quotes.
;;     o Invoicing.
;;     o Import/Export to and from Gnucash format (maybe).
;;     o Import/Export to and from Quicken qif format (maybe).

;;; ChangeLog:
;;
;;  This is just a place holder so `emoney-commentary' will work
;;  properly.  See the ChangeLog file for changes.

;;; Code:

;; Drag in what we need.
(eval-and-compile
  (autoload 'clear-rectangle "rect" nil t)
  (autoload 'completing-read "minibuf")
  (autoload 'customize-group "cus-edit")
  (autoload 'lm-commentary "lisp-mnt")
  (autoload 'sort-numeric-fields "sort" nil t)
  (autoload 'untabify "tabify" nil t)
  (autoload 'with-electric-help "ehelp"))

;; Custom.
(defgroup emoney nil
  "Customisations for `emoney-mode'."
  :prefix "emoney-"
  :group 'tools)

(defcustom emoney-credit-transaction-types
  '("autotellcr" "atmcr" "bankcredit" "bcr" "deposit" "dep" "directcr"
    "dcr" "internetcr" "netcr" "phonecr" "phcr" "transfercr" "xfrcr")
  "*List of valid credit transaction types.

The default types are:

          autotellcr -- Automatic Teller Machine deposit
               atmcr -- short version of 'autotellcr'

          bankcredit -- Bank initiated credits like interest etc
                 bcr -- short version of 'bankcredit'

             deposit -- Manual, non-direct deposits
                 dep -- short version of 'deposit'

            directcr -- Direct credit transactions
                 dcr -- short version of 'directcr'

          internetcr -- Internet credit transactions
               netcr -- short version of 'internetcr'

             phonecr -- Telephone credit transactions
                phcr -- short version of 'phonecr'

          transfercr -- Transfer credit transactions
               xfrcr -- short version of 'transfercr'"
  :type '(repeat string)
  :group 'emoney)

(defcustom emoney-debit-transaction-types
  '("autotelldb" "atmdb" "bankfee" "fee" "directdb" "ddb" "eftpos"
    "eft" "internetdb" "netdb" "phonedb" "phdb" "withdrawal" "wdl"
    "transferdb" "xfrdb")
  "*List of valid debit transaction types.

The default types are:

          autotelldb -- Automatic Teller Machine transaction
               atmdb -- short version of 'autoteller'

             bankfee -- Bank fees
                 fee -- short version of 'bankfee'

            directdb -- Direct debit transactions
                 ddb -- short version of 'directdb'

              eftpos -- Electronic Funds Transfer Point Of Sale
                 eft -- short version of 'eftpos'

          internetdb -- Internet transactions
               netdb -- short version of 'internet'

             phonedb -- Telephone transactions
                phcr -- short version of 'phone'

          withdrawal -- Over the counter withdrawal.
                 wdl -- short version of 'withdrawal'

          transferdb -- Transfer
               xfrdb -- short version of 'transfer'"
  :type '(repeat string)
  :group 'emoney)

(defcustom emoney-date-format "%Y-%m-%d"
  "*The format that `emoney-mode' uses for dates."
  :type '(choice
	  (const :tag "yyyy-mm-dd"
		 :value "%Y-%m-%d")
	  (const :tag "yyyy/mm/dd"
		 :value "%Y/%m/%d")
	  (const :tag "mm/dd/yy"
		 :value "%m/%d/%y")
	  (const :tag "mm/dd/yyyy"
		 :value "%m/%d/%Y")
	  (const :tag "mm-dd-yy"
		 :value "%m-%d-%y")
	  (const :tag "mm-dd-yyyy"
		 :value "%m-%d-%Y")
	  (const :tag "dd/mm/yy"
		 :value "%d/%m/%y")
	  (const :tag "dd/mm/yyyy"
		 :value "%d/%m/%Y")
	  (const :tag "dd-mm-yy"
		 :value "%d-%m-%y")
	  (const :tag "dd-mm-yyyy"
		 :value "%d-%m-%Y")
	  (const :tag "yy/mm/dd"
		 :value "%y/%m/%d")
	  (const :tag "yy-mm-dd"
		 :value "%y-%m-%d"))
  :group 'emoney)

(defcustom emoney-uk-cheque-spelling t
  "*When non-nil, use UK spelling: \"chq\" instead of \"chk\"."
  :type 'boolean
  :group 'emoney)

(defcustom emoney-save-after-recalculate t
  "*If non-nil, save the buffer after a recalculate.

See `emoney-recalculate-buffer'."
  :type 'boolean
  :group 'emoney)

;;; Internal variables
(defconst emoney-version "0.01"
  "The current version of eMoney.")

(defconst emoney-codename "Accountant"
  "The codename of the current version of eMoney.")

(defvar emoney-is-beta t
  "Non-nil if the current version of eMoney is beta.")

(defconst emoney-in-demo-mode nil
  "Non-nil when eMoney is in demo mode.")

;;;###autoload
(defun emoney-version (&optional arg)
  "*Display the current version information for eMoney.

Optional Argument ARG (prefix arg), print version information at point
in the current buffer."
  (interactive "P")
  (if emoney-is-beta
      (if arg
	  (insert (format "eMoney \"%s\", v%s [Beta]"
			  emoney-codename emoney-version))
	(message (format "eMoney \"%s\", v%s [Beta]"
			 emoney-codename emoney-version)))
    (if arg
	(insert (format "eMoney \"%s\", v%s"
			emoney-codename emoney-version))
      (message (format "eMoney \"%s\", v%s"
		       emoney-codename emoney-version)))))

;;;###autoload
(defun emoney-commentary ()
  "*Display the commentary section of emoney.el."
  (interactive)
  ;; Yeah, yeah, I know there is `finder-commentary', but this is
  ;; better.
  (with-electric-help
   '(lambda ()
      (insert
       (with-temp-buffer
	 (erase-buffer)
	 (insert (lm-commentary (locate-library "emoney.el")))
	 (goto-char (point-min))
	 (while (re-search-forward "^;+ ?" nil t)
	   (replace-match "" nil nil))
	 (buffer-string (current-buffer)))))
   "*eMoney Commentary*"))

;;;###autoload
(defun emoney-copyright ()
  "*Display the copyright notice for eMoney."
  (interactive)
  (with-electric-help
   '(lambda ()
      (insert
       (with-temp-buffer
	 (erase-buffer)
	 (insert-file-contents (locate-library "emoney.el"))
	 (goto-char (point-min))
	 (re-search-forward ";;; Commentary" nil t)
	 (beginning-of-line)
	 (narrow-to-region (point-min) (point))
	 (while (re-search-backward "^;+ ?" nil t)
	   (replace-match "" nil nil))
	 (buffer-string (current-buffer)))))
   "*eMoney Copyright Notice*"))

;;;###autoload
(defun emoney-customise ()
  "*Convenience function to customise eMoney."
  (interactive)
  (customize-group 'emoney))

;; Because lots of people in the world can't spell.
;;;###autoload
(defalias 'emoney-customize 'emoney-customise)

(defvar emoney-transaction-types '("init" "")
  "A list of valid transaction types.

It is a combination of `emoney-credit-transaction-types',
`emoney-debit-transaction-types', and a blank type.")

(defconst emoney-cheque-type
  (if emoney-uk-cheque-spelling
      "chq[ \t]+\\([0-9]+\\)"
    "chk[ \t]+\\([0-9]+\\)")
  "Type field for cheque transactions.

This is defined separately from the other transaction types because
it is used by functions that perform special operations on check
transactions.")

(defconst emoney-date-column 0
  "Column where transaction date begins.")

(defconst emoney-type-column 11
  "Column where transaction type begins.")

(defconst emoney-clear-column 22
  "Column where status appears.")

(defconst emoney-description-column 24
  "Column where transaction description begins.")

(defconst emoney-sign-column 52
  "Column where transaction sign begins.")

(defconst emoney-amount-column 54
  "Column where transaction amount begins.")

(defconst emoney-current-balance-column 65
  "Column where current emoney begins.")

(defconst emoney-tab-stop-list
  (list
   emoney-date-column
   emoney-type-column
   emoney-clear-column
   emoney-description-column
   emoney-sign-column
   emoney-amount-column
   emoney-current-balance-column)
  "List of tab stops that define the start of all transaction fields.")

(defvar emoney-mode-map
  (let ((map (make-sparse-keymap 'emoney-mode-map)))
    (define-key map "\C-c\C-b" 'emoney-backward-field)
    (define-key map "\C-c\C-c" 'emoney-recalculate-buffer)
    (define-key map "\C-c\C-d" 'emoney-clear-current-field)
    (define-key map "\C-c\C-f" 'emoney-forward-field)
    (define-key map "\C-c\C-n" 'emoney-append-next-cheque)
    (define-key map "\C-c\C-r" 'emoney-summarise-cheques-region)
    (define-key map "\C-c\C-s" 'emoney-summarise-cheques-buffer)
    (define-key map "\C-c\C-t" 'emoney-append-transaction)
    (define-key map [tab]      'emoney-forward-field)
    map)
  "Keymap for emoney buffer.")

(defconst emoney-mode-menu
  '("eMoney"
    ["New Cheque Transaction" emoney-append-next-cheque t]
    ["New Transaction" emoney-append-transaction t]
    "---"
    ["Next Field" emoney-forward-field t]
    ["Previous Field" emoney-backward-field t]
    ["Clear Current Field" emoney-clear-current-field t]
    "---"
    ["Summary of cheques (buffer)" emoney-summarise-cheques-buffer t]
    ["Summary of cheques (region)" emoney-summarise-cheques-region t]
    "---"
    ["Recalculate Buffer" emoney-recalculate-buffer t])
  "Menu for `emoney-mode' buffers.")

(easy-menu-define
 emoney-mode-easymenu nil "eMoney" emoney-mode-menu)

(defconst emoney-largest-balance "999999999.99"
  "This is only used for formatting purposes.")

(defconst emoney-header
  "Date       Type       C Description                         Amount      Balance
===============================================================================\n"
  "The header inserted at the top of a `emoney-mode' buffer.")

(defun emoney-insert-header-maybe ()
  "Insert `emoney-header' at the top of the buffer if it doesn't exist."
  (save-excursion
    (goto-char (point-min))
    (unless (re-search-forward emoney-header nil t)
      (goto-char (point-min))
      (insert emoney-header))))

;;;###autoload
(defun emoney-mode ()
  "Major mode for editing a buffer containing financial transactions.
The following bindings provide the main functionality of this mode:

\\{emoney-mode-map}

Transactions occur on a single line and have the following fields (in
order):

 date          The transaction date.  See `emoney-date-format'.
 type          This field must either be blank or match one of the
               expressions defined in `emoney-credit-transaction-types'
               and `emoney-debit-transaction-types'.
 clear         Status of transaction, 'o' is open, 'x' is cleared.  New
               transactions default to 'o'.
 description   A possibly blank transaction description.
 sign          This field must either be '+', '-' or '='.  '+' means
               credit, '-' means debit, and '=' resets balance.  eMoney
               will usually guess the correct sign to use.
 amount        The transaction amount.
 balance       The balance after this transaction.  This field will be
               computed upon recalculation, you _don't_ need to fill it
               in.  Just do: \\[emoney-recalculate-buffer].

Any line in the buffer that does not begin with a date will be
considered a comment and ignored.  Among other things, this allows
the transaction description to span several lines.

Changing any amount and recalculating again will update all visible
balances.  Transactions may be commented out by putting a semi-colon
\(or any other non-numerical character\) at the beginning of the line.

Entering `emoney-mode' runs the `emoney-mode-hooks' if any exist.

You can get a feel for how eMoney works via `emoney-demo'."
  (interactive)
  (setq major-mode 'emoney-mode)
  (setq mode-name "eMoney")
  (use-local-map emoney-mode-map)
  (easy-menu-add emoney-mode-easymenu)
  (emoney-insert-header-maybe)
  (make-local-variable 'tab-stop-list)
  (setq tab-stop-list emoney-tab-stop-list)
  (make-local-variable 'indent-tabs-mode)
  (setq indent-tabs-mode nil)
  (setq indent-line-function 'emoney-forward-field)
  (overwrite-mode 1)
  (run-hooks 'emoney-mode-hooks))

(defconst emoney-demonstration-data
  "2003-05-25 init       o Opening Balance             = 1256.89
2003-05-25 bankfee    o Account keeping fee         - 5

Any line that doesn't begin with a number is ignored so you can intersperse
your transaction lists with comments, notes etc.  Blank lines are also
ignored.

2003-05-25 chq 000001 o Acme Automotive             - 375.60
2003-05-25 eft        o Groceries                   - 275.90
2003-05-25 directcr   o Salary                      + 625.45
2003-05-25 chq 000002 o Dodgey  Brothers'  Computer - 48.95
                        Hardware    and    Software
                        Ripoff Merchants Pty Ltd
2003-05-25 eftpos     o Cigarettes                  - 20"
  "The data used in `emoney-demo'.")

(defconst emoney-demonstration-help
  "		 Welcome to the eMoney demonstration
		 ===================================

eMoney is based on `balance.el', so if you are familiar with that,
you'll find eMoney a breeze.

Play around with this, it is a fully functional demo.  The only
difference between this demonstration and using eMoney in the real
world is that your changes here are not automatically saved.  The next
time you run \\[emoney-demo] you'll have the same initial data to play
with.

The Key bindings:
----------------
\\{emoney-mode-map}

One of the first things that you might like to do is to \"recalculate
the buffer\", use: \\[emoney-recalculate-buffer]


Special Note to US citizens:
---------------------------

A \"cheque\" is a small piece of paper that represents a monetary amount
that the person who wrote the cheque agrees to give to the \"payee\"
\(person stated on the cheque as the one to receive payment\).

A \"check\" is something usually done with some form of measuring device.
The UNIX command `du' can be used to check on how much hard drive space
is available.  A check can also be the word used to describe the mark
used to denote an item on a list that has been checked or is correct.
"
  "The help text used in `emoney-demo'.")

;;;###autoload
(defun emoney-demo ()
  "A demonstration of `emoney-mode'."
  (interactive)
  (let* ((demo-buffer (get-buffer-create "*eMoney Demo*"))
	 (demo-data emoney-demonstration-data)
	 (demo-help emoney-demonstration-help))
    (switch-to-buffer demo-buffer)
    (insert demo-data)
    (emoney-mode)
    (make-variable-buffer-local 'emoney-in-demo-mode)
    (setq emoney-in-demo-mode t)
    (with-electric-help
     (lambda ()
       (insert
	 (with-temp-buffer
	   (emoney-mode)
	   (erase-buffer)
	   (insert (substitute-command-keys demo-help))
	   (buffer-string (current-buffer))))))))

(defun emoney-current-line ()
  "Return the current buffer line at point."
  (save-excursion
    (beginning-of-line)
    (count-lines (point-min) (point))))

(defun emoney-completing-read (prompt table &optional predicate require-match
			       initial-contents history default)
  "Like `completing-read', but also accepts strings.

Arguments PROMPT, TABLE, PREDICATE, REQUIRE-MATCH, INITIAL-CONTENTS,
HISTORY, DEFAULT are as per `completing-read'."
  (completing-read
   prompt
   (if (vectorp table)
       table
     (mapcar 'list table))
   predicate require-match initial-contents history default))

(defun emoney-forward-field ()
  "Move the cursor to the next field on the current line."
  (interactive)
  (move-to-tab-stop))

(defun emoney-last (list)
  "Return last element in LIST."
  (cond
   ((null list)
    '())
   ((null (cdr list))
    (car list))
   (t (emoney-last (cdr list)))))

(defun emoney-find-largest-less-than (list item)
  "Search a sorted LIST of numbers, return the largest number that is < ITEM."
  (let ((list-car (car list))
	(list-cdr (cdr list))
	(last nil))
    (while (and list-car (< list-car item))
      (setq last list-car)
      (setq list-car (car list-cdr))
      (setq list-cdr (cdr list-cdr)))
    last))

(defun emoney-find-largest-less-than-equal (list item)
  "Return cdr of LIST starting @ the largest number that is <= to ITEM."
  (let ((list-car (car list))
	(list-cdr (cdr list))
	(last nil))
    (while (and list-car (<= list-car item))
      (setq last (cons list-car list-cdr))
      (setq list-car (car list-cdr))
      (setq list-cdr (cdr list-cdr)))
    last))

(defun emoney-find-field (column)
  "Return a list of the start and end of the field around COLUMN.

End may be nil if column is after the last defined tab stop."
  (let ((field (emoney-find-largest-less-than-equal
		emoney-tab-stop-list column)))
    (if (equal 1 (length field))
	(list (car field) nil)
      (list (nth 0 field) (nth 1 field)))))

(defun emoney-backward-field ()
  "Move the cursor to the previous entry field on the current line."
  (interactive)
  (let* ((col (current-column))
	 (prev (emoney-find-largest-less-than emoney-tab-stop-list col)))
    (if prev
	(move-to-column prev)
      (move-to-column (emoney-last emoney-tab-stop-list)))))

(defun emoney-clear-current-field ()
  "Fill the field around point with spaces, leave point at start of field."
  (interactive)
  (let* ((field (emoney-find-field (current-column)))
         (line-start (progn (beginning-of-line) (point)))
         (line-end (progn
		     (end-of-line)
		     (untabify line-start (point))
		     (point)))
         (field-start (+ line-start (nth 0 field)))
         (field-end (if (nth 1 field)
                        (+ line-start (nth 1 field))
		      line-end)))
    (clear-rectangle field-start field-end)
    (goto-char field-start)))

(defsubst emoney-build-types-list ()
  "Dynamically build a list of transaction types.

This is done dynamically so that the user can change or add to the
list of transaction types without having to reload eMoney."
  ;; Initialise to ("init" "").
  (setq emoney-transaction-types '("init" ""))
  ;; Load the debit and credit transaction types.
  (setq emoney-transaction-types
	(append emoney-transaction-types
		emoney-debit-transaction-types
		emoney-credit-transaction-types)))

(defun emoney-append-transaction ()
  "Add a transaction to the end of current buffer using today's date."
  (interactive)
  (goto-char (point-max))
  (if (not (equal 0 (current-column)))
      (newline))
  (insert (format-time-string emoney-date-format))
  (move-to-tab-stop)
  (emoney-build-types-list)
  (let* ((type (emoney-completing-read "Transaction type: "
				       emoney-transaction-types)))
    (insert type)
    (move-to-tab-stop)
    (insert "o")
    (move-to-tab-stop)
    (let ((before-descript (point))
	  (start-column (current-column))
	  (fill-column (- emoney-sign-column
			  emoney-description-column
			  1)))
      (save-excursion
	(insert (read-string "Description: "))
	(save-restriction
	  (narrow-to-region before-descript (point))
	  (goto-char before-descript)
	  (while (progn (move-to-column fill-column) (not (eobp)))
	    (search-forward " " nil t)
	    (insert "\n"))
	  (fill-paragraph 1)
	  (goto-char before-descript)
	  (forward-line 1)
	  (while (progn (beginning-of-line) (not (eobp)))
	    (indent-to-column start-column)
	    (forward-line 1)))))
    (move-to-tab-stop)
    (cond ((member type emoney-credit-transaction-types)
	   (insert "+"))
	  ((member type emoney-debit-transaction-types)
	   (insert "-"))
	  ((string= type "init")
	   (insert "="))
	  (t (if (y-or-n-p "Is this a credit transaction? ")
		 (insert "+")
	       (if (y-or-n-p "Is this a debit transaction? ")
		   (insert "-")
		 (if (y-or-n-p "Is this an initialising transaction? ")
		     (insert "=")
		   (warn "Couldn't determine +/-/=, leaving blank"))))))
    (move-to-tab-stop)
    (insert (read-string "Amount: "))))

(defun emoney-append-next-cheque ()
  "Add a cheque transaction to the end of the current buffer using today's date.

Inserts the cheque number following the last cheque number written into the
transaction type column.  Loses if you write cheques out of order."
  (interactive)
  (goto-char (point-max))
  (if (not (equal 0 (current-column)))
      (newline))
  (insert (format-time-string emoney-date-format))
  (move-to-tab-stop)
  (let (cheque cheque-number noinit)
    (save-excursion
      (if (search-backward-regexp emoney-cheque-type 0 t)
	  (progn
	    (setq cheque (buffer-substring (match-beginning 1) (match-end 1)))
	    (setq noinit nil))
	(move-to-column emoney-type-column)
	(if (< (current-column) emoney-type-column)
	    (indent-to-column emoney-type-column))
	(if emoney-uk-cheque-spelling
	    (insert "chq 000001")
	  (insert "chk 000001"))
	(setq noinit t)))
    (unless noinit
      (setq cheque-number (1+ (string-to-number cheque)))
      (move-to-column emoney-type-column)
      (if (< (current-column) emoney-type-column)
	  (indent-to-column emoney-type-column))
      (if emoney-uk-cheque-spelling
	  (insert (format "chq %06d" cheque-number))
	(insert (format "chk %06d" cheque-number))))
    (move-to-tab-stop)
    (insert "o")
    (move-to-tab-stop)
    (let ((before-descript (point))
	  (start-column (current-column))
	  (fill-column (- emoney-sign-column
			  emoney-description-column
			  1)))
      (save-excursion
	(insert (read-string "Payable To: "))
	(save-restriction
	  (narrow-to-region before-descript (point))
	  (goto-char before-descript)
	  (while (progn (move-to-column fill-column) (not (eobp)))
	    (search-forward " " nil t)
	    (insert "\n"))
	  (fill-paragraph 1)
	  (goto-char before-descript)
	  (forward-line 1)
	  (while (progn (beginning-of-line) (not (eobp)))
	    (indent-to-column start-column)
	    (forward-line 1)))))
    (move-to-tab-stop)
    (insert "-")
    (move-to-tab-stop)
    (insert (read-string "Amount: "))))

(defsubst emoney-build-type-regexp ()
  "Return a regular expression that will match any valid transaction type.

This is done dynamically so users can redefine the valid transactions in
their `user-init-file' even after this file has been loaded."
  (emoney-build-types-list)
  (let ((types (push emoney-cheque-type emoney-transaction-types)))
    (concat
     "^\\("
     (mapconcat
      (lambda(x) x) types "\\|")
     "\\)[ \t]*$")))

(defun emoney-check-transaction-type (line-start)
  "Check to make sure a valid transaction type has been used.

Argument LINE-START is the starting point."
  (let* ((type-regexp (emoney-build-type-regexp))
         (type-start (+ line-start emoney-type-column))
         (type-end (+ line-start emoney-clear-column))
         (type-string (buffer-substring type-start type-end)))
    (if (string-match type-regexp type-string)
	nil
      (error "Line %d, invalid type: %s"
	     (1+ (emoney-current-line)) type-string))))

(defun emoney-find-next-transaction ()
  "Find the next line that is a complete transaction.

Return a list of the line start, numeric data start and line end
points."
  (let ((found nil)
	(line-regexp
	 "^\\([0-90-9]\\)+.*$")
	line-start
	line-end
	data-start)
    (while (and
	    (not found)
	    (search-forward-regexp line-regexp (point-max) t))
      (setq line-start (match-beginning 0))
      (untabify (match-beginning 0) (match-end 0))
      (setq line-end (progn (end-of-line) (point)))
      (setq data-start (+ line-start emoney-sign-column))
      (if (> line-end data-start)
	  (setq found t)))
    (if found
	(list line-start data-start line-end)
      nil)))

(defun emoney-parse-transaction-data (data)
  "Return a list of floating point numbers from DATA.

DATA is a string representing the sign, amount and optionally balance of a
transaction.  Balance is nil if not present."
  (let ((data-regexp "\\([-+=p]\\)[ \t]*\\([0-9.]+\\)?[ \t]*\\([-]?[0-9.]+\\)?")
	(balance nil)
	(reset nil)
	(printit nil)
	string sign amount)
    (string-match data-regexp data)
    (if (match-beginning 1)
	(setq sign (substring data (match-beginning 1) (match-end 1)))
      (error "Line %d, missing sign" (1+ (emoney-current-line))))
    (if (equal sign "p")
	(progn
	  (setq printit t)
	  (setq sign "-")))
    (if (equal "=" sign)
	(progn
	  (setq sign "+")
	  (setq reset t)))
    (if (match-beginning 2)
	(progn
	  (setq string (substring data (match-beginning 2) (match-end 2)))
	  (setq amount (string-to-number (concat sign string))))
      (error "Line %d, missing amount" (1+ (emoney-current-line))))
    (if (match-beginning 3)
	(progn
	  (setq string (substring data (match-beginning 3) (match-end 3)))
	  (setq balance (string-to-number string))))
    (if reset (setq sign "="))
    (if printit (setq sign "p"))
    (list sign amount balance)))

(defun emoney-same (amount1 amount2)
  "Compare two dollar amounts, AMOUNT1 AMOUNT2, for equivalence."
  (let ((string1 (format "%10.2f" amount1))
	(string2 (format "%10.2f" amount2)))
    (equal string1 string2)))

(defun emoney-form-transaction-data (sign amount balance)
  "Given SIGN, AMOUNT and a BALANCE, return a string.

The string is suitable for placing in the numeric region of a
transaction, based on the defined input columns."
  (let* ((amount (abs amount))
         (width1 (- emoney-amount-column emoney-sign-column))
         (width2 (- emoney-current-balance-column emoney-amount-column))
         (len (length emoney-largest-balance))
         (gap (- width2 len))
         (value (concat "%" (number-to-string len) ".2f"))
         (format-string (concat "%-" (number-to-string width1)
				"s" value
				"%" (number-to-string gap) "s" value)))
    (format format-string sign amount "" balance)))

(defun emoney-number-it (the_num)
  "Return a string giving words for a number from 1 to 99, recursive!

Argument THE_NUM is the number to convert from digits to words."
  (let* ((num-string ""))
    (cond
     ((>= the_num 90)
      (setq num-string "Ninety ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 80)
      (setq num-string "Eighty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 70)
      (setq num-string "Seventy ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 60)
      (setq num-string "Sixty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 50)
      (setq num-string "Fifty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 40)
      (setq num-string "Forty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 30)
      (setq num-string "Thirty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((>= the_num 20)
      (setq num-string "Twenty ")
      (setq num-string (concat num-string (emoney-number-it (% the_num 10)))))
     ((= 19 the_num)
      (setq num-string "Nineteen "))
     ((= 18 the_num)
      (setq num-string "Eighteen "))
     ((= 17 the_num)
      (setq num-string "Seventeen "))
     ((= 16 the_num)
      (setq num-string "Sixteen "))
     ((= 15 the_num)
      (setq num-string "Fifteen "))
     ((= 14 the_num)
      (setq num-string "Fourteen "))
     ((= 13 the_num)
      (setq num-string "Thirteen "))
     ((= 12 the_num)
      (setq num-string "Twelve "))
     ((= 11 the_num)
      (setq num-string "Eleven "))
     ((= 10 the_num)
      (setq num-string "Ten "))
     ((= 9 the_num)
      (setq num-string "Nine "))
     ((= 8 the_num)
      (setq num-string "Eight "))
     ((= 7 the_num)
      (setq num-string "Seven "))
     ((= 6 the_num)
      (setq num-string "Six "))
     ((= 5 the_num)
      (setq num-string "Five "))
     ((= 4 the_num)
      (setq num-string "Four "))
     ((= 3 the_num)
      (setq num-string "Three "))
     ((= 2 the_num)
      (setq num-string "Two "))
     ((= 1 the_num)
      (setq num-string "One ")))
    num-string))

;;; FIXME: Don't limit this to $99,999.99
(defun emoney-print-make-words (bucks)
  "Return a string giving the amount of BUCKS in words.

Suitable for cheque writing.  Limited to under a hundred grand!  Code
inspired by original Inform library code of Graham Nelson."
  (let* ((words "")
	 (dollars (truncate bucks))
	 ; avoid floating point errors
	 (cents (- (* 100 bucks) (* 100 dollars))))
    (if (>= dollars 100000)
	(error "Checks over 99999.99 not supported!"))
    (if (>= dollars 1000)
	(progn
	  (setq words (concat words (emoney-number-it (truncate (/ dollars 1000)))
			      "Thousand "))
	  (setq dollars (% dollars 1000))))
    (if (>= dollars 100)
	(progn
	  (setq words (concat words (emoney-number-it (truncate (/ dollars 100)))
			      "Hundred "))
	  (setq dollars (% dollars 100))))
    (if (>= dollars 0)
	(progn
	  (setq words (concat words (emoney-number-it dollars)))))
    (if (> cents 0)		 ; still a rounding problem with cents
	(setq words (concat words "and " (format "%02d" cents) "/100"))
      (setq words (concat words "and 00/100")))
    words))

;;; FIXME: This is badly crippled it MUST be customisable!  Currently,
;; it's not.  It is a remnant of the old balance.el.
(defun emoney-print-check (data-string amount check-count)
  "Output this check for printing.

Argument DATA-STRING the transaction line of the cheque to print.
Argument AMOUNT the dollar amount of the cheque to print.
Argument CHECK-COUNT the number of cheques to print."
  (let* ((print-buffer (get-buffer-create "*Print Checks*"))
	 (print-date (substring data-string 0 emoney-type-column))
	 (print-payto (substring data-string emoney-description-column
				 emoney-sign-column))
	 (print-memo "")
	 (note-pos nil)
	 (decimal-amount (format "%.2f" (abs amount)))
	 (print-decimal-amount (emoney-print-make-words (abs amount))))
    (if (setq note-pos (string-match "|" print-payto))
	(progn
	  (setq print-memo (substring print-payto (1+ note-pos)))
	  (setq print-payto (substring print-payto 0 note-pos))))
    (save-excursion
      (set-buffer print-buffer)
      (end-of-buffer)
      (if (= check-count 1)
	  (progn
	    (insert "\\documentclass[12pt]{article}\n")
	    (insert "% ********************************************\n")
	    (insert "% This is hardwired for Bob Newell's printer and check forms\n")
	    (insert "% There are just too many variables to easily parameterize\n")
	    (insert "% TO PRINT YOUR CHECKS:\n")
	    (insert "% --Save this buffer with a tex extension \(i.e. ctrl-x ctrl-s foo.tex\)\n")
	    (insert "% --Run LaTeX \(ctrl-c ctrl-c if you have AucTeX set up, otherwise use a shell\)\n")
	    (insert "% --Run dvips or dvilj in a shell, or whatever you normally do to\n")
	    (insert "%   make a print file from LaTeX output\n")
	    (insert "% --Print on your check forms as usual.\n")
	    (insert "% THIS IS SET UP FOR THREE-TO-PAGE CHECKS\n")
	    (insert "% It will BOMB OUT if you print non-multiples of 3!\n")
	    (insert "% On different printers you only need mess with the picture offset\n")
	    (insert "% For different check forms it's a whole new game\n")
	    (insert "% It takes about an hour to measure and fine tune everything\n")
	    (insert "% ********************************************\n")
	    (insert "\\oddsidemargin -1in\n")
	    (insert "\\topmargin 0in\n")
	    (insert "\\textwidth 7.5in\n")
	    (insert "\\textheight = 55\\baselineskip\n")
	    (insert "\\begin{document}\n")
	    (insert "\\unitlength=1in\n")))
      (if (= (% check-count 3) 1)
	  (progn
	    (insert "\\begin{picture}\(8.5,10.5\)\(0.37,-1.15\)\n")
	    (insert "%date\n")
	    (insert "  \\put(7.0,10.0\){\\makebox\(0,0\)[l]{")
	    (insert print-date)
	    (insert "}}\n")
	    (insert "%payee\n")
	    (insert "  \\put\(1.2,9.45\){\\makebox\(0,0\)[l]{")
	    (insert print-payto)
	    (insert "}}\n")
	    (insert "%amount in words\n")
	    (insert "  \\put\(0.5,9.15\){\\makebox\(0,0\)[l]{")
	    (insert print-decimal-amount)
	    (insert "}}\n")
	    (insert "%amount in decimal\n")
	    (insert "  \\put\(7.0,9.5\){\\makebox\(0,0\)[l]{")
	    (insert decimal-amount)
	    (insert "}}\n")
	    (insert "%memo\n")
	    (insert "  \\put\(0.7,8.1\){\\makebox\(0,0\)[l]{")
	    (insert print-memo)
	    (insert "}}\n")))
      (if (= (% check-count 3) 2)
	  (progn
	    (insert "%date\n")
	    (insert "  \\put\(7.0,6.5\){\\makebox\(0,0\)[l]{")
	    (insert print-date)
	    (insert "}}\n")
	    (insert "%payee\n")
	    (insert "  \\put\(1.2,5.95\){\\makebox\(0,0\)[l]{")
	    (insert print-payto)
	    (insert "}}\n")
	    (insert "%amount in words\n")
	    (insert "  \\put\(0.5,5.65\){\\makebox\(0,0\)[l]{")
	    (insert print-decimal-amount)
	    (insert "}}\n")
	    (insert "%amount in decimal\n")
	    (insert "  \\put\(7.0,6.0\){\\makebox\(0,0\)[l]{")
	    (insert decimal-amount)
	    (insert "}}\n")
	    (insert "%memo\n")
	    (insert "  \\put\(0.7,4.6\){\\makebox\(0,0\)[l]{")
	    (insert print-memo)
	    (insert "}}\n")))
      (if (= (% check-count 3) 0)
	  (progn
	    (insert "%date\n")
	    (insert "  \\put\(7.0,3.0\){\\makebox\(0,0\)[l]{")
	    (insert print-date)
	    (insert "}}\n")
	    (insert "%payee\n")
	    (insert "  \\put\(1.2,2.45\){\\makebox\(0,0\)[l]{")
	    (insert print-payto)
	    (insert "}}\n")
	    (insert "%amount in words\n")
	    (insert "  \\put\(0.5,2.15\){\\makebox\(0,0\)[l]{")
	    (insert print-decimal-amount)
	    (insert "}}\n")
	    (insert "%amount in decimal\n")
	    (insert "  \\put\(7.0,2.5\){\\makebox\(0,0\)[l]{")
	    (insert decimal-amount)
	    (insert "}}\n")
	    (insert "%memo\n")
	    (insert "  \\put\(0.7,1.1\){\\makebox\(0,0\)[l]{")
	    (insert print-memo)
	    (insert "}}\n")
	    (insert "\\end{picture}\n")
	    (insert "\\newpage\n"))))))

(defun emoney-recalculate (start end)
  "Recalculate the balances for region START END.

The final balance, uncleared total, and the number of balances that
changed, and the transaction count are returned in a list."
  (let ((current-balance 0)
	(changes 0)
	(uncleared 0)
	(transactions 0)
	line-points)
    (save-excursion
      (save-restriction
	(narrow-to-region start end)
	(goto-char (point-min))
	(while (setq line-points (emoney-find-next-transaction))
	  (setq transactions (1+ transactions))
	  (let* ((line-start (nth 0 line-points))
		 (data-start (nth 1 line-points))
		 (data-end (nth 2 line-points))
		 (clear-flag (buffer-substring
			      (+ line-start emoney-clear-column)
			      (+ 1 line-start emoney-clear-column)))
		 (data-string (buffer-substring data-start data-end))
		 (data-values (emoney-parse-transaction-data data-string))
		 (sign (nth 0 data-values))
		 (amount (nth 1 data-values))
		 (balance (nth 2 data-values))
		 (new-balance (if (equal sign "=")
				  amount
				(+ current-balance amount)))
		 (new-uncleared (if (equal clear-flag "x")
				    uncleared
				  (+ uncleared amount)))
		 (new-string
		  (emoney-form-transaction-data sign amount new-balance)))
	    (emoney-check-transaction-type line-start)
	    (setq current-balance new-balance)
	    (setq uncleared new-uncleared)
	    (if (or (null balance) (not (emoney-same balance new-balance)))
		(setq changes (1+ changes)))
	    (if (not (equal data-string new-string))
		(progn
		  (delete-region data-start data-end)
		  (goto-char data-start)
		  (insert new-string)))))
	(widen)))
    (list current-balance uncleared changes transactions)))

(defun emoney-recalculate-buffer ()
  "Recalculate the current buffer.

See `emoney-recalculate'."
  (interactive)
  (let* ((result (emoney-recalculate (point-min) (point-max)))
         (balance (nth 0 result))
	 (uncleared (nth 1 result))
         (changes (nth 2 result))
         (total (nth 3 result)))
    (when (and emoney-save-after-recalculate
	       (not emoney-in-demo-mode))
      (save-buffer (current-buffer)))
    (message
     (format "book bal %.2f unclrd %.2f bank bal %.2f (%d/%d recalcs)"
	     balance uncleared (- balance uncleared) changes total))
    (if (> changes 0)
	(end-of-line))))

(defun emoney-recalculate-region (start end)
  "Recalculate the current region, START END.

See `emoney-recalculate'."
  (interactive "r")
  (let* ((result (emoney-recalculate start end))
         (balance (nth 0 result))
	 (uncleared (nth 1 result))
         (changes (nth 2 result))
         (total (nth 3 result)))
    (message (format "Region balance %.2f uncleared %.2f (%d/%d recalcs)"
		     balance uncleared changes total))
    (if (> changes 0)
	(end-of-line))))

(defun emoney-summarise-cheques (start end)
  "Create a buffer that lists only the cheques in the specified region.

The region is denoted by START END.

The list is sorted on cheque number.  Breaks in sequence are denoted by lines
containing an asterisk between the cheques where the break occurs.  The buffer
is also recalculated, thus showing to total of the cheques summarised."
  (let ((emoney-buffer (current-buffer))
	(summary-buffer (get-buffer-create "*cheque summary*"))
	(cheque-count 0)
	(sequence-breaks 0)
	line-points)
    (save-excursion
      (save-restriction
	(set-buffer summary-buffer)
	(delete-region (point-min) (point-max))
	(set-buffer emoney-buffer)
	(narrow-to-region start end)
	(goto-char (point-min))
	(while (setq line-points (emoney-find-next-transaction))
	  (let* ((line-start (nth 0 line-points))
		 (line-end (nth 2 line-points))
		 (type-start (+ line-start emoney-type-column))
		 (type-end (+ line-start emoney-description-column))
		 (type-string (buffer-substring type-start type-end)))
	    (if (string-match emoney-cheque-type type-string)
		(progn
		  (append-to-buffer
		   summary-buffer line-start (1+ line-end))
		  (setq cheque-count (1+ cheque-count))))))
	(widen)
	(set-buffer summary-buffer)
	(sort-numeric-fields 3 (point-min) (point-max))
	(setq sequence-breaks (emoney-find-sequence-breaks))
	(goto-char (point-max))
	(insert (format "\n%d cheque%s summarised, %d sequence break%s\n"
			cheque-count
			(if (equal 1 cheque-count) "" "s")
			sequence-breaks
			(if (equal 1 sequence-breaks) "" "s")))
	(set-buffer emoney-buffer)))
    (pop-to-buffer summary-buffer)
    (goto-char (point-max))))

(defun emoney-find-sequence-breaks ()
  "Find cheque sequence breaks in the current cheque summary buffer.

Mark breaks in sequence by inserting a line with an asterisk between
the offending cheques.  Return the count of sequence breaks found."
  (let ((last-cheque nil)
	(sequence-breaks 0))
    (goto-char (point-min))
    (while (search-forward-regexp
	    (concat "\\([0-9/]+\\)[ \t]+" emoney-cheque-type)
	    (point-max) t)
      (let* ((cheque-start (match-beginning 2))
	     (cheque-end (match-end 2))
	     (cheque-string (buffer-substring cheque-start cheque-end))
	     (cheque-number (string-to-int cheque-string)))
	(if (not last-cheque)
            (setq last-cheque cheque-number)
	  (if (not (equal cheque-number (1+ last-cheque)))
	      (progn
		(setq sequence-breaks (1+ sequence-breaks))
		(beginning-of-line)
		(open-line 1)
		(insert "*")
		(next-line 2)))
	  (setq last-cheque cheque-number))))
    sequence-breaks))

(defun emoney-summarise-cheques-buffer ()
  "Summarise the cheque transactions in the current buffer."
  (interactive)
  (emoney-summarise-cheques (point-min) (point-max))
  (emoney-recalculate (point-min) (point-max)))

(defun emoney-summarise-cheques-region (start end)
  "Summarise the cheque transactions in region START - END."
  (interactive "r")
  (emoney-summarise-cheques start end)
  (emoney-recalculate (point-min) (point-max)))

;;;###autoload(add-to-list 'auto-mode-alist '("\\.emy$" . emoney-mode))

(provide 'emoney)

;;; emoney.el ends here

;Local Variables:
;time-stamp-start: "Last-Modified:[ 	]+\\\\?[\"<]+"
;time-stamp-end: "\\\\?[\">]"
;time-stamp-line-limit: 10
;time-stamp-format: "%4y-%02m-%02d %02H:%02M:%02S (%u)"
;End:
