;;; etail.el --- Tail files and manage them visually

;; $Id: etail.el,v 1.13 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; The etail packages allows you to watch certain log files for new output.
;; Internally it uses 'tail --follow' which uses the select(2) system call to
;; pay attention to when a file has been modified.  This means that Emacs
;; doesn't have to do any polling to see when the file has been changed.
;;
;; There are other packages that allow you to watch logfiles (analog.el and
;; tail.el) but they have certain disadvantages.  Analog.el puts all its output
;; into one buffer and doesn't automatically refresh itself (it can use an
;; interval if you want).  Tail.el pops up a window while you are editing and
;; can get in the way.  There are also some significant bugs which I belive will
;; never be fixed due to timing issues (I tried to fix them and new ones would
;; continually pop up due to the way Emacs is setup).
;;
;; The etail package tries to solve all these problems by using the native
;; 'tail' command to watch for new output as soon as it arives.  It also
;; provides a GUI display of all the tail buffers in one central location so
;; that you can view all your log activity in one place.
;;
;; In order to avoid taking up a lot memory over time, a timer is run in the
;; background to trim the head of each buffer so that only etail-max-length is
;; displayed.  This has the advantage that if a process generates a lot of
;; output you can still view it.

;;; Install:

;; Add a (require 'etail) to your .emacs file.  You should might also add a call
;; to (etail-file "/some/file/name") so that when emacs is started up again you
;; can have the same set of tailed files.

;; TODO:
;;
;; - Don't support a recursive edit.  this isn't complicated enough to do a
;;   recursive edit.  IE we don't need it.
;;
;; - Ability to snart a WHOLE directory of log files.
;;
;;   - Display this as a tree under the index.  Something like:
;;
;;   Buffer                                             Filename                                                        NumLines           LastModified
;;   ------                                             --------                                                        --------           ------------
;;   tail.log                                           /tmp/tail.log                                                   10                 5 minutes ago
;;   /projects/reptile/bin/distribution/reptile/logs:   
;;       turbine.log                                    /projects/reptile/bin/distribution/reptile/log/turbine.log
;; 
;; - Ability to monitor remote log files?  Possibly through tramp?  This would
;;   require some sort of lisp implementation of tail.  I don't know if this is
;;   possible without running a command on the remote machine.
;;

;; - The ability to view ETail in two main buffers.  A tail *index* and a main
;;   tail buffer.  The index buffer is used to change the buffer used to display
;;   the etail..  This way we can have a LARGE number of etail buffers without
;;   crowding the display.
;;
;;    - change the face to bold on the index buffer when text is inserted into 
;;      it.
;;
;;    - ability to 'clear' the tailed buffer from the index buffer.

;; The etail index buffer should probably look something like

;; Buffer                       Filename                      
;; ------                       ---------------------------------------------------
;; messages                     /var/log/messages

;;    - ability to easily truncate the log file down to X lines.
;; 

;; 
;; - is it possible to keep the buffer from scrolling too far when it is
;;   reverted?  Try to use a process filter to insert lines into the etail
;;   buffer so that I can fit it correctly each time.
;;
;; - add the ability to run in a major mode other than shell-mode.  This way I
;;   can enable special major modes (syslog-mode) with special font-locks.  It
;;   may depend on the above item and using process filters.
;;
;; - I should rework etail-file so that it can work on the fly...
;; (pop-to-buffer).  The way you config etail should not depend on a function.

;;; History:
;; 

;; - Wed Dec 05 2001 01:35 AM (burton@openprivacy.org): only allow etail to be
;;   launched once.  Don't continue to go into recursive edits.
;;
;; - Wed Dec 05 2001 12:29 AM (burton@openprivacy.org): should not launch "tail"
;; until the user actually *USES* 'etail.  There is no sense launching etail on
;; (require) because we end up wasting CPU and resources.


;;; Code:

(defcustom etail-max-length 100
  "The maximum number of lines the tail buffer should contain after cleanup."
  :group 'etail)

(defcustom etail-clean-delay 30
  "Number of seconds delay before cleaning etail buffers."
  :group 'etail)

(defvar etail-window-configuration nil
  "Window configuration to restore after etail is displayed.")

(defvar etail-start-message "etail visual mode enabled, escape to quit.")

(defvar etail-tailed-files '() "List of files that etail should monitor.")

(defvar etail-activated nil "True if etail is activated.")

(defun etail()
  "Show all etail buffers on the screeen at once."
  (interactive)

  (if etail-activated
      (message "etail visual mode already activated")
    (catch 'exit
      (progn
        (setq etail-window-configuration (current-window-configuration))
        
        (etail-setup-windows)

        (message etail-start-message)
        
        (setq etail-activated t)
        
        (recursive-edit))
      (etail-quit))))

(defun etail-fit(buffer window size)
  "Fit the buffer within its window so that it its point is at the bottom of the
window and there is exactly 1 line between it and "

  ;;update the point so that it is at the end of its buffer
  (set-buffer buffer)

  (goto-char (point-max))

  (setq size (- size 2))
  
  ;;update the start of the window.
  (let(window-start)
    (save-excursion

      ;;only set the window start if the buffer is larger than the window
      (if (> (count-lines (point-min)
                          (point-max))
             size)
          (progn
      
            (previous-line size)

            (goto-char (point-at-bol))
      
            (setq window-start (point)))))

    (if window-start
        (set-window-start window window-start))

    (etail-trim-tail buffer)

    (set-buffer buffer)
    (goto-char (point-max))))

(defun etail-quit()
  "Quit the etail visual layout and restore the previous window configuration."

  (setq etail-activated nil)
  
  (set-window-configuration etail-window-configuration))
  
(defun etail-setup-windows()
  "Setup all etail windows so that they are visible within the current frame."

  (let(etail-buffers)

    (if (equal (length etail-tailed-files) 0)
        (error "There are no etail buffers"))
  
    ;;go ahead and setup the windows.
    (delete-other-windows)

    (let((i 0)
         current-window-height
         new-window-height
         filename
         buffer-name
         buffer)

      (setq current-window-height (window-height))

      (setq new-window-height (/ current-window-height (length etail-tailed-files)))
    
      (while (< i (length etail-tailed-files))

        ;;get the next buffer.
        (setq filename (nth i etail-tailed-files))
        
        (setq buffer-name (etail-file-get-buffer-name filename))

        (if (not (get-buffer buffer-name))
            (etail-file-create-buffer-and-process filename))

        (setq buffer (get-buffer buffer-name))
        
        ;;move the current etail buffer into the current window
        (set-window-buffer (selected-window) buffer-name)

        ;;update this window so that the cursor is right at the end...
        (etail-fit buffer (selected-window) new-window-height)

        ;;now figure out if we should split the window.  This should only happen
        ;;if this isn't the LAST itteration of this while loop.

        (if (< i (1- (length etail-tailed-files)))
            (progn
        
              ;;split the window
              (split-window-vertically new-window-height)
        
              ;;move onto the next window.
              (other-window 1)))

        (setq i (1+ i)))))

  ;;now select the first window in this frame.

  (other-window 1))

(defun etail-sort-predicate(buff-one buff-two)
  "Used with sort as a predicate."

  (string-lessp (buffer-name buff-one)
                (buffer-name buff-two)))
  
(defun etail-clean()
  "Periodic task to make sure to trim the tail buffers."

  ;;go through each etail buffer trying to clean it up...

  (let((bl (buffer-list))
       (i 0)
       buffer)

    (setq bl (buffer-list))

    ;;go through each

    (while (< i (length bl))

      ;;get the next buffer.
      (setq buffer (nth i bl))

      (if (string-match "^\\*etail" (buffer-name buffer))
          (etail-clean-buffer buffer))
      
      (setq i (1+ i)))))

(defun etail-clean-buffer(buffer)
  "Remove extra lines at the beginning of the given buffer."

  (save-excursion

    (set-buffer buffer)

    (let(line-count delete-line-count)

      (setq line-count (count-lines (point-min) (point-max)))

      ;;if the current line-count is greater than the maximum we should ever
      ;;have...

      (if (> line-count etail-max-length)
          (progn

            ;;determine the number of lines we should delete
            (setq delete-line-count (- line-count etail-max-length))

            ;;now remove the lines

            (goto-char (point-min))

            (let(start end)
              (setq start (point))

              (forward-line delete-line-count)

              (setq end (point))

              (delete-region start end)))))))

(defun etail-trim-tail(buffer)
  "Trim any leading whitespace at the end of this buffer"

  (save-excursion

    (set-buffer buffer)

    (goto-char (point-max))

    (if (re-search-backward "[^ \n]" nil t)
        (let(start end)
            (setq start (+ 1 (point)))

            (setq end (point-max))

            (if (< start end)
                (progn
                  (delete-region start end)
                  (goto-char (point-at-eol))
                  (insert "\n")))))))

(defun etail-file-get-buffer-name(filename)
  "Given a filename, get the etail buffer it should use."

  (let(buffer)

    (setq buffer (format "*etail: %s*" filename))

    buffer))

(defun etail-file-create-buffer-and-process(filename)
  "Create the buffer and associated 'tail' process for the given filename."

  (let(buffer-name)

    (save-window-excursion
      
      (setq buffer-name (etail-file-get-buffer-name filename))
      
      (if (not (get-buffer buffer-name))
          (progn
            (set-buffer (get-buffer-create buffer-name))
            
            (shell-command (concat "tail --follow --lines="
                                   (number-to-string etail-max-length)
                                   " "
                                   filename
                                   " &")
                           (current-buffer)))))

    ;;update some misc settings in this buffer:
    (set-buffer buffer-name)

    (make-local-variable 'scroll-step)
    (setq scroll-step 1)

    ;;turn on hscroll mode because this allows for long lines and doesn't hurt
    ;;our wrapping
    (hscroll-mode 1)
    
    (if (interactive-p)
        (display-buffer buffer-name))))
  
(defun etail-file(file)
  "Tail the given file."
  (interactive
   (list (read-file-name "File: ")))

  (setq file (expand-file-name file))
  
  (if (not (file-exists-p file))
      (error "File does not exist: %s" file))

  (if (not (file-readable-p file))
      (error "File is not readable: %s" file))

  (if (file-directory-p file)
      (error "File is actually a directory: %s" file))

  (add-to-list 'etail-tailed-files file))

;;run etail-clean every once in a while to make sure our buffers are short
(run-with-timer etail-clean-delay etail-clean-delay 'etail-clean)

(provide 'etail)

;;; etail.el ends here
