;;; fileset.el --- Manages files across Emacs instances.

;; $Id: fileset.el,v 1.20 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;;; History:

;; - Before I list files.  Make sure that they exist.  If the DONT exist, remove
;;   them from fileset-list.

;;; TODO:
;;
;; - What do we do about Tramp file URLs?  We can't 'ls' these.  What data
;; should be put...
;;
;;
;; - Ability to expire files..
;;
;;    - use fileset-list to list the files.
;;
;;    - prompt to expire (close) the files.
;;
;; - Need the ability to set the MAXIMUM number of files, it turns out that
;; about 50 files is the maximum number that we can realistically reload without
;; having to take too much time on Emacs startup.  We need to have an LRU
;; algorithm so that we keep pushing down files.  When we open up a new file,
;; push out a file from the stack.   

;;; Code:

(defvar fileset-list '() "List of files to restore.")

(defvar fileset-exclusion-modes '() "List of major modes which should be
excluded.  This is used if you need to exclude certain files based on their
major mode.")

(defvar fileset-file "~/.fileset.el" "File used to back fileset data.")

(defvar fileset-list-buffer-name "*fileset-list*" "Buffer name used to display
the managed files.")

(defvar fileset-restore-files-prompt t
  "Set to true if you should prompt before restoring files.")

(defvar fileset-use-ls t "Use the 'ls' command to list files.  Set this to nil
if you are on a platform which does not support ls.")

(defvar fileset-max-expire-time (list 9 14976) "The maximum amount of time (in days)
that a file is allowed to be managed by fileset.  The default (9 14976) is one
week.")

(defun fileset--save(&optional no-message)
  "Save the word list."
  
  (if (not no-message)
      (message "Saving fileset..."))

  (save-excursion
  
    (let(buffer)

      (setq buffer (get-buffer-create "*fileset*"))
      
      (set-buffer buffer)

      (erase-buffer)

      (insert "(setq fileset-list '")
      (prin1 fileset-list (current-buffer))
      (insert ")")

      ;;save this to disk
      (write-region (point-min) (point-max) fileset-file)

      (kill-buffer buffer))

    (if (not no-message)
        (message "Saving fileset...done"))))
  
(defun fileset--restore()
  "Restore fileset data from disk."

  (if (and (null fileset-list)
           (file-exists-p fileset-file))
      (load-file fileset-file)))

(defun fileset--find-file-hook()
  "Hook used to keep track of new files."

  (let(filename)

    (setq filename (buffer-file-name))

    (if filename
        (progn

          (setq filename (expand-file-name filename))
    
          (if (and (not (equal filename fileset-file))
                   (not (member filename fileset-list)))

              (progn

                (add-to-list 'fileset-list filename)

                (fileset--save t)))))))

(defun fileset--kill-buffer-hook()
  "Hook used to kill buffers."

  (let(filename)

    (setq filename (buffer-file-name))
    
    (if filename
        (progn
          
          (setq filename (expand-file-name filename))
        
          (if (and (not (equal filename fileset-file))
                   (member filename fileset-list))
              (progn
              
                (setq fileset-list (delete filename fileset-list))
              
                (fileset--save t)))))))

(defun fileset-restore-files--should-restore()
  "Return nil if we should not restore files."

  (if fileset-restore-files-prompt
      (progn
        
        (fileset-list)

        (let(should-restore)
        
          (setq should-restore (yes-or-no-p "Should 'fileset' restore the above file(s)? "))

          (other-window 1)

          (delete-other-windows)
          
          should-restore))
          
    t))

(defun fileset-restore-files()
  "Restore all files available.  This method is somewhat smart (although this
doesn't help on startup) so that we don't load files multiple times."
  (interactive)

  (if (fileset-restore-files--should-restore)
      (progn
        (message "Restoring %i file(s)..." (length fileset-list))
  
        (let((i 0)
             current-file)

          (while (< i (length fileset-list))

            (setq current-file (elt fileset-list i))

            (if (and (not (get-file-buffer current-file))
                     (file-exists-p current-file))
                (progn

                  (message "Restoring file: %s..." current-file)
           
                  (find-file-noselect current-file)

                  (message "Restoring file: %s...done" current-file)))
     
            (setq i (1+ i))))

        (message "Restoring %i file(s)...done" (length fileset-list)))))

(defun fileset-ls-files()
  "List all files (via ls)."

  ;;fileset-list-buffer-name

  (let(ls-args)

    (setq ls-args fileset-list)

    ;;use the -t option so that we can see this sorted by the last modified time
    ;;of each file.  This is especially nice when we are coming back into Emacs
    ;;after a crash because we get to see the list of files we were editing
    ;;right before anything bad happened.
    
    (add-to-list 'ls-args "-alt") 

    (apply 'call-process "ls" nil fileset-list-buffer-name nil ls-args)))

(defun fileset-list(&optional use-list)
  "List the files that fileset is managing.  Optionally developers may specify
the variable `use-list' which is a list of files."
  (interactive)

  (if (null use-list)
      (setq use-list fileset-list))
  
  (fileset--cleanup-list)
  
  (save-excursion
    (let((i 0)
         current-file)

      (set-buffer (get-buffer-create fileset-list-buffer-name))

      (erase-buffer)

      (insert "//FILESET:  ---------------------------------------------------------------------\n")
      (insert (format "//FILESET:  The following %i files are known.\n" (length use-list)))
      (insert "//FILESET:  ---------------------------------------------------------------------\n")

      (if fileset-use-ls
          (fileset-ls-files)
        (while (< i (length use-list))
        
          (setq current-file (elt use-list i))
          
          (insert current-file)
          (insert "\n")
          
          (setq i (1+ i))))
      
      (fileset-list-mode)))

  (pop-to-buffer fileset-list-buffer-name))

(defun fileset--cleanup-list()
  "Cleanup any files in the list that do not exist."

  (let((i 0)
       (current-file nil))
      
    (while (< i (length fileset-list))

      (setq current-file (elt fileset-list i))

      (if (not (file-exists-p current-file))
          (progn
            
            (setq fileset-list (delete current-file fileset-list))))
      
      (setq i (1+ i)))))

(defun fileset-expire()
  "Cleanup any files in the list that do not exist."
  (interactive)

  (fileset--cleanup-list)
  
  (let((i 0)
       (current-file nil)
       (current-time (current-time))
       (file-mod-time nil)
       (file-mod-difference-days nil)
       (file-mod-difference-high nil)
       (file-mod-difference-low nil)
       (file-expire-list '()))
    (save-excursion
      
      (while (< i (length fileset-list))

        (setq current-file (elt fileset-list i))

        ;;get the file mode time.
        (setq file-mod-time (elt (file-attributes current-file) 5))

        ;;determine the high and low differences.
        
        (setq file-mod-difference-high (- (elt current-time 0) (elt fileset-max-expire-time 0)))
        (setq file-mod-difference-low (- (elt current-time 1) (elt fileset-max-expire-time 1)))

        ;;print the oldest date that a file could be... 

        
;;         (message "FIXME (debug): ---------------------------")
;;         (message "FIXME (debug): current-file: %s" current-file)
;;         (message "FIXME (debug): current-time: %s" current-time)
;;         (message "FIXME (debug): file-mod-time: %s" file-mod-time)
;;         (message "FIXME (debug): file-mod-time(as string): %s" (current-time-string file-mod-time))
;;         (message "FIXME (debug): file-mod-difference-high: %s" file-mod-difference-high)
;;         (message "FIXME (debug): file-mod-difference-low: %s" file-mod-difference-low)
;;         (message "FIXME (debug): file-mod-difference-days: %s" file-mod-difference-days)

        ;;determine if this file is old

        (if (or (and (= (elt file-mod-time 0) file-mod-difference-high)
                     (< (elt file-mod-time 1) file-mod-difference-low))
                (< (elt file-mod-time 0) file-mod-difference-high))
            (progn
              
              (add-to-list 'file-expire-list current-file)
              
              (message "FIXME (debug): file is too old: %s" current-file)

              ))
        
        ;;now see if the file is too old
        
        (setq i (1+ i))))
    
    ;;only display if actually found files.

    (if (> (length file-expire-list) 0)
        (progn 
    
          (fileset-list file-expire-list)

          ;;TODO: if no... delete/kill the window.
          
          (if (yes-or-no-p (format "Should the above files (older than %s) be expired? "
                                   (current-time-string (list file-mod-difference-high file-mod-difference-low))))
              
              (fileset--expire-files file-expire-list)))
      (message "No files old enough to expire."))))
    
(defun fileset--expire-files(expire-file-list)
  "Expire files in the given list."
  
  (let((i 0)
       (buffer nil)
       (current-file nil))
      
    (while (< i (length expire-file-list))

      (setq current-file (elt expire-file-list i))

      ;;remove the file from the fileset-list
      (setq fileset-list (delete current-file fileset-list))

      ;;now to to kill the buffer (if it is open)
      
      (setq buffer (get-file-buffer current-file))

      (kill-buffer buffer)
      
      (setq i (1+ i))))

  (fileset--save))

(define-derived-mode fileset-list-mode fundamental-mode "Fileset"
  "Mode for viewing fileset lists.."

  (goto-char (point-min))

  (font-lock-mode 1))

;;final misc stuff

(remove-hook 'find-file-hooks 'fileset--find-file-hook)
(remove-hook 'kill-buffer-hook 'fileset--kill-buffer-hook)

;;so that we can restore files on startup...
(add-hook 'emacs-startup-hook 'fileset-restore-files)

(font-lock-add-keywords 'fileset-list-mode '(("^//.*$"
                                              (0 'font-lock-comment-face t))))

(fileset--restore)

(provide 'fileset)

;;; fileset.el ends here
