;;; FILESTATS.EL --- Show file statistics

;; Copyright (C) 2001 Thomas Link

;; Author: Thomas Link <thomas.link@a1.net>
;; Maintainer: Thomas Link <thomas.link@a1.net>
;; URL: http://members.a1.net/t.link/software/filestats.html
;; Created: 12 Feb 2001
;; Version: 1.1
;; Keywords: file statistics, word counting
 
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; A copy of the GNU General Public License can be obtained from this
;; program's author (send electronic mail to <t.link@gmx.at>) or from
;; the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
;; 02139, USA.

;;; Commentary:

;; Display some file statistics.

;; Put this file into your load path -- type "\c-h v load-path" to
;; find a suitable directory. Put

;; (autoload 'filestats "filestats" "Display file statistics" t)
;; (autoload 'filestats-concordance "filestats" "Build concordance table" t)

;; into your startup file, which is probably called "~/.emacs.el".
;; Invoke by typing "M-x filestats".

;; The cursor will remain in your main window; the filestats window
;; will disappear after a few seconds. In case this feature makes
;; troubles, set filestats-auto-hide-secs to 0.

;; Add your own definitions to the list 'filestats-list-optional'. Put
;; these definitions into your startup file. A valid entry is a list
;; with four values: (buffer-mode title accumulate-p fun). See the
;; documentation by typing "\C-h v filestats-list". Examples:

;; (add-to-list 'filestats-list-optional
;; 		   '(("Scheme" "Lisp" (lambda (buffer) (set-buffer buffer)
;; 						    (equal comment-start ";")))
;; 			"functions" 
;; 			t
;; 			(lambda () (filestats-search-forward "\n(define"))))
;; (add-to-list 'filestats-list-optional '(t "Vowels" t "[aeiouAEIOU]"))

;; You can modify the way how results are displayed. E.g.
;; (add-to-list 'filestats-view-functions-alist
;; 		   '("words" . (lambda (buffer unit result)
;; 					  (filestats-insert buffer unit "=" result "\n"))))

;;; Credits:

;; This package is based upon by work done by:
;; - Colin Walters ... "ibuffer.el"
;; - Chris Beggy <chrisb@kippona.com>, sshteingold@cctrading.com
;;   and ccurley@trib.com ... "sds-word-count.el"
;; - Bob Glickstein ... "wcount.el"

;;; Change log:

;; 1.1.
;; - New command: filestats-concordance
;; - New variables: filestats-kill-on-manual-quit-p,
;; filestats-preserve-if-current-buffer-p, filestats-expert-p
;; filestats-kill-on-auto-quit-p, filestats-preserve-stat-list-p
;; - Change: viewer's arguments are: buffer unit result (not as a string)
;; - Change: don't delete the filestats buffer, if it's the current buffer
;; (controlled by filestats-preserve-if-current-buffer-p)
;; - Change: an entry for filestats-list-optional now may have the form
;; (mode title accumulate-p regexp), e.g. '(t "a and b" t "[ab]")
;; - Bug fix: don't select deleted buffer

;; 1.0 
;; Initial version

;;; To do:

;; - filestats-pre-count-hook ... initialize values if necessary
;; - filestats-post-count-hook ... compute sums if necessary
;; - modify accumulate-p dependencies so that we can pass a function
;; for accumulating results

;;; Code:

(require 'thingatpt)
(require 'timer)

(defvar filestats-target-buffer nil)
(defvar filestats-target-buffer-file-name nil)
(defvar filestats-stat-list nil)
(defvar filestats-buffer-timer nil)
(defvar filestats-concordance-table nil)
(setplist 'filestats-buffer-timer-list nil)

(defvar filestats-view-functions-alist nil
  "Additional viewers. A valid entry is a pair of a unit 
qualifier and function, which takes 3 arguments: 
(unit . (lambda (buffer unit fun) ...)

buffer ... The buffer where the result should be displayed

unit ... Title of what we display

fun ... A function that takes two arguments. The units name and the
result.")

(defvar filestats-list-sds
  '(("" "words" nil filestats-sds-count-words)
    ("" "character" nil filestats-sds-count-chars)
    ("" "lines" nil filestats-sds-count-lines))
  "See documentation for filestats-list.")

(defvar filestats-list
  '(("" "words" t (lambda () (if (forward-word 1) 1 0)))
    ("" "sentences" t (lambda () (forward-sentence 1) 1))
    ("" "paragraphs" t (lambda () (forward-paragraph 1) 1))
    ("" "whitespace" t (lambda () (if (forward-whitespace 1) 1 0))))
  "A list of lists of type (mode title accumulate-p fun). Add 
your own definitions to filestats-list-optional.

mode ... Determines if a specific statistics should be produced for
this buffer. Possible values are: t (accept), the name of a mode, or a
function taking the buffer as an argument and returning t (accept) or
nil (don't accept). You may also use a list of names and functions.

title ... A String telling what to count

accumulate-p ... Is fun a function which has to be called several
times to get the result?

fun ... This is a regular expression or a function, which takes no
arguments. If accumulate-p is nil, this function should simply return
the result. If accumulate-p is t, the function should return the
number added to the result. In most cases this will be 1. If the
return value is nil or the buffer limits are exceeded, the counting
process terminates.")

(defvar filestats-list-optional
  '((("TeX" "Fundamental") "sections" t
	(lambda ()
	  (filestats-search-forward-regexp
	   "\\\\\\(chapter\\|subsubsection\\|subsection\\|section\\)")))
    ("Emacs-Lisp" "functions" t
	(lambda () (filestats-search-forward "\n(defun"))))
  "These definitions will be appended to filestats-list.
See documentation for filestats-list.")

(defvar filestats-mode-map
  (let ((map (make-sparse-keymap)))
    (define-key map (kbd "q") 'filestats-quit)
    (define-key map (kbd "u") 'filestats-update)
    (define-key map (kbd "h") 'describe-mode)
    (define-key map (kbd "SPC") 'filestats-switch-other-window)
    map))

(defgroup filestats nil
  "Show File Statistics."
  :prefix "filestats-"
  :group 'files)

(defcustom filestats-hooks nil
  "Hooks run when `filestats' is called."
  :type 'hook
  :group 'filestats)

(defcustom filestats-mode-hooks nil
  "Hooks run upon entry into `filestats-mode'."
  :type 'hook
  :group 'filestats)

(defcustom filestats-auto-hide-secs 4
  "Remove filestats windows after N seconds. This doesn't work alway
as expected. Set this value to 0 to switch this feature off."
  :type 'integer
  :group 'filestats)

(defcustom filestats-resize-window-p t
  "Shrink filestats window as necessary."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-switch-window-p t
  "Switch to original window."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-preserve-stat-list-p t
  "Preserve list of statistics between calls to 'filestats'."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-kill-on-manual-quit-p t
  "Don't just bury the buffer, but remove it."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-preserve-if-current-buffer-p t
  "Don't delete filestats buffer, if it's the current buffer."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-kill-on-auto-quit-p nil
  "On auto-quit, don't just bury the buffer, but remove it."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-sds-fast-count-p nil
  "Uses a different and possibly faster method for counting
 words, lines, and characters."
  :type 'boolean
  :group 'filestats)

(defcustom filestats-expert-p nil
  "No messages, no warnings."
  :type 'boolean
  :group 'filestats)


;;; these are adapted from Colin Walters' "ibuffer.el"
(defun filestats (&optional stat-list)
  "Begin using `filestats' to edit a list of buffers.
Type 'h' after entering filestats for more information."
  (interactive)
  (let* ((target-buffer (current-buffer))
	    (target-buffer-file-name (buffer-name target-buffer))
	    (target-buffer-name (concat "*Filestats: " target-buffer-file-name "*"))
	    (fs-buffer nil))
    (pop-to-buffer (get-buffer-create target-buffer-name))
    (setq fs-buffer (current-buffer))
    (unless (eq major-mode 'filestats-mode)
	 (filestats-mode))
;     (set (make-local-variable 'filestats-target-buffer)
; 		target-buffer)
;     (set (make-local-variable 'filestats-target-buffer-file-name)
; 		target-buffer-file-name)
;     (set (make-local-variable 'filestats-stat-list) stat-list)
    (setq filestats-target-buffer target-buffer)
    (setq filestats-target-buffer-file-name target-buffer-file-name)
    (if (or (not filestats-preserve-stat-list-p) stat-list)
		   (setq filestats-stat-list stat-list))
    (filestats-update fs-buffer)
    (run-hooks 'filestats-hooks)
    (set-buffer fs-buffer)
    (filestats-message "Commands: u to update, q to quit; h for help")
    (if filestats-switch-window-p
	   (filestats-switch-other-window))))

(defun filestats-switch-other-window ()
  "Switch from filestats buffer to original window."
  (interactive)
  (switch-to-buffer-other-window filestats-target-buffer))

(defun filestats-mode ()
  "Show file statistics.
Press 'u' to update, 'q' to quit.
SPC ... switch to other window."
;   (interactive)
  (kill-all-local-variables)
  (use-local-map filestats-mode-map)
  (setq major-mode 'filestats-mode)
  (setq mode-name "Filestats")
  (setq buffer-read-only nil)
  (make-local-variable 'filestats-target-buffer)
  (make-local-variable 'filestats-target-buffer-file-name)
  (make-local-variable 'filestats-stat-list)
  (make-local-variable 'filestats-buffer-timer)
  (buffer-disable-undo)
  (setq truncate-lines t)
  (run-hooks 'filestats-mode-hooks))

(defun filestats-auto-quit (buffer)
  (if (and filestats-preserve-if-current-buffer-p
		 (eq buffer (current-buffer)))
	 (filestats-reset-buffer-timer buffer)
    (filestats-do-the-quit filestats-kill-on-auto-quit-p buffer)))

(defun filestats-quit ()
  "Quit this `filestats' session."
  (interactive)
  (filestats-do-the-quit filestats-kill-on-manual-quit-p (current-buffer)))

(defun filestats-do-the-quit (kill-buffer-p buffer)
  (if (equal mode-name "Filestats")
	 (progn
	   (filestats-remove-buffer-timer buffer)
	   (if kill-buffer-p
		  (kill-buffer buffer)
		(bury-buffer buffer))
	   (if (> (count-windows) 1)
		  (delete-window)))
    (if (> (count-windows) 1)
	 (delete-other-windows))))


;;; These are adapted from a file I received from Chris Beggy
;;; <chrisb@kippona.com>. The original source says: credit to
;;; sshteingold@cctrading.com and ccurley@trib.com
(defun filestats-sds-count-chars (&optional buffer) 
  (let* ((result "")
	    (cb (current-buffer))
	    (this-buffer (or buffer cb)))
    (set-buffer this-buffer)
    (save-excursion
	 (save-restriction
	   (let ((start (point-min))
		    (end (point-max)))
		(setq result (- end start)))))
    (set-buffer cb)
    result))

(defun filestats-sds-count-lines (&optional buffer)
  (let* ((result "")
	    (cb (current-buffer))
	    (this-buffer (or buffer cb)))
    (set-buffer this-buffer)
    (save-excursion
	 (save-restriction
	   (let ((start (point-min))
		    (end (point-max)))
		(setq result
			 (count-lines start end)))))
    (set-buffer cb)
    result))

(defun filestats-sds-count-words (&optional buffer)
  (let* ((result "")
	    (cb (current-buffer))
	    (this-buffer (or buffer cb)))
    (set-buffer this-buffer)
    (save-excursion
	 (save-restriction
	   (let ((start (point-min))
		    (end (point-max)))
		(goto-char start)
		(setq result (string-to-number (how-many "\\<"))))))
    (set-buffer cb)
    result))


;;; A generalised count procedure based upon Bob Glickstein's "wcount.el"
(defun filestats-general-count (mode what fun &optional show-message-p buffer)
  "what ... A String telling what to count
fun ... The function used to find the next occurence of what to count
buffer ... Which buffer to choose"
  (set-buffer (or buffer (current-buffer)))
  (save-excursion
    (save-restriction
      (goto-char (point-min))
      (let* ((result 0)
		   (message-prefix (concat "Counting " what " ... "))
		   (show-message
		    (lambda (&optional txt)
			 (if show-message-p
				(filestats-message (concat message-prefix txt))))))
	   (funcall show-message)
	   (while (not (eobp))
		(let ((r (funcall fun)))
		  (if r
			 (setq result (+ result r))
		    (goto-char (point-max)))))
	   (funcall show-message "done")
	   result))))


;;; Utilities you might want to use
(defun filestats-search-forward (find-string)
  (if (null (search-forward find-string nil t)) 
	 nil 1))

(defun filestats-search-forward-regexp (find-string)
  (if (null (search-forward-regexp find-string nil t))
	 nil 1))

(defun filestats-make-concordance nil
  (if (forward-word 1)
	 (progn
	   (let ((wd 
; 			(let* ((start (point))
; 				  (end (progn 
; 					    (backward-word 1)
; 					    (point)))
; 				  (wdd (buffer-substring start end)))
; 			  (forward-word 1)
; 			  wdd))
  			(thing-at-point 'word))
		    )
		(let ((prct (/ (* 100 (point)) (point-max))))
		  (if (= (mod prct 10) 0)
			 (message (concat "Building concordance ... "
						   (number-to-string prct) "%%"))))
; 		(setq filestats-concordance-table
; 			 (filestats-concordance-inc
; 			  filestats-concordance-table
; 			  wd))
		(filestats-concordance-inc wd)
		1))
    nil))

(defun filestats-concordance-inc (word)
  (let ((hd (assoc word filestats-concordance-table)))
    (if hd
	   (setcdr hd (1+ (cdr hd)))
	 (setq filestats-concordance-table
		  (cons `(,word . 1) filestats-concordance-table)))))

; v02
; (defun filestats-concordance-inc (word-list word)
;   (if (null (assoc word word-list))
; 	 (cons (cons word 1) word-list)
;     (mapcar (lambda (this-pair)
; 		    (let* ((wd (car this-pair))
; 				 (n (cdr this-pair)))
; 			 (if (equal wd word)
; 				(cons word (1+ n))
; 			   this-pair)))
; 		  word-list)))

; v01
; (defun filestats-concordance-inc (word-list word)
;   (cond
;    ((null word-list) (cons (cons word 1) nil))
;    ((listp word-list)
;     (let* ((hd (car word-list))
; 		 (tl (cdr word-list))
; 		 (wd (car hd))
; 		 (n (cdr hd)))
; 	 (if (equal wd word)
; 		(cons (cons word (1+ n)) tl)
; 	   (cons hd (filestats-concordance-inc tl word)))))
;    (t nil)))

(defun filestats-concordance-viewer (buffer result)
  (mapcar (lambda (this-pair)
		  (let* ((wd (car this-pair))
			    (n (cdr this-pair)))
		    (filestats-insert buffer wd ": " n "\n")))
		result))

(defun filestats-concordance-viewer-v01 (buffer result)
  (if (and result (listp result))
	 (let* ((hd (car result))
		   (wd (car hd))
		   (n (cdr hd)))
	   (filestats-insert buffer wd ": " n "\n")
	   (filestats-concordance-viewer buffer (cdr result)))))

(defun filestats-concordance-view (buffer unit result)
  (filestats-insert buffer (filestats-format-result unit "") "\n")
  (let ((start (point)))
    (filestats-concordance-viewer buffer filestats-concordance-table)
    (set-buffer buffer)
    (setq buffer-read-only nil)
    (sort-lines nil start (point-max))
    (goto-char (point-min))))

(defun filestats-concordance nil "Build concordance table"
  (interactive)
  (let ((tmp filestats-list))
    (setq filestats-concordance-table nil)
    (add-to-list
	'filestats-view-functions-alist
	'("concordance" . (lambda (b u r) (filestats-concordance-view b u r))))
    (filestats
	'((t "concordance" t (lambda () (filestats-make-concordance)))))
    (setq filestats-list tmp)))


;;; Core functions
(defun filestats-get-mode (this-entry) (nth 0 this-entry))
(defun filestats-get-entity (this-entry) (nth 1 this-entry))
(defun filestats-get-type (this-entry) (nth 2 this-entry))
(defun filestats-get-fun (this-entry) (nth 3 this-entry))

(defun filestats-get-view (unit) 
  (cdr (assoc unit filestats-view-functions-alist)))

(defun filestats-check-mode-p (mode this-buffer)
  (cond
   ((null mode) nil)
   ((functionp mode) (funcall mode this-buffer))
   ((listp mode)
    (or (filestats-check-mode-p (car mode) this-buffer)
	   (filestats-check-mode-p (cdr mode) this-buffer)))
   ((equal mode t) t)
   ((equal mode "") t)
   ((stringp mode)
    (let ((cb (current-buffer))
		(result nil))
	 (set-buffer this-buffer)
	 (setq result (equal mode mode-name))
	 (set-buffer cb)
	 result))
   (t nil)))

(defun filestats-set-buffer-timer (buffer timer)
  (set-buffer buffer)
  (if (timerp filestats-buffer-timer)
	 (cancel-timer filestats-buffer-timer))
  (setq filestats-buffer-timer timer))

(defun filestats-set-buffer-timer-v01 (buffer timer)
  (let ((old-timer (get 'filestats-buffer-timer-list buffer)))
    (if (timerp old-timer)
	   (cancel-timer old-timer)))
  (put 'filestats-buffer-timer-list buffer timer))

(defun filestats-reset-buffer-timer (buffer)
  (filestats-set-buffer-timer 
   buffer 
   (filestats-create-buffer-timer buffer)))

(defun filestats-remove-buffer-timer (buffer)
  (filestats-set-buffer-timer buffer nil))

(defun filestats-create-buffer-timer (buffer)
  (run-at-time filestats-auto-hide-secs 
			nil
			'filestats-auto-quit
			buffer))


(defun filestats-run-count (fsl this-buffer)
  (if (null fsl)
	 nil
    (let* ((this-entry (car fsl))
		 (mode (filestats-get-mode this-entry)))
	 (if (filestats-check-mode-p mode this-buffer)
		(let* ((what (filestats-get-entity this-entry))
			  (type (filestats-get-type this-entry))
			  (how1 (filestats-get-fun this-entry))
			  (how (if (stringp how1)
					 (lambda ()
					   (filestats-search-forward-regexp how1))
				    how1))
			  (this-result 
			   (progn
				(set-buffer this-buffer)
				(if type
				    (filestats-general-count
					type what how nil this-buffer)
				  (funcall how)))))
		  `((,what . ,this-result)
		    . ,(filestats-run-count (cdr fsl) this-buffer)))
	   (filestats-run-count (cdr fsl) this-buffer)))))

(defun filestats-get-max-chars (this-buffer)
  (let ((result 0)
	   (cb (current-buffer)))
    (set-buffer this-buffer)
    (save-excursion
	 (save-restriction
	   (setq result (- (point-max) (point-min)))))
;    (setq result (buffer-size))
    (set-buffer cb)
    result))

(defun filestats-message (text)
  (unless filestats-expert-p
    (message text)))

(defun filestats-insert-this (text)
  (cond
   ((null text) t)
   ((listp text) 
    (filestats-insert-this (car text))
    (filestats-insert-this (cdr text)))
   ((stringp text)
    (insert text))
   ((numberp text)
    (insert (number-to-string text)))
   (t nil)))

(defun filesets-prepare-fs-buffer (buffer)
    (set-buffer buffer)
    (setq buffer-read-only nil))
 
(defun filestats-insert (buffer &rest text)
  (let ((cb (current-buffer)))
    (filesets-prepare-fs-buffer buffer)
    (filestats-insert-this text)
    (setq buffer-read-only t)
    (set-buffer cb)))

(defun filestats-format-result (what result &optional tail)
  (concat
   (format "%-15s" (concat (capitalize what) ":"))
   (format "%10s" result)
   tail))

(defun filestats-update-get-result (this-buffer)
  (if this-buffer
	 (progn
	   (if filestats-concordance-table
		  (setq filestats-concordance-table nil))
	   (filestats-run-count
	    (if filestats-stat-list
		   filestats-stat-list
		 (append
		  (if filestats-sds-fast-count-p
			 filestats-list-sds
					   filestats-list)
		  filestats-list-optional))
	    this-buffer))
    nil))

(defun filestats-update-init-buffer (fs-buffer this-buffer-name)
  (set-buffer fs-buffer)
  (filestats-remove-buffer-timer fs-buffer)
  (setq buffer-read-only nil)
  (erase-buffer)
  (filestats-insert fs-buffer "File statistics: " this-buffer-name "\n"))

(defun filestats-update-cleanup (fs-buffer)
  (set-buffer fs-buffer)
  (setq buffer-read-only t)
  (goto-char 1)
  (if (> filestats-auto-hide-secs 0)
	 (filestats-reset-buffer-timer fs-buffer))
  (if filestats-resize-window-p
	 (shrink-window-if-larger-than-buffer
	  (get-buffer-window fs-buffer))))

(defun filestats-update-view-characters (fs-buffer buffer result result-string)
  (let* ((characters-total
		(filestats-get-max-chars buffer))
	    (characters-readable (- characters-total result)))
    (set-buffer fs-buffer)
    (filestats-insert 
	fs-buffer
	(filestats-format-result
	 "characters"
	 (number-to-string characters-total)
	 (concat " (" (number-to-string characters-readable)
		    " + " result-string ")"))
	"\n")))

(defun filestats-update (&optional buffer)
  "Update file statistics"
  (interactive)
  (let* ((fs-buffer (or buffer (current-buffer)))
	    (this-buffer (if (buffer-name filestats-target-buffer)
					 filestats-target-buffer
				    nil)))
    (if this-buffer
	   (let* ((this-buffer-name
			 (or filestats-target-buffer-file-name buffer-file-name))
			(result-list (filestats-update-get-result this-buffer)))
		(progn
		  (filestats-update-init-buffer fs-buffer this-buffer-name)
		  (mapcar
		   (lambda (this-pair)
			(let* ((unit (car this-pair))
				  (view (filestats-get-view unit))
				  (result (cdr this-pair))
				  (result-string (number-to-string result)))
			  (if (> result 0)
				 (cond (view
					   (funcall view fs-buffer unit result))
					  ((equal unit "whitespace")
					   (filestats-update-view-characters
					    fs-buffer this-buffer result result-string))
					  (t (filestats-insert 
						 fs-buffer
						 (filestats-format-result unit result-string)
						 "\n"))))))
		   result-list)
		  (filestats-update-cleanup fs-buffer)))
	 (filestats-do-the-quit t fs-buffer))))

(provide 'filestats)

;;; FILESTATS.EL ends here

;;; Local Variables:
;;; auto-recompile:1
;;; End:

