;;; htmlify.el -- Convert a plain-text buffer into clickable html.

;; Copyright (C) 1999 Colin Rafferty

;; Author: Colin Rafferty <colin@xemacs.org>
;; Keywords: html

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; A copy of the GNU General Public License can be obtained from this
;; program's author (send electronic mail to colin@xemacs.org) or from
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;; Send bug reports to colin@xemacs.org

;;; Commentary:

;; Sometimes, people send you an email with html entries, and you
;; would like to be able to click, but you can't.  This will create
;; another buffer that you can load into Netscape, or run
;; `w3-preview-this-buffer'.

;; In order to use this, you will need htmlize.el by Hrvoje Niksic
;; <hniksic@srce.hr>.  Its latest version should be available at:
;;
;;        <URL:http://jagor.srce.hr/~hniksic/htmlize.el>

;; To get the latest version of htmlify, look at:
;;
;; <URL:ftp://ftp.mindspring.com/pub/users/rafferty/emacs/htmlify.el>


;;; Code:

;; I refuse to reinvent the wheel.
(require 'htmlize)

(defconst htmlify-version "1.00")

(defgroup htmlify nil
  "HTMLify buffers to make tags clickable."
  :group 'hypermedia)

;; Regexp taken from gnus-art.el, by Per Abrahamsen <abraham@iesd.auc.dk>.

(defcustom htmlify-url-regexp "\\b\\(s?https?\\|ftp\\|file\\|gopher\\|news\\|telnet\\|wais\\|mailto\\):\\(//[-a-zA-Z0-9_.]+:[0-9]*\\)?\\([-a-zA-Z0-9_=!?#$@~`%&*+|\\/:;.,]\\|\\w\\)+\\([-a-zA-Z0-9_=#$@~`%&*+|\\/]\\|\\w\\)"
  "Regular expression that matches URLs."
  :group 'htmlify
  :type 'regexp)

(defun htmlify-find-html ()
  "Find the next html entry in the current buffer.
Return (START . END).

Return ((point-max) . nil) if none is found."
  (save-excursion
    (save-match-data
      (if (search-forward-regexp htmlify-url-regexp nil t)
	  (cons (match-beginning 0) (match-end 0))
	(cons (point-max) nil)))))

(defun htmlify-convert-to-a (string)
  "Convert the string to a tag."
  (format "<a href=\"%s\">%s</a>" string string))

;;;###autoload
(defun htmlify-buffer (&optional buffer)
  "HTML-ify BUFFER."
  (interactive)
  (or buffer
      (setq buffer (current-buffer)))
  (let ((newbuf (generate-new-buffer "*html*"))
	next-html)
    (switch-to-buffer newbuf)
    (buffer-disable-undo)
    (insert (format "<!-- Created by htmlify-%s. -->\n" htmlify-version))
    (insert "<html>\n  <head>\n    <title>"
	    (htmlize-protect-string (if (stringp buffer) buffer
				      (buffer-name buffer)))
	    "</title>\n  </head>")
    (insert "\n <body>\n    <pre>\n")
    (with-current-buffer buffer
      (save-excursion
	(goto-char (point-min))
	(while (not (eobp))
	  (setq next-html (htmlify-find-html))
	  (princ (htmlize-protect-string
		  (buffer-substring-no-properties (point) (car next-html)))
		 newbuf)
	  (goto-char (car next-html))
	  (when (cdr next-html)
	    (princ (htmlify-convert-to-a (buffer-substring-no-properties
					  (car next-html) (cdr next-html)))
		   newbuf)
	    (goto-char (cdr next-html))))))
    (insert "</pre>\n  </body>\n</html>\n")
    (buffer-enable-undo)))

(provide 'htmlify)

;;; htmlify.el ends here

