;;; identity.el --- Set and manage user identities

;; $Id: identity.el,v 1.3 2002/02/10 19:14:54 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Includes all code necessary to keep track of identities and set them.

;;; History:

;; Sun Jan 27 2002 02:46 PM (burton@openprivacy.org): added identity-show so
;; that you could quickly display all aliases formated.

;;; TODO:
;;
;; - add the ability to setup an alias map from filenames to alias.  This way we
;;   could have
;;
;; '(("/projects/reptile" . "openprivacy")
;;   ("/projects/xml-commons" . "apache"))
;;
;;   This way one could change identities based on files.
;;
;;     - need the ability to set a default identity and the ability to set this
;;       identity when identity.el is loaded.
;;
;; 


;;; Code:

(defvar identity-alist '() "Associated list of identities.")

(defvar identity-show-buffer-name "*identities*" "Buffer name for showing identities.")

(defcustom identity-file-name ".identity.el"
  "File name evalulated when we switch buffers of find new files."
  :group 'identity
  :type 'string)

(defcustom identity-default-alias nil
  "Alias used when we can't find another alias to use."
  :group 'identity
  :type 'string)

(defun identity-create(alias user-full-name user-mail-address)
  "Create a new identity"
  (interactive
   (list
    (read-string "Alias: ")
    (read-string "User full name: ")
    (read-string "User mail address: ")))

  (add-to-list 'identity-alist (list alias
                                     (cons 'user-full-name user-full-name)
                                     (cons 'user-mail-address user-mail-address)))

  (identity-save)

  (message "Created new identity: %s" alias))

(defun identity-rename(old-alias new-alias)
  "Delete an alias from the identity database and save."
  (interactive
    (list
     (identity-prompt-alias)
     (read-string "New alias: ")))

  (let(old-alias-value)

    (setq old-alias-value (assoc old-alias identity-alist))

    (identity-delete old-alias)

    (identity-create new-alias
                     (cdr (assoc 'user-full-name old-alias-value))
                     (cdr (assoc 'user-mail-address old-alias-value))))

  (identity-show)
  
  (message "Identity renamed to '%s'.  Any code that refers to this alias will need to be updated." 
           new-alias))
  
(defun identity-set(alias)
  "Use completion to prompt for an alias and set identity."
  (interactive
    (list
     (identity-prompt-alias)))

  (let(values)

    (setq values (assoc name identity-alist))

    ;;remove the name ..
    (setq values (delete name values))

    (setq user-full-name (cdr (assoc 'user-full-name values)))
    (setq user-mail-address (cdr (assoc 'user-mail-address values))))

  (message "Now using identiity: %s" name))

(defun identity-show()
  "Show all identities in a dedicated buffer."
  (interactive)
  
  (save-excursion

    (set-buffer (get-buffer-create identity-show-buffer-name))

    (erase-buffer)
    
    (dolist(alias identity-alist)

      (insert (format "%s" (car alias)))

      (when (string-equal (car alias) identity-default-alias)
        (insert " (DEFAULT)"))
      
      (insert "\n")
      
      (insert (format "\tuser-full-name: %s\n" (cdr (assoc 'user-full-name alias))))
      (insert (format "\tuser-mail-address: %s\n" (cdr (assoc 'user-mail-address alias))))

      (insert "\n")))

  (shrink-window-if-larger-than-buffer
   (get-buffer-window
    (pop-to-buffer identity-show-buffer-name))))
  
(defun identity-delete(alias)
  "Delete an identity from the list of known aliases and save to disk."
  (interactive
   (list
    (identity-prompt-alias)))

  (setq identity-alist (delete (assoc alias identity-alist) identity-alist))

  (identity-save)

  (message "Deleted identity: %s" alias))
  
(defun identity-prompt-alias()
  "Use completion to prompt for a identity alias."

  (let(name table (i 0))
    
    (while (< i (length identity-alist))

      (setq name (elt (elt identity-alist i) 0))

      (add-to-list 'table (list name i))
      
      (setq i (1+ i)))
    
    (completing-read "Alias: " table)))

(defun identity-update-menu()
  "Update the tools menu in Emacs."

  (easy-menu-add-item 'menu-bar-tools-menu nil (list "Identity"
                                                     ["Create/Edit identity" identity-create t]
                                                     ["Prompt and set identity" identity-set t]
                                                     ["Delete identity" identity-delete t])))

(defun identity-save()
  "Save the identities to disk so that they can be restored later."

  (customize-save-variable 'identity-alist identity-alist))  

(defun identity-set-default(alias)
  "Set the default alias."
  (interactive
   (list
    (identity-prompt-alias)))

  (setq identity-default-alias alias)

  (customize-save-variable 'identity-default-alias identity-default-alias))

(identity-update-menu)

(provide 'identity)

;;; identity.el ends here
