;;; ifind.el --- Find files using GNU find.

;; $Id: ifind.el,v 1.9 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Wrapper around GNU find.  This provides a fast way to lookup files with find.
;; If you are used to using this from the console then it becomes nice to have
;; this integrated into Emacs.
;;

;;; TODO
;;
;; - Show a message when find is done... show the number of files found.
;;
;; - Interface to ifind options:
;;
;;     - follow
;;     
;;
;; - FIXME: right now this package is broken because ifind-buffer-name is not
;; setup in the sentinel...  

;;; Code:
(defvar ifind-hit-count 0 "Number of matches found.")

(defvar ifind-buffer-name "*ifind*" "Buffer name to use for ifind.")

(defun ifind-by-name(directory name)
  (interactive
   (list
    (read-file-name "Find within directory: " nil default-directory)
    (read-string "File name pattern (regexp is acceptable): ")))

  (if (not (file-directory-p directory))
      (error (format "%s is not a directory" directory)))

  (setq ifind-hit-count 0)
  
  (setq directory (expand-file-name directory))
  
  (set-buffer (get-buffer-create ifind-buffer-name))
  (toggle-read-only -1)
  (erase-buffer)

  (goto-char (point-max))
  
  (insert (format "//Searching for %s within %s\n" name directory))

  ;;(goto-char (point-max))
  
  (let(default-directory command process)

    (setq default-directory directory)

    (setq command (concat "find . -name \"" name "\""))

    (message "Running find in directory %s..." directory)

    (save-excursion

      ;;use start-process and set a process filter.
      
      (setq process (start-process "find" nil "find" directory "-name" name "-ls"))
      
      (set-process-filter process 'ifind-process-filter)

      (set-process-sentinel process 'ifind-process-sentinel)))
  
  ;;(ifind-clean-buffer directory)
  
  (display-buffer ifind-buffer-name)

  (set-buffer ifind-buffer-name)
  (ifind-mode))

(defun ifind-process-filter(process string)
  "Filter the input so that we can display it to the user."
  
  (set-buffer ifind-buffer-name)

  (insert string)
  
  (if (string-match "\n" string)
      (save-excursion

        ;;at this point we would have a full line... clean it.
        (let(begin end)

          (forward-line -1)
          
          (goto-char (point-at-bol))
          (setq begin (point))

          (if (re-search-forward "[d-]?[rwx-][rwx-][rwx-][rwx-][rwx-][rwx-][rwx-][rwx-][rwx-]" nil t)
              (delete-region begin (match-beginning 0)))))))
  
(defun ifind-process-sentinel(process event)
  "Waits for the ifind process to finish."
  
  (if (string-equal event "finished\n")
      (let(message)

        (setq message (format "ifind finished.  Found %i matches." ifind-hit-count))
        
        (message message)

        (save-excursion

          (set-buffer ifind-buffer-name)

          (goto-char (point-max))
          
          (insert (concat "\n//" message))))))

(defun ifind-clean-buffer(directory)
  "Clean up the ifind buffer so that we show full directory paths."

  (save-excursion

    (set-buffer ifind-buffer-name)

    (goto-char (point-min))

    (while (re-search-forward "^\\./" nil t)
      (replace-match (expand-file-name directory)))))

(defun ifind-mode-find-current-file()
  "Find the file on the current line."
  (interactive)

  (let(filename start end pol)

    (save-excursion

      (goto-char (point-at-eol))

      (setq pol (point)))

    (save-excursion

      (goto-char (point-at-bol))

      (assert (re-search-forward "[0-9] \\(/.*\\)$" pol t)
              nil "No file on current line.")
    
      (setq filename (buffer-substring-no-properties (match-beginning 1)
                                                     (match-end 1)))

      (find-file-other-window filename))))

(define-derived-mode ifind-mode fundamental-mode "IFind"
  "Major mode with key bindings to jump to files."
  (interactive)

  ;;(toggle-read-only 1)

  (font-lock-mode 1))

(font-lock-add-keywords 'ifind-mode '(("^//.*$"
                                       (0 'font-lock-comment-face t))))

(font-lock-add-keywords 'ifind-mode '((" \\(/.*\\)$"
                                       (1 'font-lock-keyword-face t))))

(define-key ifind-mode-map [return] 'ifind-mode-find-current-file)

(provide 'ifind)

;;; ifind.el ends here
