;;; ljupdate.el --- LiveJournal posting tool for Emacs

;; Copyright (C) 2001, 2002 Edward O'Connor <ted@oconnor.cx>
;; Copyright (C) 2002 Theodore Ts'o <tytso@mit.edu>

;; Maintainer: Edward O'Connor <ted@oconnor.cx>
;; Keywords: convenience

;; This file is NOT part of GNU Emacs.

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of version 2 of the GNU General Public License
;; as published by the Free Software Foundation.

;; This file is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with your Emacs; If you're using GNU Emacs, try typing C-h
;; C-c to bring it up. If you're using XEmacs, C-h C-l does this. If
;; not, write to the Free Software Foundation, Inc., 59 Temple Place -
;; Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; To use ljupdate, add something like the following to your .emacs
;; file:

;; (require 'ljupdate)

;; Use the command `lj-login' to log in to LiveJournal. `lj-compose'
;; will do this for you if you haven't, and in any case you only have
;; to do this once per Emacs session.

;; ljupdate.el's main entry point is the command `lj-compose', which
;; brings up a `*LiveJournal*' buffer in which you can write a
;; LiveJournal update. This mode pretends to be something similar to
;; the default `compose-mail' setup, with good reason: You get update
;; archiving for free, in your preferred mail format (babyl, mbox,
;; whatever).

;; If you want to post to a shared journal, add a `Target' header,
;; with the shared journal's name as it's value.

;; I would like bug reports, feature requests, and anything else for
;; that matter. If you just want to complain about the ickier parts of
;; the code, that's cool.

;;; History:
;; This is here so checkdoc doesn't complain. :)

;;; Code:

(condition-case nil (require 'md5)
  (error
   (message "%s %s" "MD5 library not installed."
            "This makes ljupdate very insecure!")
   (sit-for 3)
   (message "md5.el: http://www.gentei.org/~yuuji/software/md5.el")
   (sit-for 3)))
(require 'sendmail)
(require 'thingatpt)
(when (featurep 'xemacs)
  (require 'passwd))
(require 'cl)

;;; Constants:

(defconst lj-debug-p t
  "Whether or not you're debugging ljupdate.el or not.")

(defconst lj-client-version
  (concat "Emacs-ljupdate/" (cadr (split-string "$Revision: 1.40 $")))
  "Version of ljupdate.el, to be reported to the server.")

(defconst lj-use-md5-p (fboundp 'md5)
  "Whether or not to use md5.")

(defconst lj-password-argname
  (concat "&" (if lj-use-md5-p "h" "") "password="))

;;; Variables & Customize stuff:

;;;###autoload
(defgroup lj nil
  "LiveJournal customization group."
  :prefix "lj-"
  :group 'local)

;;;###autoload
(defvar lj-proxy nil
  "*If non-nil, the hostname of the proxy server to use.
If nil, don't use a proxy.")

;;;###autoload
(defvar lj-proxy-port 80
  "*Port on the proxy to use.")

;;;###autoload
(defvar lj-user-name nil
  "*Your LiveJournal username.")

;;;###autoload
(defvar lj-default-subject nil
  "*The default subject for a Live Journal entry.")

;;;###autoload
(defvar lj-default-mood nil
  "*The default mood for a Live Journal entry.")

;;;###autoload
(defvar lj-update-default-access-level "public"
  "*Access level to use by default for LiveJournal posts.")

;;;###autoload
(defvar lj-update-timeout-interval 180
  "*How long to wait for the LiveJournal server response. In seconds.")

;;;###autoload
(defvar lj-archive-file-name (expand-file-name "~/ljarchive.mbox")
  "*Mailbox into which to archive LiveJournal updates.
Set to nil if you don't want this to happen.")

;; Not-for-user-customization variables:

(defvar lj-update-mode-hook nil
  "Hooks to be executed when `lj-update-mode' is entered.")

(defvar lj-update-mode-map nil
  "Keyboard mapping used by `lj-update-mode'.")

(defvar lj-password nil
  "Your LiveJournal password.")

(defvar lj-update-mode-abbrev-table nil
  "Local abbrev table for lj-update mode.")

(define-abbrev-table 'lj-update-mode-abbrev-table ())

(defvar lj-logged-in-p nil
  "Whether or not you have logged into the LiveJournal system.")

(defvar *livejournal-buffer* nil
  "The current lj-update buffer. Please don't mess with this.")

(defvar lj-picture-list nil
  "List of user icons downloaded at login time")

(defvar lj-fast-p nil
  "A true value means that the fast LJ cookie should be included in requests.")

(defvar lj-login-response nil
  "The response from the LJ server when the user logged in")

(defvar lj-last-request nil
  "The last request sent to the LJ server.  (For debugging purposes)")

(defvar lj-last-response nil
  "The last response sent to the LJ server.  (For debugging purposes)")

(defvar lj-variable-list
  '(lj-user-name lj-picture-list lj-mood-alist)
  "LiveJournal variables saved in the startup file.")

(defvar lj-startup-file "~/.ljournal.eld"
  "The location of the LJ startup file.")

(defvar lj-startup-loaded-p nil
  "This is t if the lj-startup file has been loaded, and nil otherwise.")

(defvar lj-startup-modified-p nil
  "This is t if the lj-startup file needs to saved, and nil otherwise.")

(defvar lj-mood-alist nil
  "Alist of the various LiveJournal moods.")

;;; Some initialization type things:

(unless lj-use-md5-p
  (if (fboundp 'identity)
      (fset 'md5 (function identity))
    (defun md5 (arg) arg)))

(unless (fboundp 'char-int)
  (if (fboundp 'identity)
      (fset 'char-int (function identity))
    (defun char-int (char)
      char)))

(unless (fboundp 'char-to-int)
  (if (fboundp 'identity)
      (fset 'char-to-int (function identity))
    (defun char-to-int (char)
      char)))

(unless lj-update-mode-map
  (if (featurep 'xemacs)
      (progn
        (setq lj-update-mode-map (make-sparse-keymap))
        (set-keymap-parent lj-update-mode-map text-mode-map))
    (setq lj-update-mode-map (nconc (make-sparse-keymap) text-mode-map)))
  (define-key lj-update-mode-map "\C-i" 'lj-headers-complete-maybe)
  (define-key lj-update-mode-map "\C-c?" 'describe-mode)
  (define-key lj-update-mode-map "\C-c\C-c" 'lj-post-update-then-exit)
  (define-key lj-update-mode-map "\C-c\C-s" 'lj-post-update))

;;; Functions and Macros:

;;; Startup file support:

(defun lj-print-variables ()
  "Insert LJ variables in lisp format."
  (let ((print-quoted t)
	(print-escape-newlines t))
    (insert ";; -*- emacs-lisp -*-\n")
    (insert ";; LJ startup file.\n")
    (let ((variables lj-variable-list)
	  (print-quoted t)
	  (print-readably t)
	  (print-escape-multibyte nil)
	  variable)
      (while variables
	(when (and (boundp (setq variable (pop variables)))
		   (symbol-value variable))
	  (insert "(setq " (symbol-name variable) " '")
	  (prin1 (symbol-value variable) (current-buffer))
	  (insert ")\n"))))))

(defun lj-save-startup-file ()
  "Save .ljournal.eld file."
	  ;; Save .newsrc.eld.
  (when lj-startup-modified-p
    (with-current-buffer (get-buffer-create " *LiveJournal-startup*")
      (set (make-local-variable 'version-control) 'never)
      (setq buffer-file-name lj-startup-file)
      (setq default-directory (file-name-directory buffer-file-name))
      (buffer-disable-undo (current-buffer))
      (erase-buffer)
      (message "Saving %s..." lj-startup-file)
      (lj-print-variables)
      (run-hooks 'lj-save-startup-hook)
      (save-buffer)
      (kill-buffer (current-buffer))
      (message "Saving %s...done" lj-startup-file)
      (setq lj-startup-modified-p nil))))

(defun lj-load-startup-file ()
  "Load the .ljournal.eld file."
  (when (not lj-startup-loaded-p)
    (load lj-startup-file t t t)
    (setq lj-startup-loaded-p t)))



;; This helper function goes through a list and if an element contains
;; a space, it will add surrounding double quotes.
(defun lj-space-quote-list (list)
  (mapc (lambda (arg)
          (if (string-match " " arg)
              (concat "\"" arg "\"")
            arg))
        list))



;; Helper function which handles the generic completion functionality
(defun lj-do-complete (range alist)
  (let* ((thing (if range (buffer-substring (car range) (cdr range))))
	 (completion (try-completion (if thing thing "") alist)))
    (unless thing
      (setq thing ""))
    (when (and (car range) (cdr range))
      (delete-region (car range) (cdr range)))
    (cond ((equal completion t)
	   (insert thing))
	  ((stringp completion)
	   (insert completion)
	   (let ((completions (all-completions thing alist)))
	     (when (> (length completions) 1)
	       (message "Possible completions: %s"
			(lj-space-quote-list completions)))))
	  (t (beep)
             (insert thing)
             (message "No completion for %s!" thing)))))



(defun lj-headers-complete-maybe ()
  "Handle completion of the mood and picture headers."
  (interactive)
  (let* (range
	 thing
	 alist
	 complete
	 eoh
	 (completion-ignore-case t)
	 (the-point (point))
	 (eol-point (save-excursion
		      (end-of-line)
		      (if (re-search-backward "[^ 	]" the-point t)
			  (+ the-point 1)
			the-point))))
    (beginning-of-line)
    (cond ((re-search-forward "^Mood:" eol-point t)
           (setq eoh (point))
           (goto-char the-point)
           (setq range (bounds-of-thing-at-point 'word)
                 alist lj-mood-alist
                 complete t))
	  ((re-search-forward "^Picture:" eol-point t)
           (setq eoh (point))
           (if (re-search-forward "[^ 	]" (+ eol-point 1) t)
               (backward-char 1))
           (setq range (cons (point) eol-point)
                 alist lj-picture-list
                 complete t)))
    (goto-char the-point)
    (if complete
	(if (or (< the-point eoh)
		(> the-point eol-point))
	    (beep)
	  (lj-do-complete range alist))
      (insert "\C-i"))))



(defun lj-buffer-kill-hook ()
  (setq *livejournal-buffer* nil))



(defun lj-update-mode ()
  "Major mode for editing a LiveJournal update."
  ;; Normally, major mode functions are interactive. This one isn't,
  ;; and will hopefully only be called from lj-compose.

  (kill-all-local-variables)

  (setq major-mode 'lj-update-mode)
  (setq mode-name "LiveJournal")

  (add-hook (make-local-variable 'kill-buffer-hook)
            'lj-buffer-kill-hook)

  ;; Since we're pretending to be like mail, why don't we let the mail
  ;; font locking do the work for us? :)
  (set (make-local-variable 'font-lock-defaults)
       '(mail-font-lock-keywords t))

  (use-local-map lj-update-mode-map)

  ;; These two lines are arguably controversial. I'm just making sure
  ;; that you don't auto-fill, because the result is really
  ;; lame-looking.
  (set (make-local-variable 'truncate-lines) nil)
  (if (fboundp 'turn-off-auto-fill)
      (turn-off-auto-fill)
    (auto-fill-mode -1))

  ;; Abbrev support
  (setq local-abbrev-table lj-update-mode-abbrev-table)

  ;; Make paragraph fill functions work correctly
  (make-local-variable 'paragraph-separate)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat (regexp-quote mail-header-separator)
				"$\\|\t*\\([-|#;>* ]\\|(?[0-9]+[.)]\\)+$"
				"\\|[ \t]*[a-z0-9A-Z]*>+[ \t]*$\\|[ \t]*$\\|"
				"-- $\\|---+$\\|"
				page-delimiter))
  (setq paragraph-separate paragraph-start)


  ;; But if you don't like the above two lines, why not leave them
  ;; there and add a hook that will fix things for you? :) Some
  ;; suggested things to turn on in this hook: flyspell (we get spell
  ;; checking for free. yay.)
  (run-hooks 'lj-update-mode-hook))



(defun lj-read-response-alist ()
  "Create an alist from the LiveJournal server response."
  (let  ((lj-response-alist nil)
         (here (point))
         (end (progn (goto-char (point-max))
                     (point)))
         key
         val)

    (goto-char here)
    (while (< (point) end)
      (beginning-of-line)
      (setq key (buffer-substring (point) (progn (end-of-line)
                                                 (point))))
      (forward-line 1)
      (beginning-of-line)
      (setq val (buffer-substring (point) (progn (end-of-line)
                                                 (point))))
      (forward-line 1)
      (add-to-list 'lj-response-alist (cons key val)))
    lj-response-alist))



;; This function handles sending things to the LiveJournal server and
;; parsing return values from it.
(defun lj-talk-to-server (msg)
  "Send MSG to the LiveJournal server.
Return value is a cons: the car is non-nil if the transaction worked,
and nil if it didn't. The cdr is an alist of values from the server."
  (setq lj-last-request msg)
  (let* ((worked-or-not nil)
         (response-alist nil)
         (msg-to-send
          (if (featurep 'xemacs)
              msg
            (if (coding-system-p 'utf-8)
                (encode-coding-string
                 (decode-coding-string msg 'emacs-mule)
                 'utf-8)
              msg)))
         (proc (open-network-stream "LiveJournal"
                                    "*LiveJournal-Server-Response*"
                                    (or lj-proxy "www.livejournal.com")
                                    (if lj-proxy
                                        lj-proxy-port
                                      80)))
         (buf (process-buffer proc))
         (headers (concat "POST "
                          (if lj-proxy
                              "http://www.livejournal.com"
                            "")
                          "/cgi-bin/log.cgi HTTP/1.0\r\n"
                          "Host: www.livejournal.com\r\n"
			  (if lj-fast-p
			      "Cookie: ljfastserver=1\r\n"
			    "")
                          "Content-type:"
                          "application/x-www-form-urlencoded\r\n")))

    ;; Perhaps this will allow for multibyte posting? We'll see.
    (unless (featurep 'xemacs)
      (when (coding-system-p 'utf-8)
        (set-process-coding-system proc 'utf-8 'utf-8)))

    (process-send-string proc
                         (concat headers "Content-length: "
                                 (format "%d" (length msg-to-send))
                                 "\r\n\r\n" msg-to-send))

    (message "Waiting for a response from the LiveJournal server...")
    (while (equal (process-status proc) 'open)
      (when (not (accept-process-output proc lj-update-timeout-interval))
        (delete-proc proc)
        (error "Server error: timed out while waiting!")))

    (message "Response received; processing...")

    (with-current-buffer buf
      (goto-char (point-min))
      (replace-string "\C-m" "")
      (goto-char (point-min))

      (if (looking-at "^HTTP/1.* 200 OK$")
	  (progn
	    (setq worked-or-not t)

	    (while (not (looking-at "^$"))
	      (forward-line 1))
	    (forward-line 1)
	    (let* ((lj-response-alist (lj-read-response-alist))
		   (val (assoc "success" lj-response-alist)))
	      (setq response-alist lj-response-alist)
	      (if val
		  (if (equal (cdr val) "OK")
		      (setq worked-or-not t)
		    (setq worked-or-not nil)
		    (message "Server error, errmsg is %s"
			     (assoc "errmsg" lj-response-alist)))
		(setq worked-or-not nil)
		(message "Server error: No success key found."))))
	(setq worked-or-not nil)
	(message "Server error: Not 200 OK.")))

    (when worked-or-not
      (message "Successfully talked to LiveJournal.")
      (kill-buffer buf))

    (setq lj-last-response response-alist)
    (cons worked-or-not response-alist)))



;;;###autoload
(defun lj-login (arg)
  "Login to the LiveJournal server.
Prompts for a password. If given a prefix ARG, prompts for a username,
usually taken from `lj-user-name'."

  (interactive "P")

  (setq lj-fast-p t)   ; Assume we need to use the slow servers first
  (lj-load-startup-file)
  (let ((login-message
         (concat "mode=login"
                 "&user="
                 (lj-url-escape
                  (progn
                    (setq lj-user-name
                          (if (or arg (not lj-user-name))
			      (progn
				(setq lj-startup-modified-p t)
				(read-string "LiveJournal username: "))
			    lj-user-name))
                    lj-user-name))
                 lj-password-argname
                 (lj-url-escape
                  (progn
                    (setq lj-password
                          (md5 (read-passwd (format "LiveJournal password for %s: " lj-user-name))))
                    lj-password))
                 "&clientversion=" (lj-url-escape lj-client-version)
		 "&getmoods="
		 (int-to-string (apply 'max
                                       (cons 0 (mapcar 'cdr lj-mood-alist))))
		 "&getpickws=1"
                 "\r\n")))
    (setq lj-login-response (lj-talk-to-server login-message))
    (setq lj-logged-in-p (car lj-login-response))
    (when lj-logged-in-p
      ;; Get the list of valid pictures
      (let ((count 0)
            (maxcount (string-to-number
                       (cdr (assoc "pickw_count" lj-login-response))))
            picklist)
        (while (< count maxcount)
          (setq count (+ count 1))
          (setq picklist (cons (cons (cdr (assoc
                                           (concat "pickw_"
                                                   (int-to-string count))
                                           lj-login-response))
                                     count)
                               picklist)))
        (when (not (equal lj-picture-list picklist))
          (setq lj-startup-modified-p t)
          (setq lj-picture-list picklist)))
      ;; Get any new moods
      (let* ((count 0)
             (mood_count (cdr (assoc "mood_count" lj-login-response)))
             maxcount picklist)
        (when mood_count
          (setq maxcount (string-to-number mood_count))
          (setq lj-startup-modified-p t)
          (while (< count maxcount)
            (setq count (+ count 1))
            (setq lj-mood-alist
                  (cons (cons
                         (cdr (assoc (concat "mood_"
                                             (int-to-string count)
                                             "_name")
                                     lj-login-response))
                         (string-to-number
                          (cdr (assoc (concat "mood_"
                                              (int-to-string count)
                                              "_id")
                                      lj-login-response))))
                        lj-mood-alist)))))
      ;; Check to see if we're allowed to use the fast servers
      (setq lj-fast-p (assoc "fastserver" lj-login-response))))
  (lj-save-startup-file))



;; This function is the main entry point for this code. It pops up a
;; buffer similar to what you would get with C-x m.
;;;###autoload
(defun lj-compose ()
  "Compose a LiveJournal update."
  (interactive)
  (or *livejournal-buffer*
      (setq *livejournal-buffer* (generate-new-buffer "*LiveJournal*")))

  (switch-to-buffer *livejournal-buffer*)
  (delete-region (point-min) (point-max))
  (lj-update-mode)

  (insert (concat "Subject: " lj-default-subject "\n"
                  "Music: \n"
                  "Mood: " lj-default-mood "\n"
                  "Picture: \n"
                  "Target: \n"
                  (if lj-archive-file-name
                      (concat "FCC: " lj-archive-file-name "\n")
                    "")

                  (if lj-update-default-access-level
                      (concat "Access: "
                              lj-update-default-access-level "\n")
                    "")))

    (set (make-local-variable 'header-end)
         (if (and (not (featurep 'xemacs))
                    (>= emacs-major-version 21))
           (point-marker)
         (point)))

    (put-text-property (point)
                       (progn
                         (insert mail-header-separator "\n")
                         (1- (point)))
                       'category 'mail-header-separator)
    (if (not lj-logged-in-p) (lj-login nil)))



(defmacro lj-between-p (elt left right)
  "Is ELT numerically in between LEFT and RIGHT?"
  `(and (>= ,elt ,left) (<= ,elt ,right)))



(defun lj-url-escape (string)
  "Escape the STRING."
  (when (stringp string)
    (let ((escaped-list '()))
      (dolist (elt (split-string string ""))
        (let ((char (char-to-int (or (string-to-char elt)
                                     ?\ ))))
          (cond ((or (lj-between-p char (char-int ?A) (char-int ?Z))
                     (lj-between-p char (char-int ?a) (char-int ?z))
                     (lj-between-p char (char-int ?0) (char-int ?9)))
                 (setq escaped-list (nconc escaped-list (list elt))))
                ((= char ?\ )
                 (setq escaped-list (nconc escaped-list (list "+"))))
                (t
                 (setq escaped-list
                       (nconc escaped-list
                              (list (format "%%%02x" char))))))))
      (apply 'concat escaped-list))))


;;
;; Modified by TYT to strip out trailing whitespace.
;;
(defun lj-update-fetch-header (header)
  "Grab the requested HEADER field."
  (goto-char (point-min))
  (if (re-search-forward (format "^%s:[ ]" header) (mail-header-end) t)
      (let ((begin (point)))
	(end-of-line)
	(if (re-search-backward "[^ 	]" begin t)
	    (buffer-substring begin (+ (point) 1))
	  ""))
    ""))



(defun lj-fetch-date-and-time-alist ()
  "Create a date and time alist from the current time."
  (let ((time-list (split-string (current-time-string) "[ :]+"))
        (month-list '("Jan" "Feb" "Mar" "Apr" "May" "Jun"
                      "Jul" "Aug" "Sep" "Oct" "Nov" "Dec"))
        (dt-alist '()))

    (add-to-list 'dt-alist (cons 'year (car (last time-list))))
    (add-to-list 'dt-alist
                 (cons 'mon
                       (format "%d"
                               (1+ (- (length month-list)
                                      (length (member (cadr time-list)
                                                      month-list)))))))
    (add-to-list 'dt-alist (cons 'day (caddr time-list)))
    (add-to-list 'dt-alist (cons 'hour (cadddr time-list)))
    (add-to-list 'dt-alist (cons 'min (fifth time-list)))
    dt-alist))



(defun lj-post-update ()
  "Post this LiveJournal update,."
  (interactive)

  (when (equal (current-buffer) *livejournal-buffer*)

    (unless lj-logged-in-p
      (lj-login nil))

    ; Reformat the paragraphs so that they are on a single line; LJ will
    ; convert newlines to <BR>, so we want to get rid of them from
    ; the buffer.
    (let ((fill-column 65536)
	  (sentence-end-double-space t))
      (fill-region
       (progn (goto-char (mail-header-end))
	      (forward-line 1)
	      (point))
       (point-max) 'left t))

    (let* ((subject (lj-update-fetch-header "Subject"))
           (music (lj-update-fetch-header "Music"))
           (mood (lj-update-fetch-header "Mood"))
           (picture (lj-update-fetch-header "Picture"))
           (moodid (or (cdr (assoc mood lj-mood-alist)) -1))
           (target (lj-update-fetch-header "Target"))
           (access (lj-update-fetch-header "Access"))
           (date-time-alist (lj-fetch-date-and-time-alist))
	   (picture (if (and (string-equal picture "")
			     (assoc-ignore-case mood lj-picture-list))
			mood picture))
           (msg (concat "mode=postevent"
                 "&user=" (lj-url-escape lj-user-name)

                 ;; either &password= or &hpassword=, depending on md5
                 ;; availability.
                 lj-password-argname (lj-url-escape lj-password)

                 "&year=" (cdr (assoc 'year date-time-alist))
                 "&mon="  (cdr (assoc 'mon  date-time-alist))
                 "&day="  (cdr (assoc 'day  date-time-alist))
                 "&hour=" (cdr (assoc 'hour date-time-alist))
                 "&min="  (cdr (assoc 'min  date-time-alist))

                 (if (not (string-equal music ""))
                     (concat "&prop_current_music="
                             (lj-url-escape music))
                   "")

                 (cond ((not (= moodid -1))
                        (concat "&prop_current_moodid="
                                (int-to-string moodid)))
                       ((not (string-equal mood ""))
                        (concat "&prop_current_mood="
                                (lj-url-escape mood)))
                       (t ""))

                 (if (not (string-equal subject ""))
                     (concat "&subject="
                             (lj-url-escape subject))
                   "")

                 (if (not (string-equal target ""))
                     (concat "&usejournal="
                             (lj-url-escape target))
                   "")

		 (if (not (string-equal picture ""))
		     (concat "&prop_picture_keyword="
			     (lj-url-escape picture))
		   "")

                 (if (string-match "rivate" access)
                     "&security=private"
                   "&security=public")

                 "&event=" (lj-url-escape (string-as-unibyte
                            (buffer-substring
                             (progn (goto-char (mail-header-end))
                                    (forward-line 1)
                                    (point))
                             (point-max)))))))

      (mail-sendmail-undelimit-header)
      (mail-do-fcc header-end)
      (bury-buffer *livejournal-buffer*)
      (lj-talk-to-server msg))))



(defun lj-post-update-then-exit ()
  "Post this LiveJournal update, and kill the *LiveJournal* buffer."
  (interactive)

  (when (equal (current-buffer) *livejournal-buffer*)
    (when (car (lj-post-update))
      (kill-buffer (current-buffer))
      (setq *livejournal-buffer* nil))))



(provide 'ljupdate)

;;; ljupdate.el ends here
