;;; lynx.el --- text based lynx interface to emacs. 

;; $Id: lynx.el,v 1.15 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.
;;
;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Lynx interface for emacs.  I wanted a quick way to view HTML from within
;; Emacs and W3 was too slow in a lot of circumstances.  This allows you to have
;; an alternative to W3 with full Emacs integration.
;;
;; This also works from within browse-url via browse-url-lynx.  

;;; TODO:
;;
;; - Lynx does not support https:// URLs.  If we use one make sure to warn the
;;   user.
;;
;; - Everytime we jump to a new url by [tab] have a (message) show the URL.
;; 
;; - All rendering should be done in two phases:
;;   - Pull down the URL as HTML (cache it)
;;   - Use lynx to render the cached data.
;;
;;   - This is needed because we need the source HTML to pull things like
;;     title out. 
;; 
;; - All this code needs to be audited.  We changed feature sets while
;;   developing the code.  Write down all the features we want and then think
;;   about how we are going to rework the code so that it is totally stable. 
;;
;; - I think that we should rework it so that everything a new URL is manually
;;   browse from the commandline we should create a new buffer.  This way we
;;   will have a a new buffer so that old sites we are visiting don't get
;;   destroyed.  
;;
;; - this totally breaks if it can't resolve hosts.  
;;
;;    - TEST: try this out on URLs that don't exist
;;  
;; - need a lynx-browse-region which will format a region as html and dump it.
;;  - need to specify a nil URL
;;
;; - write a lynx-show-url-at-point() function this should display the current
;;   URL under a [3] link.
;;
;; - if we can't view a file type within emacs... prompt to save it to disk. 
;; 
;; - need to build in a facility to cache URLs.
;;
;; - add the support to view the images.  Generally though how do we do this
;;   within emacs?  We need a decent way to handle launching the image viewer.  
;; 
;; - support browser history so what we can go backward and forward.  Keep a
;;   list of the url path in a lynx-history-alist.
;; 
;; - update the Commentary
;;

;;; History:
;;
;; - Fri Mar 16 19:33:31 2001 (burton@relativity.yi.org): have support for
;; browse-url.  It would be nice if we could render the current buffer/url with
;; 'browse-url'.  This way we could load it into Mozilla.  Also we should be
;; able to click on a regular URL and load it with mozilla with <return>
;;
;; - Fri Mar 16 18:53:36 2001 (burton@relativity.yi.org): set the mime-type
;; based on the URL if this is a file:// URL or for some reason we can't get to
;; the host.
;;
;; - Sat Mar 10 15:35:55 2001 (burton@relativity.yi.org): If the file
;; we are viewing is text/html and it is source then use html-mode.
;;
;; - Sat Mar 10 15:35:40 2001 (burton@relativity.yi.org): Keep the mime-type
;; info in lynx-url-info
;;
;; - Sat Mar 10 15:35:32 2001 (burton@relativity.yi.org): only update
;; lynx-url-info within the function lynx-browse
;;
;; - Sat Mar 10 15:19:49 2001 (burton@relativity.yi.org): lynx will not be used
;;   as a library.  Make we have it ALWAYS view buffers. No (interactive-p)
;;   detection...
;;
;; - Sat Mar 10 15:18:29 2001 (burton@relativity.yi.org): quick way to view HEAD
;; info. 
;;
;; - Sat Mar 10 15:02:36 2001 (burton@relativity.yi.org): lynx-browse should take
;; the type request as a type.. IE 'browse or 'source
;;
;; - Sat Mar 10 00:35:56 2001 (burton@relativity.yi.org): dump source into
;; another buffer.
;;
;; - Sat Mar 10 00:35:01 2001 (burton@relativity.yi.org): - add a "view source"
;; option.
;;
;; - Thu Mar  8 12:14:58 2001 (burton@relativity.yi.org): need the ability to
;; refresh a documnent and pay attention to whether it is a source or a dump.
;;
;; - Thu Mar 8 05:07:43 2001 (burton@relativity.yi.org): we are one off when
;; looking for the URL
;;
;; - Thu Mar 8 05:03:09 2001 (burton@relativity.yi.org): scroll directly to
;; links (up|down) just like normal lynx.
;;
;; - Thu Mar  8 02:56:05 2001 (burton@relativity.yi.org): don't wrap at 80 cols...

(require 'w3) ;;needed for (w3-download-url).  

(defvar lynx-browse-buffer "*lynx*" "Buffer used for lynx operation.")

(defvar lynx-source-buffer "*lynx-source*" "Buffer used for lynx source data.")

(defvar lynx-head-buffer "*lynx-head*" "Buffer used for lynx head data.")

(defvar lynx-mode-string " Lynx" "The minor mode string displayed when mode is on.")

(defvar lynx-mode nil  "Non-nil means `lynx-mode' local minor mode is enabled.")
(make-variable-buffer-local 'lynx-mode)

(defvar lynx-mode-map nil "Map for lynx key bindings.")
(setq lynx-mode-map (let ((map (make-sparse-keymap)))
                      (define-key map [?\C-c?\C-b] 'lynx-browse)
                      (define-key map [?\C-c?\C-r] 'lynx-refresh)
                       (define-key map [?\C-c?\C-s] 'lynx-view-source)
                       (define-key map [?\C-c?\C-h] 'lynx-view-head)
                      (define-key map [(tab)] 'lynx-scroll-down)
                      (define-key map [?\C-cup] 'lynx-scroll-up)
                      (define-key map [?\C-cdown] 'lynx-scroll-down)
                      map))

(defvar lynx-known-browser-menu nil
  "Menu items for browsing the current buffer with another browser.")

(setq lynx-known-browser-menu (list
                               ["Browse with Mozilla" browse-url-mozilla t]))

(defvar lynx-mode-menu nil "Menu for lynx mode.")
(easy-menu-define
 lynx-mode-menu
 lynx-mode-map
     "Lynx menu"
     (list
      "Lynx"
      ["Browse new URL" lynx-browse t]
      ["Refresh" lynx-refresh t]
      "----"
      ["View Source" lynx-view-source t]
      ["View HEAD info" lynx-view-head t]
      "----"
      ["Browse with Konqueror" (lynx-browse-url-with 'konqueror) t]
      ["Browse with Mosaic" (lynx-browse-url-with 'mosaic) t]
      ["Browse with Mozilla" (lynx-browse-url-with 'mozilla) t]
      ["Browse with Netscape" (lynx-browse-url-with 'netscape) t]
      ["Browse with W3" (lynx-browse-url-with 'w3) t]
      "----"
      ["Next link" lynx-scroll-down t]
      ["Previous link" lynx-scroll-up t]))

(defvar lynx-local-map nil "Map used to click on a link")
(setq lynx-local-map (let ((map (make-sparse-keymap)))
                       (define-key map [mouse-2] 'lynx-click-link)
                       (define-key map [(return)] 'lynx-click-link)
                       (define-key map [(C-return)] 'lynx-click-link-browse-url)
                       map))

(defvar lynx-link-regexp "\\[[0-9]+\\]" "Regexp to use when finding links.")

(defvar lynx-last-link-match-end nil
  "Keep track of the end point of URLs so I can jump past them.")
(make-variable-buffer-local 'lynx-last-link-match-end)

(defvar lynx-last-link-match-start nil
  "Keep track of the end point of URLs so I can jump past them.")
(make-variable-buffer-local 'lynx-last-link-match-start)

(defvar lynx-url-info nil
  "List info used to browse this buffer.  First element is the url, second is the
method ('browse or 'source ), third is the mime-type.")
(make-variable-buffer-local 'lynx-url-info)

(defface lynx-link-face '((t (:foreground "blue"  :underline t)))
  "Face used to highlight symbolic links.")

(defface lynx-image-face '((t (:foreground "gray50" :italic t)))
  "Face use to highlight images.  Use gray so that they really aren't seen.")

(defun lynx-browse(url &optional operation-type buffer)
  "Support for browsing with lynx.  See `browse-url-lynx-emacs' for an
alternative.  If called interactively then show the buffer.  `operation-type'
can be either 'browse (download the url and browse it) or 'source (dump its
source with no interpretation).   This function also performs the following
operations:

- fetches head data for all requested URLs.
- makes sure that the given URL is supported by lynx.
- tries to guess the correct major-mode of the output based on mime-type.
- updates lynx-url-info on every request (buffer local)."
  (interactive
   (list (read-string "URL: " nil 'lynx)))

  ;;remove any text properties from the URL
  (set-text-properties 0 (length url) nil url)
  
  ;;setup the default operation-type if it isn't specified
  (if (null operation-type)
      (setq operation-type 'browse))
  
  ;;only http or file urls should be supported
  (if (not (or (string-match "^http" url)
               (string-match "^file" url)))
      (error "Only http:// urls are supported"))
  
  ;;use the default buffer if it isn't specified.
  (if (null buffer)
      (setq buffer lynx-browse-buffer))
      
  (set-buffer (get-buffer-create buffer))
  (toggle-read-only -1)
  (auto-fill-mode -1) ;;make sure auto-fill-mode is off
  
  (message "Fetching %s..." url)

  ;;get head data first
  ;;only fetch head if this is not a file url

  (if (not (string-match "^file" url))
      (lynx-head url))

  ;;If this is a media/type we dont' support (IE not text/html or text/plain
  ;;then we can't view this.  Offload this to the correct viewer (or download)
  (let(mime-type url-info)

    (setq mime-type (lynx-mime-type url))

    (if (string-match "^image/" mime-type)
        (progn
          ;;view images

          (let(filename)

            (setq filename (make-temp-name (concat (getenv "TEMP") "/")))

            (lynx-download-url url filename)

            (lynx-view-image filename)))
      
      (if (string-match "^text/" mime-type)
          (progn 
            ;;else view text
            
            (erase-buffer)
            
            (if (equal operation-type 'source)
                (insert (lynx-exec (concat "links -source " url)))
              (insert (lynx-exec (concat "links -dump " url))))
  
            (goto-char (point-min))

            (if (not (equal operation-type 'source))
                (lynx-mode-make-clickable))
  
            (message "Fetching %s...done" url)

            (toggle-read-only 1)
            
            (lynx-show-buffer))
        (error "The mime-type '%s' is not supported for %s" mime-type url)))

    ;;buffer should be set correctly here.  set the mime-type info.
    (set-buffer (get-buffer-create buffer)) ;;just in case something accidentally updates this in the future.

    ;;for GNU Emacs 21

    (setq url-info (list url operation-type mime-type))
    
    (lynx-guess-major-mode url operation-type mime-type buffer)

    (let((url-title "URL: ")
         (server-title "Server: "))
      
      (put-text-property 0 4 'face 'bold url-title)
      
      (put-text-property 0 7 'face 'bold server-title)
      
      ;;(set-text-properties 0 (length url) nil url)
      
      ;;On http URLs add the Server
      (if (string-match "^http" url)
          (setq header-line-format (concat url-title
                                           url
                                           " "
                                           server-title
                                           (lynx-get-header "Server")))
        (setq header-line-format (concat url-title url))))
    
    (toggle-read-only 1)
    
    ;;this MUST be set after the major mode is guessed.
    (setq lynx-url-info url-info))
  
  (message url))

(defun lynx-get-header(name)
  "Get the named value from the HTTP headers"

  (save-excursion
    (set-buffer lynx-head-buffer)
    (goto-char (point-min))
    (if (re-search-forward (concat "\\(^" name ": \\)\\(.*$\\)") nil t)
        (match-string 2)
      nil)))

(defun lynx-browse-file(file)
  "Browse for an html file and open it with lynx."
  (interactive 
   (let(file)
     (setq file (read-file-name "File URL: " nil nil t))

     (list file)))

  (setq file (concat "file://" file))
  
  (lynx-browse file))

(defun lynx-browse-current-buffer()
  "Browse the current buffer with lynx."
  (interactive)

  (lynx-browse-file (buffer-file-name)))

(defun lynx-view-image(file)
  "View the given image."

  ;;FIXME: need a X-platform way to view images

  (message "Viewing image...")
  (start-process "xv"  nil "xv" file)
  (message "Viewing image...done"))

(defun lynx-guess-major-mode(url operation-type mime-type buffer)
  "Guess which major mode to use for this buffer"

  (set-buffer buffer)

  ;;html-mode on source buffers.
  (if (and (equal operation-type 'source)
           (string-match "text/html" mime-type))
      (html-mode)
    (progn
      ;;else if it is text/ anything then use text-mode
      (if (string-match "text/" mime-type)
          (text-mode))))

  (lynx-mode 1))
  
(defun lynx-view-head()
  "View the header info for this buffer"
  (interactive)
  (shrink-window-if-larger-than-buffer (display-buffer lynx-head-buffer)))

(defun lynx-mime-type(url)
  "Get the mime-type from the current `lynx-head-buffer'."

  (let(mime-type)
  
    ;;if this is a file:// URL then we can't use mime-type info.  Try to
    ;;automatically guess.

    (if (string-match "^file" url)
        (progn 
          ;;default to html...

          ;;TODO: use extension to guess type.  
          (setq mime-type "text/html")))
  
    (save-excursion
      (set-buffer (get-buffer-create lynx-head-buffer))

      (goto-char (point-min))
      
      (if (re-search-forward "\\(^Content-Type: \\)\\(.*$\\)" nil t)
          (setq mime-type (match-string 2))

        ;;try to guess the URL based on the url
        (if (string-match ".html$" url)
            (setq mime-type "text/html"))
        
        (if (string-match ".gif$" url)
            (setq mime-type "image/gif"))

        (if (string-match ".jpg$" url)
            (setq mime-type "image/jpeg"))))

    ;;give back the mime-type
    mime-type))
  
(defun lynx-head(url)
  "Fetch HTTP HEAD information."

  (save-excursion 
    (set-buffer (get-buffer-create lynx-head-buffer))
    
    (toggle-read-only -1)
    (erase-buffer)
    
    (insert (lynx-exec (concat "lynx -head -dump " url)))

    (goto-char (point-min))

    (font-lock-mode 1)
    (font-lock-add-keywords nil '(("\\(^[a-zA-Z-]+: \\)" 1 'font-lock-keyword-face keep)))
    (font-lock-fontify-buffer)))
  
(defun lynx-source(url)
  "Fetch the source for the given url."
  (interactive
   (list (read-string "URL: " nil 'lynx)))

  (lynx-browse url 'source lynx-source-buffer)

  ;;FIXME: this show-buffer should not go here
  (show-buffer nil lynx-source-buffer))
  
(defun lynx-view-source()
  "View source for the current buffer."
  (interactive)

  (let(url)

    (setq url (elt lynx-url-info 0))

    (lynx-source url)))
  
(defun lynx-refresh()
  "Refresh the current lynx buffer"
  (interactive)

  (let(url type)

    (setq url (elt lynx-url-info 0))

    (setq type (elt lynx-url-info 1))

    (case type
      ('browse
       (lynx-browse url))
      ('source
       (lynx-source url)))))

(defun browse-url-lynx(url &optional new-window)
  "Browse a url with lynx.  Provided so that users can use this package from `browse-url'."
  (interactive
   (list (read-string "Lynx URL: ")))

  (lynx-browse url)
  (lynx-show-buffer))

(defun lynx-exec(command)
  "Fetch a URL and log this."

  ;;FIXME: If lynx does not exit correctly we need to warn the user.

  (let(output)
  
    (setq output (shell-command-to-string command))

    ;;clean the output if necessary.  should this be a dedicated -cleanse function?
    (if (string-match "^stty: standard input: Inappropriate ioctl for device$" output)
        (setq output (replace-match "" nil nil output)))

    output))
    
(defun lynx-get(url data)
  "Post the given data to the given URL.  The output will be stored in the
`lynx-browse-buffer'.  The variable `data' is a list which contains cons cells of name
and value pairs."

  (setq url (concat url "?"))

  (let((i 0) pair)

    (while (< i (length data))

      (setq pair (elt data i))
      
      (setq url (concat url (car pair) "=" (cdr pair)))

      (if (< i (1- (length data)))
          (setq url (concat url "&")))
      
      (setq i (1+ i))))
  
  (lynx-browse url))

(defun lynx-show-buffer(&optional buffer)
  "Show the lynx buffer and enable `lynx-mode'."
  (interactive)

  (if (null buffer)
      (setq buffer lynx-browse-buffer))
  
  (show-buffer nil buffer))

(defun lynx-show-lint-at-point()

  )

(defun lynx-click-link-browse-url()
  "Click the current link but use browse-url"
  (interactive)
  
  (lynx-click-link t))

(defun lynx-click-link(&optional browse-url)
  "Click the current lynx link.  If `browse-url' is true then use browse-url to
open this link."
  (interactive)

  ;;WARNING:.  this depends on the point being within a link.

  (let(begin end number url url-regexp)

    ;;go grab the URL...
    (save-excursion

      (forward-char 1)
      (search-backward "[" nil t)
      (setq begin (1+ (point)))

      (search-forward "]" nil t)

      (setq end (1- (point)))

      (setq number (buffer-substring begin end))

      (goto-char (point-max))

      (setq url-regexp (concat "^[ ]*" number  "\\. "))

      (re-search-backward url-regexp nil t)
      (goto-char (match-end 0))

      (re-search-forward ".*$" nil t)

      (setq url (match-string 0)))

    (if browse-url
        (browse-url url)
      (lynx-browse url))))

(defun lynx-scroll-down()
  "Search down for the next link."
  (interactive)

  (if (> (point) 1)
      (re-search-forward lynx-link-regexp nil t))
  
  (re-search-forward lynx-link-regexp nil t)

  (goto-char (match-beginning 0)))

(defun lynx-scroll-up()
  "Search down for the next link."
  (interactive)
  
  (re-search-backward lynx-link-regexp nil t))

(defun lynx-mode(&optional arg)
  "turn lynx-mode on/off"
  (interactive)
  (if (if arg
          (> (prefix-numeric-value arg) 0)
        (not lynx-mode))
      (lynx-mode-on)
    (lynx-mode-off)))

(defun lynx-mode-on()
  "Turn on lynx mode."
  (setq lynx-mode t)

  (save-excursion
    (goto-char (point-min))
    (toggle-read-only -1)
    (lynx-mode-make-clickable))
  
  ;;Look for images.
  ;;KEEP the regexp small so that it is obvious what is going on
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.jpg\\]\\)" 1 'lynx-image-face keep)))
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.gif\\]\\)" 1 'lynx-image-face keep)))
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.png\\]\\)" 1 'lynx-image-face keep)))
  
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.JPG\\]\\)" 1 'lynx-image-face keep)))
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.GIF\\]\\)" 1 'lynx-image-face keep)))
  (font-lock-add-keywords nil '(("\\(\\[[^[]+.PNG\\]\\)" 1 'lynx-image-face keep)))

  (font-lock-fontify-buffer)

  (easy-menu-add lynx-mode-menu))

(defun lynx-mode-off()
  "Turn off lynx-mode."
  (setq lynx-mode nil)
  (easy-menu-remove lynx-mode-menu))

(defun lynx-mode-make-clickable()
  "Make all links in lynx-mode clickable.  Example: [1]."
  
  (save-excursion
    (goto-char (point-min))

    (while (re-search-forward "\\(\\[[0-9]+\\]\\)" nil t)
      
      ;;fix this...
      (let((begin (match-beginning 0))
           (end (match-end 0))
           current-extent)

        (setq current-extent (make-extent begin end))

        (set-extent-property current-extent 'face 'lynx-link-face)
        
        (set-extent-property current-extent 'window (selected-window))

        ;;setup the click option to jump to the next URL
        
        (put-text-property begin end 'local-map lynx-local-map)
        (put-text-property begin end 'mouse-face 'highlight)))))

(defun lynx-reset(main-buffer)

  (set-buffer (get-buffer-create main-buffer))
  (toggle-read-only -1)
  (erase-buffer)

  (set-buffer (get-buffer-create lynx-head-buffer))
  (toggle-read-only -1)
  (erase-buffer)

  (insert "HEAD DATA NOT SET"))

(defun lynx-browse-region(start end &optional buffer)
  "Given a region, browse it with lynx."
  
  (interactive "r")

  (if (null buffer)
      (setq buffer (current-buffer)))

  (let(filename temp-buffer region-string)

    ;;get a temporary file 
    (setq filename (make-temp-name (concat
                                    (getenv "TEMP")
                                    "/lynx.el")))

    ;;add .html to the name...
    (setq filename (concat filename ".html"))
    
    (save-excursion 

      (message "Lynx temp file: %s" filename)
      
      ;;get the region
      (set-buffer buffer)

      (setq region-string (buffer-substring start end))

      ;;open the temp file...
      (setq temp-buffer (find-file filename))
      
      ;;set the content of the temp file
      (set-buffer temp-buffer)
      (insert region-string)
      (save-buffer)
      
      (kill-buffer temp-buffer))

    ;;browse the temp buffer as a file:// URL. 
    (lynx-browse-file filename)))

(defun lynx-browse-region-replace(start end &optional buffer)
  "Replace the html in the given region with the content of it rendered after
lynx is done with it. "

  (interactive "r")

  (let(html-rendered-content)

    (save-window-excursion

      (lynx-browse-region start end)

      (set-buffer lynx-browse-buffer)

      (setq html-rendered-content (buffer-substring (point-min)
                                                    (point-max))))

    (delete-region start end)
    (insert html-rendered-content)))

(defun lynx-download-url(url filename)
  "Download a URL and save it to a file."

  (w3-download-url url filename))

(defun lynx-browse-url-with(browser &optional url)
  "Browse the given URL with the given browser and url."

  (if (null url)
      (setq url (elt lynx-url-info 0)))

  (message "Browsing with %s: %s" browser url)

      ["Browse with Konqueror" (lynx-browse-url-with 'konqueror) t]
      ["Browse with Mosaic" (lynx-browse-url-with 'mosaic) t]
      ["Browse with Mozilla" (lynx-browse-url-with 'mozilla) t]
      ["Browse with Netscape" (lynx-browse-url-with 'netscape) t]
      ["Browse with W3" (lynx-browse-url-with 'w3) t]

  (case browser

      ('konqueror
       (browse-url-konqueror url))

      ('mosaic
       (browse-url-mosaic url))

      ('mozilla
       (browse-url-mozilla url))

      ('mozilla
       (browse-url-netscape url))

      ('w3
       (browse-url-w3 url))

      ('otherwise
       (error "Unknown browser: %s" browser))))

;;add to minor-mode
(add-to-list 'minor-mode-alist
             '(lynx-mode lynx-mode-string))

;;setup map
(let ((a (assoc 'lynx-mode minor-mode-map-alist)))
  (if a
      (setcdr a lynx-mode-map)
    (add-to-list 'minor-mode-map-alist
                 (cons 'lynx-mode lynx-mode-map))))

(provide 'lynx)
