;; -*-Emacs-Lisp-*-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This program is free software; you can redistribute it and/or         ;;
;; modify it under the terms of the GNU General Public License           ;;
;; version 2 as published by the Free Software Foundation.               ;;
;;                                                                       ;;
;; This program is distributed in the hope that it will be useful, but   ;;
;; WITHOUT ANY WARRANTY; without even the implied warranty of            ;;
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     ;;
;; General Public License for more details.                              ;;
;;                                                                       ;;
;; Written and (c) by Franois Fleuret                                   ;;
;; Contact <francois.fleuret@inria.fr> for comment & bug reports         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; mp3play v0.6.3
;;
;; A simple front-end major mode for mpg321 in remote-control
;; mode. Tested with emacs versions 21.2.1 and 21.3.5 on GNU/Linux
;; Debian, with mpg321 version 0.2.10 and aumix version 2.7
;;
;; My main concern is the CPU usage of emacs when playing a mp3
;; (between 0.5% and 2%) due to the huge quantity of "@F" lines mpg321
;; sends. We have to deal with them to be able to detect when an mp3
;; is over, so that a new one can start in automatic mode (and also to
;; detect pause and various stuff).
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; To use this mode it, put the .el file where emacs can find it
;; (which means, in a directory appearing in load-path), load it, and
;; set your list of mp3 directories. For instance, to do that, just add in
;; your .emacs:
;;
;; (load "mp3play")
;; (setq mp3play-dirlist '("~/my_mp3_dir" "/nfs/stuff/my_other_dir"))
;;
;; M-x mp3play builds a *mp3play* buffer. In this one, type 'h' for
;; help, <return> to play a song and 'q' to burry the buffer.
;;
;; I personnally also use the following shortcuts:
;;
;; ;; M-\ go to the mp3play buffer
;; (define-key global-map [(meta \\)] 'mp3play)
;; ;; C-x C-m C-s stops the current mp3
;; (define-key global-map [(control x) (control m) (control s)] 'mp3play-stop)
;; ;; C-x C-m C-p pauses it
;; (define-key global-map [(control x) (control m) (control p)] 'mp3play-pause)
;; ;; C-x C-m C-p shows mp3 informations
;; (define-key global-map [(control x) (control m) (control i)] 'mp3play-show-current-information)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar mp3play-dirlist nil "List of the directories containing the mp3s for mp3play.")
(defvar mp3play-player "mpg321" "Program to use to play mp3s.")
(defvar mp3play-cleanup-filename 'mp3play-native-cleanup-filename "The function to clean up the filenames before displaying them.")
(defvar mp3play-file-filter "\\(mp3\\|og[gm]\\)$" "Regexp used to keep files to show.")

(defvar mp3play-mode-map nil "Keymap for mp3play mode.")
(defvar mp3play-auto-mode nil "Boolean flag for the automatic playing mode.")

(make-empty-face 'mp3play-title-face)
;;(set-face-attribute 'mp3play-title-face nil :weight 'bold)
(set-face-attribute 'mp3play-title-face nil :underline t)

(make-empty-face 'mp3play-nonexisting)
(set-face-attribute 'mp3play-nonexisting nil :background "red3")

(setq mp3play-process nil ;; this is the mpg321 process
      mp3play-buffer nil ;; this is the main buffer (only one, we need to know it cf. mp3play-filter-subfunctions)
      mp3play-current-overlay nil ;; the overlay for the highlighted line (current playing tune)
      mp3play-on-air nil) ;; the name of the last tune we got a message from mpg321 about

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The interactive functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun mp3play-help () (interactive)
  (message "RET:Play  p:Pause  a:Auto-mode  h:This help  i:Show tune informations  k:Kill  q:Bury  g:Import list  \
N:next tune  P:previous tune  \
s:Stop  t:Insert title  <S-left>:Fast backward  \
<S-right>:Fast forward  <S-up>:Carry up  <S-down>:Carry down  +:Volume up  -:Volume down"))

(defun mp3play-play-current-line () "Starts mpg321 with the file on the current line and goes down." (interactive)
  (goto-char (point-at-bol))
  (if (not (re-search-forward "^FILE:\\([^\n]*\\)" (point-at-eol) t)) 
      (progn
        (message "No file on this line!")
        (move-overlay mp3play-current-overlay 0 0) ;; dirty way to hide it ...
        )
    (mp3play-play (substring-no-properties (match-string 1)))
    (goto-char (point-at-bol))
    (next-line)
    (re-search-forward "^FILE:" nil t)
    ))

(defun mp3play-play-next () "Plays the tune after the current one." (interactive)
  (save-excursion (goto-char (overlay-start mp3play-current-overlay))
                  (next-line 1)
                  (re-search-forward "^FILE:" nil t)
                  (mp3play-play-current-line)))

(defun mp3play-play-prev () "Plays the tune before the current one." (interactive)
  (save-excursion (goto-char (overlay-start mp3play-current-overlay))
                  (re-search-backward "^FILE:" nil t)
                  (mp3play-play-current-line)))

(defun mp3play-play (filename) "Starts playing a tune."
  (if (not (file-exists-p filename)) (message "Non-existing file!")
    (mp3play-send-to-mpg321 (format "LOAD %s\n" filename))
    (setq mp3play-on-air filename)
    (move-overlay mp3play-current-overlay (point-at-bol) (point-at-eol))))

(defun mp3play-auto () "Commutes the auto-mode." (interactive)
  (setq mp3play-auto-mode (not mp3play-auto-mode))
  (force-mode-line-update)
  )

(defun mp3play-stop () "Stops the currently playing tune." (interactive)
  (mp3play-send-to-mpg321 "STOP\n")
  )

(defun mp3play-pause () "Pauses the currently playing tune." (interactive)
  (mp3play-send-to-mpg321 "PAUSE\n")
  )

(defun mp3play-forward () "Goes 1000 frames forward in the current song." (interactive)
  (mp3play-send-to-mpg321 "JUMP +1000\n")
  (message "Fast forward")
  )

(defun mp3play-backward () "Goes 1000 frames backward in the current song." (interactive)
  (mp3play-send-to-mpg321 "JUMP -1000\n")
  (message "Fast rewind")
  )

(defun mp3play-move-item-up () "Move the current item one line up." (interactive)
  (transpose-regions (line-beginning-position) (1+ (line-end-position))
                     (line-beginning-position 0) (1+ (line-end-position 0))))

(defun mp3play-move-item-down () "Move the current item one line down." (interactive)
  (transpose-regions (line-beginning-position) (1+ (line-end-position))
                     (line-beginning-position 2) (1+ (line-end-position 2))))

(defun mp3play-show-current-information () (interactive)
  (if (not mp3play-on-air) (message "No song playing!")
    (message "Playing [%s] - %dkb/s %gKhz %s"
             (file-name-nondirectory mp3play-on-air)
             (string-to-number (elt mp3play-current-information 10))
             (/ (string-to-number (elt mp3play-current-information 2)) 1000.0)
             (elt mp3play-current-information 3))))

(defun mp3play-volume-up () (interactive) "Call aumix to increase the volume."
  (call-process "aumix" nil nil nil "-v" "+3")
  (message "Volume up")
  )

(defun mp3play-volume-down () (interactive) "Call aumix to decrease the volume."
  (call-process "aumix" nil nil nil "-v" "-3")
  (message "Volume down")
  )

(defun mp3play-dont-use-that-key () (interactive) (error ""))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The displaying functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun mp3play-native-cleanup-filename (str)
  "Clean up the filename by replacing the \"_\" by \" \", removing the extension and changing a few other things."
  (replace-regexp-in-string
   "\\(-\\)[^ ]" "- "
   (replace-regexp-in-string
    "[^ ]\\(-\\)" " -"
    (replace-regexp-in-string
     " *.\\(mp3\\|og[gm]\\)$" ""
     (replace-regexp-in-string
      " +" " "
      (replace-regexp-in-string "_" " " str)))
    nil nil 1)
   nil nil 1))

(defun mp3play-add-file (filename size shift already-there)

  (defun string-from-size (size)
    (if (< size 1024) (format "%4db" size)
      (if (< size 1048576) (format "%4dk" (ash size -10))
        (format "%4dM" (ash size -20)))))

  (when (and (file-regular-p filename) (string-match mp3play-file-filter filename) (not (gethash filename already-there nil)))

    (insert (format
             "FILE:%s%s%s -- %s\n"
             filename
             (make-string (* 4 shift) ?\ )
             (string-from-size size)
             (eval `(,mp3play-cleanup-filename (file-name-nondirectory filename)))))
    )

  (when (file-directory-p filename)
    (mp3play-add-directory filename (1+ shift) already-there))

  )

(defun mp3play-add-directory (dir shift already-there)

  (unless (gethash dir already-there nil)
    (goto-char (point-max))
    (insert (format "DIR:%s%s%s\n" dir (make-string (* 4 shift) ?\ ) dir)))

  (mapc (lambda (file)
          (unless (string-match "^\\." (car file))
            (mp3play-add-file (command-line-normalize-file-name (concat dir "/" (car file)))
                              (nth 8 file)
                              shift
                              already-there)))
        (directory-files-and-attributes dir))
  )

(defun mp3play-import-list () "Add all new files from mp3play-dirlist in the current buffer."

  (let ((already-there (make-hash-table :test 'equal)))

    (goto-char (point-min))

    (while (re-search-forward "^\\(FILE\\|DIR\\):\\([^\n]*\\)" nil t)
      (puthash (match-string 2) t already-there)
      (goto-char (match-end 2)))

    (message "Importing tune list ...")
    (save-excursion
      (goto-char (point-max))
      (mapc (lambda (dir) (mp3play-add-directory (expand-file-name dir) 0 already-there)) mp3play-dirlist)
      (message "done!"))
    ))

(defun mp3play-rebuild-list () "Build the buffer content from scratch" (interactive)
  (if (not (and (boundp 'mp3play-dirlist) mp3play-dirlist)) (error "Nothing in mp3play-dirlist!")
    (save-excursion
      (mp3play-import-list)
      (mp3play-put-properties)
      ))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The functions to deal with the process
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq mp3play-filter-subfunctions

      '(("P 0" . (with-current-buffer mp3play-buffer
                   (if mp3play-auto-mode
                       (save-excursion
                         (goto-char (overlay-start mp3play-current-overlay))
                         (next-line 1)
                         (re-search-forward "^FILE:" nil t)
                         (mp3play-play-current-line))
                     (move-overlay mp3play-current-overlay 0 0) ;; dirty way to hide it ...
                     (setq mp3play-on-air nil))))

        ("P 1" . (message "Song [%s] paused" (file-name-nondirectory mp3play-on-air)))

        ("P 2" . (message "Song [%s] continues" (file-name-nondirectory mp3play-on-air)))

        ("I" . (setq mp3play-currently-loaded (match-string 2 str)))

        ("S" . (progn
                 (setq mp3play-current-information (vconcat (split-string (match-string 2 str))))
                 (mp3play-show-current-information)))
        )
      )

(defun mp3play-parse (str start)
  (when (string-match "^@\\(P 0\\|P 1\\|P 2\\|S\\|I\\) *\\(.*\\)$" str start)
    (let ((next (1+ (match-end 0))))
      (eval (cdr (assoc (match-string 1 str) mp3play-filter-subfunctions)))
      (mp3play-parse str next))
    ))

;; We ignore all errors (due either to too many lines or a bad
;; handling of the last one ...)
(defun mp3play-filter (process str)
  (condition-case nil (mp3play-parse str 0) (error nil)))

(defun mp3play-sentinel (process str) ()
  (unless (eq (process-status mp3play-process) 'run) (setq mp3play-process nil))
  (message "Mp3play process got %s" (replace-regexp-in-string "\n" "" str)))

(defun mp3play-send-to-mpg321 (string)
  (unless mp3play-process
    ;; Set up the process for mpg321
    (setq mp3play-process (start-process "mp3player" nil mp3play-player "-R" "dummy"))
    (set-process-filter mp3play-process 'mp3play-filter)
    (set-process-sentinel mp3play-process 'mp3play-sentinel)
    (process-kill-without-query mp3play-process))
  (process-send-string mp3play-process string)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The settings of the major mode itself
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun mp3play-kill-cleanup () (interactive)
  (if mp3play-process (delete-process mp3play-process))
  (setq minor-mode-alist (remove '(:eval (mp3play-minor-mode-string)) minor-mode-alist)
        mp3play-process nil
        mp3play-buffer nil
        mp3play-current-overlay nil
        mp3play-on-air nil))

;; This mode is not supposed to be used on two buffers at once, as the
;; function dealing with the process have to manipulate the buffer
;; from time to time.

(defun mp3play-mode ()
  "Major mode to play songs with mpg321 (or mpg123)."
  (interactive)

  (when mp3play-buffer (error "Can not have two mp3play-buffers at once!"))

  (kill-all-local-variables)

  (unless mp3play-mode-map

    (setq mp3play-mode-map (make-sparse-keymap))
    (suppress-keymap mp3play-mode-map)

    (mapc (lambda (x) (define-key mp3play-mode-map (car x) (cdr x)))
          `(("p" . mp3play-pause)
            (,(kbd "RET") . mp3play-play-current-line)
            (" " . mp3play-play-current-line)
            ("N" . mp3play-play-next)
            ("P" . mp3play-play-prev)
            ("q" . bury-buffer)
            ("k" . kill-this-buffer)
            ("s" . mp3play-stop)
            ("a" . mp3play-auto)
            ("g" . mp3play-rebuild-list)
            ("d" . mp3play-delete-title)
            ("h" . mp3play-help)
            ("?" . mp3play-help)
            ("i" . mp3play-show-current-information)
            ([(shift up)] . mp3play-move-item-up)
            ([(shift down)] . mp3play-move-item-down)
            ([(shift left)] . mp3play-backward)
            ([(shift right)] . mp3play-forward)
            ("+" . mp3play-volume-up)
            ("-" . mp3play-volume-down)
            ([(control j)] . mp3play-dont-use-that-key)
            ([(control d)] . mp3play-dont-use-that-key)
            (,(kbd "DEL") . mp3play-dont-use-that-key)))
    )

  (use-local-map mp3play-mode-map)
  (add-hook 'kill-buffer-hook 'mp3play-kill-cleanup nil t)
  (setq mode-name "Mp3play")
  (setq mp3play-current-overlay (make-overlay 0 0)) ;; dirty way to hide it ...
  (overlay-put mp3play-current-overlay 'face 'highlight)

  (setq minor-mode-alist (cons '(:eval (mp3play-minor-mode-string)) minor-mode-alist)
        major-mode 'mp3play-mode)

  (setq mp3play-buffer (current-buffer))

  (mp3play-put-properties)

  )

;; Hide what has to be hidden, put the titles in bold, put back the
;; overlay on the current mp3's line, and put the cursor on it
(defun mp3play-put-properties ()

  (let ((m (buffer-modified-p)))

    ;; First, we narrow so that the COMMENT at the beginning can not
    ;; be altered
    (goto-char (point-min))
    (while (re-search-forward "^COMMENT.*\n" nil t) (goto-char (match-end 0)))
    (narrow-to-region (point) (point-max))

    ;; Then we hide all line beginning (before the ^_)
    (goto-char (point-min))
    (while (re-search-forward "^\\([^\n]*\\)[^\n]*$" nil t)
      (add-text-properties (match-beginning 1) (match-end 1) '(invisible t))
      (goto-char (match-end 0)))

    ;; We turn the titles into bold
    (goto-char (point-min))
    (while (re-search-forward "^DIR:\\([^\n]*\\) *\\(.*\n\\)" nil t)
      (add-text-properties (match-beginning 2) (match-end 2) '(face mp3play-title-face))
      (goto-char (match-end 0)))

    ;; Highlight the non-existing file
    (goto-char (point-min))
    (while (re-search-forward "^FILE:\\([^\n]*\\)\\(.*\n\\)" nil t)
      (unless (file-exists-p (match-string 1))
        (add-text-properties (match-beginning 2) (match-end 2) '(face mp3play-nonexisting)))
      (goto-char (match-end 0)))

    ;; Look for the current file, put the overlay there
    (goto-char (point-min))
    (when (and mp3play-on-air (re-search-forward (format "^FILE:%s.*$" mp3play-on-air) nil t))
      (move-overlay mp3play-current-overlay (match-beginning 0) (match-end 0))
      )

    (set-buffer-modified-p m))
  )

(defun mp3play-minor-mode-string ()
  (concat
   (if mp3play-on-air " Playing" " Paused")
   (if mp3play-auto-mode "/auto"))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The main function
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq mp3play-header "COMMENT: ;;-*-mp3play-*-\n")

(defun mp3play ()

  "Switches to the mp3play buffer, creates it if necessary. This
buffer contains the list of all the mp3s from the directories
listed in `mp3play-dirlist'. You can invoke mpg321 on a given mp3
by typing <return> on the mp3 in this buffer. Use 'q' to burry
the buffer, and 'k' to kill it."

  (interactive)

  ;; If we already have a mp3play-buffer, we switch to it, if not,
  ;; we create one

  (if mp3play-buffer (switch-to-buffer mp3play-buffer)
    ;; Set up the buffer, and switch to it
    (switch-to-buffer (get-buffer-create "*mp3play*"))
    ;; Put the "-*-mp3play-*-" and stuff
    (insert mp3play-header)
    ;; Get all the mp3 names
    (mp3play-import-list)
    ;; Set the major mode
    (mp3play-mode)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

