;;; newsticker.el --- A Newsticker for Emacs.
;;
;;  Copyright (C) 2003 by Ulf Jasper
;;
;;  This file is NOT part of GNU Emacs.
;;
;;  Author:      Ulf Jasper <ulf.jasper@web.de>
;;  Filename:    newsticker.el
;;  Created:     17. August 2003
;;  Keywords:    News
;;  Time-stamp:  "19. Juni 2003, 21:57:49 (ulf)"
;;  CVS-Version: $Id: newsticker.el,v 1.5 2003/06/19 19:57:52 ulf Exp $

(defconst newsticker-version 0.91 "Version number of newsticker.el.")

;; ======================================================================
;;
;;  This program is free software; you can redistribute it and/or
;;  modify it under the terms of the GNU General Public License as
;;  published by the Free Software Foundation; either version 2 of the
;;  License, or (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful, but
;;  WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;  General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License along
;;  with this program; if not, write to the Free Software Foundation, Inc.,
;;  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;;
;; ======================================================================
;;; Commentary:
;; 
;;  This package requires wget.
;;
;;  This package provides a newsticker. A newsticker is a thing that
;;  asynchronously retrieves a list of headlines (which are contained in
;;  RDF files) from a list of news sites, displays these headlines, and
;;  allows for loading the corresponding articles in a web browser.
;;
;; ======================================================================
;;; Usage:
;;
;;  Add the following line to your Emacs startup file (`~/.emacs'):
;;    (require 'newsticker)
;;  or
;;    (autoload 'newsticker-start "newsticker" "Emacs Newsticker" t)
;;
;;  Then say M-x `newsticker-start'. This will retrieve headlines from each
;;  url in `newsticker-url-list' every `newsticker-retrieval-interval'
;;  seconds. All headlines are placed in the buffer
;;  `*newsticker*'. Clicking mouse-button 2 or pressing RET in that
;;  buffer on a headline will call `browse-url' to load the
;;  corresponding news story in your favourite web browser. When mouse
;;  or point enters a headline the message buffer will show the
;;  description of that headline.
;;
;;  Every `newsticker-display-interval' a headline is shown in the
;;  message buffer (if `newsticker-display-interval' is positive).
;;
;;  In order to stop the newsticker say M-x `newsticker-stop'. This will
;;  stop all timers.
;;
;; ======================================================================
;;; History:
;;  
;;  0.91 First bugfix: *newsticker* is read-only
;;
;;  0.9  First release
;;       Tested with Emacs 21.3.2 and wget 1.8.2
;;
;; ======================================================================
;;; To Do:
;;
;;  * Scramble news items for the ticker?
;;
;; ======================================================================
;;; Code:

(require 'xml)

;; ======================================================================
;; Customizables
;; ======================================================================
(defgroup newsticker nil
  "Newsticker")

(defcustom newsticker-url-list
  '(("heise" "http://www.heise.de/newsticker/heise.rdf")
    ("tagesschau" "http://www.tagesschau.de/newsticker.rdf")
    ("slashdot" "http://slashdot.org/slashdot.rdf")
    ("wiki" "http://www.emacswiki.org/cgi-bin/wiki.pl?action=rss"))
  "Newsticker url list..."
  :type '(alist :key-type (string :tag "Label")
		:value-type (group (string :tag "URL")))
  :group 'newsticker)

(defcustom newsticker-display-interval
  10
  "Time interval for displaying news items (seconds).
If equal or less than 0 no messages are shown."
  :type 'integer
  :group 'newsticker)

(defcustom newsticker-retrieval-interval
  3600
  "Time interval for retrieving new news items (seconds).
Slashdot will ban you if you make it less than 1800 seconds (30 minutes)!"
  :type 'integer
  :group 'newsticker)

(defcustom newsticker-wget-arguments
  '("-q" "-O" "-")
  "Arguments which are passed to wget."
  :type '(repeat (string :tag "Argument"))
  :group 'newsticker)

;; ======================================================================
;; Internal stuff -- keep out!
;; ======================================================================
(defvar newsticker-display-timer nil "Timer for newsticker display.")
(defvar newsticker-retrieval-timer nil "Timer for newsticker news retrieval.")
(defvar newsticker-item-list nil "List of newsticker items.")
(defvar newsticker-item-position 0
  "Actual position in list of newsticker items.")

(defun newsticker-get-all-news ()
  "Launches retrieval of news from all configured newsticker sites."
  (interactive)
  (setq newsticker-item-list (list))
  (set-buffer (get-buffer-create "*newsticker*"))
  (let ((inhibit-read-only t))
    (set-buffer-modified-p nil)
    (erase-buffer)
    (set-buffer-modified-p nil))
  (setq buffer-read-only t)
  (mapcar 'newsticker-get-news newsticker-url-list))

(defun newsticker-get-news (name-and-url)
  "Gets news from a news site.
Argument NAME-AND-URL must be a list of the form (name . url)."
  (let ((buffername (concat "*wget-newsticker-" (car name-and-url) "*")))
    (save-current-buffer
      (set-buffer (get-buffer-create buffername))
      (erase-buffer)
      (let* ((args (append newsticker-wget-arguments (list (cadr name-and-url))))
	     (proc (apply 'start-process (car name-and-url) buffername
			   "wget" args)))
	(set-process-sentinel proc 'newsticker-sentinel)))))
  
(defun newsticker-click-1 (event)
  "Callback for click EVENTs in the newsticker buffer."
  (interactive "e")
  (save-current-buffer
    (set-buffer (window-buffer (posn-window (event-end event))))
    (message (get-text-property (posn-point (event-end event)) 'desc))))

(defun newsticker-click-2 (event)
  "Callback for click EVENTs in the newsticker buffer."
  (interactive "e")
  (save-current-buffer
    (set-buffer (window-buffer (posn-window (event-end event))))
    (browse-url (get-text-property (posn-point (event-end event)) 'link))))

(defun newsticker-callback-return ()
  "Callback for return in the newsticker buffer."
  (interactive "")
  (let ((url (get-text-property (point) 'link)))
    (when url
      (browse-url url))))

(defun newsticker-callback-enter (&optional line col)
  "Callback for return in the newsticker buffer."
  (interactive "")
  (message (get-text-property (point) 'desc)))

(defun newsticker-sentinel (process event)
  "Sentinel for extracting news titles from an RDF buffer.
Argument PROCESS is the process which has just changed its state.
Argument EVENT tells what has happened to the process."
  (let ((node-list (save-current-buffer
		     (set-buffer (process-buffer process))
		     (xml-parse-region (point-min) (point-max))))
	(name (process-name process))
	(map (make-sparse-keymap))
	(pos 0))
    (set-buffer (get-buffer "*newsticker*"))
    (define-key map [mouse-1] 'newsticker-click-1)
    (define-key map [mouse-2] 'newsticker-click-2)
    (local-set-key [return] 'newsticker-callback-return)
    (let ((inhibit-read-only t))
      (save-excursion
	(goto-char (point-max))
	(setq pos (point))
	(insert name " (" (current-time-string) ")")
	(insert "\n" (make-string (- (point) pos) ?=) "\n")
	(mapcar (lambda (node)
		  (when (eq (xml-node-name node) 'item)
		    (let ((title (car (xml-node-children
				       (car (xml-get-children
					     node 'title)))))
			  (link (car (xml-node-children
				      (car (xml-get-children
					    node 'link)))))
			  (desc (car (xml-node-children
				      (car (xml-get-children
					    node 'description)))))
			  (pos 0))
		      (add-to-list 'newsticker-item-list
				   (concat name ": " title) t)
		      (insert " ")
		      (setq pos (point))
		      (insert title)
		      (add-text-properties pos (point) '(mouse-face highlight))
		      (add-text-properties pos (point) (list 'link link))
		      (add-text-properties pos (point) (list 'desc desc))
		      (add-text-properties pos (point) (list 'keymap map))
		      (add-text-properties pos (point) (list 'point-entered 
							     'newsticker-callback-enter))
		      (insert "\n"))))
		(xml-get-children (car node-list) 'item))
	(insert "\n"))
      (set-buffer-modified-p nil))))

(defun newsticker-display-tick ()
  "Called from the display timer.
This function displays the next ticker item in the message buffer."
  (setq newsticker-item-position (1+ newsticker-item-position))
  (when (>= newsticker-item-position (length newsticker-item-list))
    (setq newsticker-item-position 0))
  (message (nth newsticker-item-position newsticker-item-list)))

(defun newsticker-retrieval-tick ()
  "Called from the retrieval timer.
This function retrieves new news."
  (message "Getting new news")
  (newsticker-get-all-news))

(defun newsticker-start ()
  "Start the newsticker.
Starts the timers for display and retrieval."
  (interactive)
  (if newsticker-retrieval-timer
      (message "Ooops! Another newsticker is running!")
    (message "Newsticker started!")
    (newsticker-get-all-news)
    (if (> newsticker-display-interval 0)
	(setq newsticker-display-timer
	      (run-at-time newsticker-display-interval newsticker-display-interval
			   'newsticker-display-tick)))
    (setq newsticker-retrieval-timer
	  (run-at-time newsticker-retrieval-interval
		       newsticker-retrieval-interval
		       'newsticker-retrieval-tick))))

(defun newsticker-stop ()
  "Stops the newsticker.
Cancels the timers for display and retrieval."
  (interactive)
  (when newsticker-display-timer
    (cancel-timer newsticker-display-timer)
    (setq newsticker-display-timer nil))
  (when newsticker-retrieval-timer
    (message "Newsticker stopped!")
    (cancel-timer newsticker-retrieval-timer)
    (setq newsticker-retrieval-timer nil)))

(provide 'newsticker)

;;; newsticker.el ends here


