;;; org.el --- Carsten's outline mode for keeping track of everything.
;; Copyright (c) 2003 Carsten Dominik
;;
;; Author: Carsten Dominik <dominik@science.uva.nl>
;; Keywords: outlines, hypermedia, calendar
;; Version: 1.3
;;
;; This file is not part of GNU Emacs
;;
;; This program is free software you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Commentary:
;;
;; Org-mode is a mode for keeping notes and doing project planning
;; with a fast and effective plain-text system.
;;
;; Org-mode is implemented on top of outline-mode.  Outlines allow to
;; organize a document in a hierarchical structure.  Overview over this
;; structure is achieved by folding (hiding) large parts of the document
;; to show only the general document structure and the parts currently
;; being worked on.  Org-mode greatly simplifies the use of outlines by
;; compressing the entire show/hide functionality into a single command
;; `org-cycle' which is bound to the TAB key.
;;
;; In addition to outline support, org-mode implements TODO items,
;; deadlines, links to files and web locations, and exporting to ASCII
;; and HTML.
;;
;; Installation
;; ------------
;; Byte-compile org.el and put it on your load path.  Then copy the
;; following lines into .emacs
;;
;;     (autoload 'org-mode "org" "Org mode" t)
;;     (add-to-list 'auto-mode-alist '("\\.org$" . org-mode))
;;
;; This will put all files with extension ".org" into org-mode.  As an
;; alternative, make the first line of a file look like this:
;;
;;     MY PROJECTS    -*- mode: org; -*-
;;
;; which will enforce org-mode for this buffer no matter what the file's
;; name is.  In order to facilitate global cycling (see below under TAB),
;; the first line of a file should not be a headline.
;;
;; Headlines
;; ---------
;; The Headlines in org-mode start with one or more stars, for example
;;
;; * Top level headline
;; ** Second level
;; *** 3rd level
;;     some text
;; *** 3rd level
;;     more text
;; * Another top level headline
;;
;; Motion
;; ------
;; The following commands jump to other headlines in the buffer
;;
;; C-c C-n     next heading
;; C-c C-p     previous heading
;; C-c C-f     next heading same level
;; C-c C-b     previous heading same level
;; C-c C-u     backward to higher level heading
;;
;; Visibility cycling
;; ------------------
;; Outline mode makes it possible to hide parts of the text in the
;; buffer. Org-mode uses a single command bound to the TAB key to change
;; the visibility in the buffer.
;;
;; TAB         Rotate current subtree between the states
;;                  FOLDED -> CHILDREN -> SUBTREE
;;             At the beginning of the buffer (or when called with C-u),
;;             rotate the entire buffer between the states
;;                  OVERVIEW -> CONTENTS -> SHOW ALL
;; C-c C-a     Show all
;;
;; Structure editing
;; -----------------
;; M-RET       Insert new heading with same level as current
;; M-left      Promote current heading by one level
;; M-right     Demote current level by one level
;; M-S-left    Promote the current subtree by one level
;; M-S-right   Demote the current subtree by one level
;; M-up        Move subtree up (swap with previous heading of same level)
;; M-down      Move subtree down (swap with next heading of same level)
;;
;; When there is an active region (transient-mark-mode), promotion and
;; demotion works on all headlines in the region.
;;
;; Time stamps
;; -----------
;; C-c .       Insert a date stamp at point.  If there is already a
;;             stamp in the current line, update it to current date.
;; C-u C-c .   Use the alternative time format (date and time).
;;
;; TODO items
;; ----------
;; Any headline can become a TODO item by starting it with the word TODO.
;;
;; C-c C-t     Rotate the TODO state of the current item between
;;                 (UNMARKED) -> TODO -> DONE
;; C-c C-v     View TODO items.  Folds the entire buffer, but shows all
;;             TODO items and the headings hierarchy above them.
;; C-c /       Occur.  Prompts for a regexp and shows a compact tree with
;;             all matches, in a similar way as the TODO tree.
;;
;; Deadlines
;; ---------
;; A DEADLINE keyword identifies things which have to be finished by a
;; certain date.
;;
;; C-c C-d     Insert the DEADLINE keyword, along with the current date.
;;             You can use the keys S-up and S-down to adjust the date by
;;             one day, or S-left and S-right to adjust it by one month.
;;             If there is already a DEADLINE in the current line, toggle
;;             it on and off.
;; C-c C-w     Show all deadlines which are past-due, or which will become
;;             due within `org-deadline-warning-days'.  With C-u prefix,
;;             show all deadlines in the file.  With a numeric prefix,
;;             check that many days.  For example, C-1 C-c C-w shows
;;             all deadlines due tomorrow.
;;
;; Links
;; -----
;; Org-mode supports links to files and to websites in the document.
;; Links are just plain-text URL-like locators.  Examples:
;;
;;   http://zon.astro.uva.nl/~dominik
;;   file:/home/dominik/images/jupiter.jpg       ; absolute path
;;   file:papers/last.pdf                        ; relative path
;;   news:comp.emacs                             ; Usenet link
;;   mailto:adent@galaxy.net                     ; Mail link
;;
;; C-c C-l     Insert a link to a file, with file name completion.  If
;;             the file is located in the current directory or below, the
;;             path will be relative.  Otherwise, the path will be as
;;             completed in the minibuffer.  With prefix argument, an
;;             absolute path can be forced.
;; C-c C-o     Open link at point.  This will launch a webbrowser for
;;             URLs (using `browse-url-at-point'), and a suitable
;;             application for non-text files.  Classification of files
;;             is based on file extension only.  Text files and unknown
;;             file types will be visited by an Emacs buffer. See option
;;             `org-file-viewers'.  If there is no link at point, the
;;             current subtree will be searched for one.
;;             When the file does not exist, an error results.  Use a
;;             C-u prefix do enforce launching of the application anyway.
;; mouse-2     On links, mouse-2 will open the link just like
;;             `C-c C-o' would.
;; C-c :       Search BBDB database for word under cursor.
;;
;; Exporting
;; ---------
;; For printing and sharing of notes, an org-mode document can be
;; exported as an ASCII file, or as HTML.  In the exported version,
;; the first 3 outline levels will become headlines, defining a
;; general document structure.  Additional levels will be exported as
;; itemize lists.  If you want that transition to occur at a different
;; level, specify a prefix argument.  For example,
;;             M-1 M-x org-export-as-html
;; creates only top level headlines and does the rest as items.
;;
;; C-c C-x a    Export as ASCII file.  If there is an active region,
;;              only the region will be exported.  For an org file
;;              `myfile', the ASCII file will be `myfile.txt'.
;; C-c C-x h    Export as HTML file `myfile.html'.
;; C-c C-x C-h  Export as HTML file and open it with a browser.
;; C-c C-x t    Insert template with HTML options, see below.
;;
;; If you want to include HTML tags which should be interpreted as
;; such, quote them with a "@" like in @<b>bold text@</b>.  Plain <
;; and > are always transformed to &lt; and &gt;
;; 
;; The HTML exporter can also be configured to do additional conversions.
;; However, these conversions are turned off by default - you need to
;; configure the appropriate variables.
;;  - Stars around a word can indicate *emphasized* text.
;;  - TeX-like constructs can be used for simple math:
;;     - \alpha can be converted to &alpha; etc
;;     - 10^{22} and J_n can be converted to super- and subscripts.
;;  - Lines starting with "|" can be typeset in a fixed-width font. This
;;    is useful for preformatted material, tables etc.
;;
;; The HTML exporter also recognizes special lines in the buffer which
;; can be used to provide additional information.  These lines should
;; be put before the first headline in the file.
;;
;;   +TITLE:     the title to be shown (default is the buffer name)
;;   +AUTHOR:    the author (default taken from `user-full-name')
;;   +EMAIL:     his/her email address (default from `user-mail-address')
;;   +TEXT:      Some descriptive text to be inserted at the beginning.
;;   +TEXT:      Several lines may be given.
;;
;; Comment lines
;; -------------
;; Lines starting with "#" in column zero are treated as comments and
;; will never be exported.  Also entire subtrees starting with the
;; word COMMENT will never be exported.
;;
;; C-c ;        Toggle the COMMENT keyword at the beginning of an entry.
;;              Subtrees with this keyword will never be exported.
;;
;; Picture mode
;; ------------
;; Simple ASCII drawings can be made in picture-mode.  If you'd like
;; to use it, set the variable `org-enable-picture-mode'.  Then you can
;; use C-c C-c to toggle picture mode on and off.  See picture-mode
;; documentation for details.  Some additional bindings are provided
;; by org-mode:
;;
;;      M-up         M-7 M-8 M-9  \
;; M-left  M-right   M-u     M-o   }  Draw lines in keypad-like directions
;;     M-down        M-j M-k M-o  /
;;
;;    M--              Draw line from mark to point, set mark at end
;;    S-mouse1         Freehand drawing with the mouse
;;
;; Tips and Tricks
;; ---------------
;; - Paste URLs into org-mode whenever this seems useful. For example, if
;;   you are writing notes about a paper which is available on the web,
;;   put the corresponding URL there and a direct look at the paper is
;;   only a mouseclick away.  If you have a local copy of the paper, use
;;   a file:path link.
;;
;; - If you plan to use ASCII or HTML export, make sure things you want
;;   to be exported as item lists are level 4 at least, even if that does
;;   mean there is a level jump.  For example
;;
;;     * Todays top priorities
;;     **** TODO write a letter to xyz
;;     **** TODO Finish the paper
;;     **** Pick up kids at the school
;;
;; - Alternatively, if you need a specific value for the heading/item
;;   transition in a particular file, use a file variable to set it, e.g.
;;   with this first line in a file:
;;
;;        -*- mode: org; org-export-headline-levels: 2 -*-
;;
;; Customization
;; -------------
;; There is a number of options through which org-mode can be
;; configured.  Below is a list of variables which you can set with
;; Lisp in .emacs, or through customize.  Should you need file-local
;; settings of any options, use file variables, for example by putting
;; near the end of the file something like:
;;
;;    # Local Variables: ***
;;    # mode:org ***
;;    # org-startup-with-deadline-check:nil ***
;;    # org-export-headline-levels:2 ***
;;    # org-export-html-with-toc:nil ***
;;    # End: ***
;;
;; General customization variables
;; -------------------------------
;; browse-url-browser-function
;;   Function to display a url in a WWW browser.
;;
;; org-file-apps
;;   The applications opening `file:path' links.
;;
;; org-time-stamp-formats
;;   Formats of the date and time stamps used with `org-time-stamp'.
;;
;; org-startup-folded
;;   Non-nil means, entering ORG mode will switch to OVERVIEW.
;;
;; org-startup-truncated
;;   Non-nil means, entering ORG mode will set `truncate-lines'.
;;
;; org-startup-with-deadline-check
;;   Non-nil means, entering ORG mode will run the deadline check.
;;
;; org-deadline-warning-days
;;   No. of days before expiration during which a deadline becomes active.
;;
;; org-enable-picture-mode
;;   Non-nil means, allow C-c C-c to switch to picture mode.
;;
;; org-adapt-indentation
;;   Non-nil means, adapt indentation when promoting and demoting.
;;
;; org-cycle-emulate-tab
;;   Where should `org-cycle' emulate TAB?
;;
;; Customization variables for export
;; ----------------------------------
;; org-export-headline-levels
;;   Number of levels to be exported as headlines (instead of items).
;;
;; org-export-with-section-numbers
;;   Non-nil means, add section numbers to headlines when exporting.
;;
;; org-export-ascii-show-new-buffer
;;   Non-nil means, popup buffer containing the exported ascii text.
;;
;; org-export-html-show-new-buffer
;;   Non-nil means, popup buffer containing the exported html text.
;;
;; org-export-html-with-toc
;;   Non-nil means, create a table of contents in the HTML output.
;;
;; org-export-html-preserve-breaks
;;   Non-nil means, preserve all line breaks when exporting.
;;
;; org-export-html-expand
;;   Non-nil means, for HTML export treat @<...> as HTML tag
;;
;; org-export-html-with-fixed-width
;;   Non-nil means, lines starting with "|" will be in fixed width font.
;;
;; org-export-html-with-sub-superscripts
;;   Non-nil means, interprete "_" and "^" for HTML export.
;;
;; org-export-html-with-emphasize
;;   Non-nil means, interprete *word* as <i>word</i> for HTML export.
;;
;; org-export-html-with-TeX-macros
;;   Non-nil means, convert simple TeX macros like \alpha to HTML &alpha;
;;
;; Acknowledgements
;; ----------------
;; Thanks to Juergen Vollmer for code generating the table of contents
;; in HTML output.
;;==========================================================================
;;
;;; Still to do in this mode:
;;
;;; Code:

(require 'outline)
(require 'time-date)

;; Customization variables

(defgroup org nil
  "Carsten's outline mode for keeping track of everything."
  :tag "Org"
  :group 'outlines
  :group 'hypermedia
  :group 'calendar)

(defcustom org-startup-folded t
  "Non-nil means, entering ORG mode will switch to OVERVIEW."
  :group 'org
  :type 'boolean)

(defcustom org-startup-truncated nil
  "Non-nil means, entering ORG mode will set `truncate-lines'."
  :group 'org
  :type 'boolean)

(defcustom org-startup-with-deadline-check t
  "Non-nil means, entering ORG mode will run the deadline check."
  :group 'org
  :type 'boolean)

(defcustom org-deadline-warning-days 7
  "No. of days before expiration during which a deadline becomes active."
  :group 'org
  :type 'number)

(defcustom org-adapt-indentation t
  "Non-nil means, adapt indentation when promoting and demoting.
When this is set and the *entire* text in an entry is indented, the
indentation is increased by one space in a demotion command, and
decreased by one in a promotion command.  If any line in the entry
body starts at column 0, indentation is not changed at all."
  :group 'org
  :type 'boolean)

(defcustom org-cycle-emulate-tab t
  "Where should `org-cycle' emulate TAB.
nil    Never
white  Only in completely white lines
t      Everywhere except in headlines"
  :group 'org
  :type '(choice (const :tag "Never" nil)
		 (const :tag "Only in completely white lines" white)
		 (const :tag "Everywhere except in headlines" t)
		 ))

(defconst org-file-apps-defaults-linux
  '(("jpg"    . "xv %s")
    ("gif"    . "xv %s")
    ("ppm"    . "xv %s")
    ("pgm"    . "xv %s")
    ("pbm"    . "xv %s")
    ("tif"    . "xv %s")
    ("png"    . "xv %s")
    ("ps"     . "gv %s")
    ("ps.gz"  . "gv %s")
    ("eps"    . "gv %s")
    ("eps.gz" . "gv %s")
    ("dvi"    . "xdvi %s")
    ("mpeg"   . "plaympeg %s")
    ("mp3"    . "plaympeg %s")
    ("fig"    . "xfig %s")
    ("pdf"    . "acroread %s")
    ("doc"    . "soffice %s")
    ("ppt"    . "soffice %s")
    ("pps"    . "soffice %s")
    ("html"   . "netscape -remote openURL(%s,new-window)")
    ("htm"    . "netscape -remote openURL(%s,new-window)")
    ("xs"     . "soffice %s"))
  "Default file applications on a UNIX/LINUX system.
See `org-file-apps'.")

(defconst org-file-apps-defaults-macosx
  '(("jpg"    . "xv %s")
    ("gif"    . "xv %s")
    ("ppm"    . "xv %s")
    ("pgm"    . "xv %s")
    ("pbm"    . "xv %s")
    ("tif"    . "xv %s")
    ("png"    . "xv %s")
    ("ps"     . "gv %s")
    ("ps.gz"  . "gv %s")
    ("eps"    . "gv %s")
    ("eps.gz" . "gv %s")
    ("dvi"    . "xdvi %s")
    ("mpeg"   . "open %s")
    ("mp3"    . "open %s")
    ("fig"    . "xfig %s")
    ("pdf"    . "open %s")
    ("doc"    . "open %s")
    ("ppt"    . "open %s")
    ("pps"    . "open %s")
    ("html"   . "open %s")
    ("htm"    . "open %s")
    ("xs"     . "open %s"))
  "Default file applications on a MacOS X system.
See `org-file-apps'.")

(defcustom org-file-apps '(("txt" . "emacs %s"))
  "External applications for opening `file:path' items in a document.
Org-mode has a default list of applications for different files, but
you can use this variable to add or change entries.  The entries in
this list are cons cells with a file extension and the corresponding
command.  %s in the command will be replaced by the path to the file.
For example (\"txt\" . \"emacs %s\") as an entry would launch a new
emacs for any .txt files. For more examples, see the constants
`org-file-apps-defaults-macosx'
`org-file-apps-defaults-linux'.
As a special case, if command is an empty string, it means to ask the
current Emacs process to visit this file.  This can be used to overrule
settings in the `apps-defaults' constants."
  :group 'org
  :type '(repeat
          (cons (string :tag "Extension") (string :tag "Command"))))

(defcustom org-time-stamp-formats '("<%a %d %b %Y>" . "<%Y-%m-%d %a %H:%M>")
  "Formats for `format-time-string' which are used for time stamps.
The formats must be fixed length, and must start with \"<\" and
end with \">\".  The command `org-time-stamp' uses these formats to
insert and update time stamps.  To work with deadlines, org-mode must
be able to parse the first time format - so only change this to
something `parse-time-string' can still handle.  For the meaning of
the % escapes in the format, see the documentation of
`format-time-string'."
  :group 'org
  :type '(cons string string))

(defcustom org-enable-picture-mode nil
  "Non-nil means, C-c C-c switches to picture mode."
  :group 'org
  :type 'boolean)

(defgroup org-export nil
  "Carstens outline mode for keeping track of everything."
  :tag "Org-export"
  :group 'org)

(defcustom org-export-headline-levels 3
  "The last level which is still exported as a headline.
Inferior levels will produce itemize lists when exported.
Note that a numeric prefix argument to an exporter function overrides
this setting."
  :group 'org-export
  :type 'number)

(defcustom org-export-with-section-numbers t
  "Non-nil means, add section numbers to headlines when exporting."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-ascii-show-new-buffer t
  "Non-nil means, popup buffer containing the exported ascii text.
Otherwise the buffer will just stay hidden."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-show-new-buffer nil
  "Non-nil means,  popup buffer containing the exported html text.
Otherwise, the buffer will just be saved to a file."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-with-toc t
  "Non-nil means, create a table of contents in HTML export."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-preserve-breaks nil
  "Non-nil means, preserve all line breaks when exporting."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-expand t
  "Non-nil means, for HTML export, treat treat @<...> as HTML tag."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-with-fixed-width nil
  "Non-nil means, lines starting with \"|\" will be in fexed width font.
This can be used to have preformatted text, fragments of code etc.  For
example
  | Name       | Address  | Birthday
  | Athur Dent | England  | 29.2.2100
will be looking just like this in HTML."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-with-sub-superscripts nil
  "Non-nil means, interprete \"_\" and \"^\" for HTML export."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-with-emphasize nil
  "Non-nil means, interprete *word* as emphasize for HTML export."
  :group 'org-export
  :type 'boolean)

(defcustom org-export-html-with-TeX-macros nil
  "Non-nil means, convert single TeX-like macros to HTML equivalents.
For example, convert \\alpha to &alpha;"
  :group 'org-export
  :type 'boolean)

(defvar zmacs-regions)
(defvar org-transient-mark-mode)
(defvar levels-open)

;; Define the mode

(defvar org-mode-menu-bar-map)
(define-derived-mode org-mode outline-mode "Org"
  "Carstens special outline mote for keeping notes about everything."
  (setq outline-regexp "\\*+")
  (if org-startup-folded (org-cycle t))
  (if org-startup-truncated (setq truncate-lines t))
  ;; Hack the old menu stuff out of the map
  (setq org-mode-map (delq (assoc 'menu-bar (cdr org-mode-map))
                           org-mode-map))
  (set (make-local-variable 'font-lock-unfontify-region-function)
       'org-unfontify-region)
  ;; And put in our own map
  (define-key org-mode-map [menu-bar] org-mode-menu-bar-map)
  (if org-startup-with-deadline-check
      (call-interactively 'org-check-deadlines)))


(defun org-beginning-of-line ()
  "Goto the beginning of the current line.  If that is invisible, continue
to a visible line beginning.  This make the function of C-a more intuitive."
  (interactive)
  (beginning-of-line 1)
  (backward-char 1)
  (if (outline-invisible-p)
      (while (outline-invisible-p)
        (backward-char 1)
        (beginning-of-line 1))
    (forward-char 1)))

(defun org-region-active-p ()
  "Is transient-mark-mode on and the region active?
Works on both Emacs and XEmacs."
  (if (featurep 'xemacs)
      (and zmacs-regions (region-active-p))
    (and transient-mark-mode mark-active)))

;; Visibility cycling

(defvar org-cycle-global-status nil)
(defvar org-cycle-subtree-status nil)
(defun org-cycle (&optional arg)
  "Visibility cycling for outline(-minor)-mode.

- When this function is called with a prefix argument, rotate the entire
  buffer through 3 states (global cycling)
  1. OVERVIEW: Show only top-level headlines.
  2. CONTENTS: Show all headlines of all levels, but no body text.
  3. SHOW ALL: Show everything.

- When point is at the beginning of a headline, rotate the subtree started
  by this line through 3 different states (local cycling)
  1. FOLDED:   Only the main headline is shown.
  2. CHILDREN: The main headline and the direct children are shown.  From
               this state, you can move to one of the children and
               zoom in further.
  3. SUBTREE:  Show the entire subtree, including body text.

- When point is not at the beginning of a headline, execute
  `indent-relative', like TAB normally does.  See the option
  `org-cycle-emulate-tab' for details.

- Special case: if point is the the beginning of the buffer and there is
  no headline in line 1, this function will act as if called with prefix arg."
  (interactive "P")

  (if (and (bobp) (not (looking-at outline-regexp)))
      ; special case:  use global cycling
      (setq arg t))

  (cond
   (arg ;; Global cycling

    (cond
     ((and (eq last-command this-command)
	   (eq org-cycle-global-status 'overview))
      ;; We just created the overview - now do table of contents
      ;; This can be slow in very large buffers, so indicate action
      (message "CONTENTS...")
      (save-excursion
	;; Visit all headings and show their offspring
	(goto-char (point-max))
	(catch 'exit
	  (while (and (progn (condition-case nil
				 (outline-previous-visible-heading 1)
			       (error (goto-char (point-min))))
			     t)
		      (looking-at outline-regexp))
	    (show-branches)
	    (if (bobp) (throw 'exit nil))))
	(message "CONTENTS...done"))
      (setq org-cycle-global-status 'contents))
     ((and (eq last-command this-command)
	   (eq org-cycle-global-status 'contents))
      ;; We just showed the table of contents - now show everything
      (show-all)
      (message "SHOW ALL")
      (setq org-cycle-global-status 'all))
     (t
      ;; Default action: go to overview
      (hide-sublevels 1)
      (message "OVERVIEW")
      (setq org-cycle-global-status 'overview))))

   ((save-excursion (beginning-of-line 1) (looking-at outline-regexp))
    ;; At a heading: rotate between three different views
    (outline-back-to-heading)
    (let ((goal-column 0) beg eoh eol eos nxh)
      ;; First, some boundaries
      (save-excursion
	(outline-back-to-heading)  (setq beg (point))
	(save-excursion
	  (beginning-of-line 2)
	  (while (and (not (eobp))   ;; this is like `next-line'
		      (get-char-property (1- (point)) 'invisible))
	    (beginning-of-line 2)) (setq eol (point)))
	(outline-end-of-heading)   (setq eoh (point))
	(outline-end-of-subtree)   (setq eos (point))
        (outline-next-heading)     (setq nxh (point)))
      ;; Find out what to do next and set `this-command'
      (cond
       ((= eos eoh)
	;; Nothing is hidden behind this heading
        (message "EMPTY ENTRY")
        (setq org-cycle-subtree-status nil))
       ((>= eol eos)
	;; Entire subtree is hidden in one line: open it
	(show-entry)
	(show-children)
	(message "CHILDREN")
	(setq org-cycle-subtree-status 'children))
       ((and (eq last-command this-command)
	     (eq org-cycle-subtree-status 'children))
	;; We just showed the children, now show everything.
	(show-subtree)
	(message "SUBTREE")
	(setq org-cycle-subtree-status 'subtree))
       (t
	;; Default action: hide the subtree.
	(hide-subtree)
	(message "FOLDED")
	(setq org-cycle-subtree-status 'folded)))))

   ;; TAB emulation
   (buffer-read-only (outline-back-to-heading))
   ((if (and (eq org-cycle-emulate-tab 'white)
	     (save-excursion (beginning-of-line 1) (looking-at "[ \t]+$")))
	t
      org-cycle-emulate-tab)
    (indent-relative))

   (t (outline-back-to-heading))))

;;; Vertical tree motion

(defun org-move-subtree-up (&optional arg)
  "Move the currrent subtree up past ARG headlines of the same level."
  (interactive "p")
  (org-move-subtree-down (- arg)))

(defun org-move-subtree-down (&optional arg)
  "Move the currrent subtree down past ARG headlines of the same level."
  (interactive "p")
  (let ((re (concat "^" outline-regexp))
	(movfunc (if (> arg 0) 'outline-get-next-sibling
		   'outline-get-last-sibling))
	(ins-point (make-marker))
	(cnt (abs arg))
	beg end txt folded)
    ;; Select the tree
    (outline-back-to-heading)
    (setq beg (point))
    (save-match-data
      (save-excursion (outline-end-of-heading)
		      (setq folded (outline-invisible-p)))
      (outline-end-of-subtree))
    (if (= (char-after) ?\n) (forward-char 1))
    (setq end (point))
    ;; Find insertion point, with error handling
    (goto-char beg)
    (while (> cnt 0)
      (or (funcall movfunc)
	  (progn (goto-char beg)
		 (error "Cannot move past superior level")))
      (setq cnt (1- cnt)))
    (if (> arg 0)
	;; Moving forward - still need to move over subtree
	(progn (outline-end-of-subtree)
	       (if (= (char-after) ?\n) (forward-char 1))))
    (move-marker ins-point (point))
    (setq txt (buffer-substring beg end))
    (delete-region beg end)
    (insert txt)
    (goto-char ins-point)
    (if folded (hide-subtree))
    (move-marker ins-point nil)))

;;; Promotion and Demotion

(defun org-insert-heading ()
  "Insert a new heading with same depth at point."
  (interactive)
  (let* (level
	 (head (save-excursion
		 (condition-case nil
		     (outline-back-to-heading)
		   (error (outline-next-heading)))
		 (prog1 (match-string 0)
		   (setq level (funcall outline-level))))))
    (unless (bolp) (newline))
    (insert head)
    (unless (eolp)
      (save-excursion (newline-and-indent)))
    (unless (equal (char-before) ?\ )
      (insert " "))
    (run-hooks 'org-insert-heading-hook)))

(defun org-promote-subtree (&optional nochildren)
  "Promote the entire subtree.
If prefix argument NOCHILDREN is given, do only the current headline.
See also `org-promote'."
  (interactive "P")
  (org-promote (not nochildren)))

(defun org-demote-subtree (&optional nochildren)
  "Demote the entire subtree.  See `org-demote'.
If prefix argument NOCHILDREN is given, do only the current headline.
See also `org-promote'."
  (interactive "P")
  (org-demote (not nochildren)))

(defun org-do-promote (&optional children)
  "Promote the current heading higher up the tree.
If prefix argument CHILDREN is given, promote also all the children.
If the region is active in transient-mark-mode, promote all headings
in the region."
  (interactive "P")
  (org-promote children)
  (org-fix-position-after-promote))

(defun org-do-demote (&optional children)
  "Demote the current heading lower down the tree.
If prefix argument CHILDREN is given, demote also all the children.
If the region is active in transient-mark-mode, demote all headings
in the region."
  (interactive "P")
  (org-demote children)
  (org-fix-position-after-promote))

(defun org-fix-position-after-promote ()
  "Make sure that after pro/demotion cursor position is right."
  (and (equal (char-after) ?\ )
       (equal (char-before) ?*)
       (forward-char 1)))

(defun org-promote (&optional children)
  "Promote the current heading higher up the tree.
If prefix argument CHILDREN is given, promote also all the children.
If the region is active in transient-mark-mode, promote all headings
in the region."
  (interactive "P")
  (if (and transient-mark-mode mark-active)
      (org-map-region 'org-promote (region-beginning) (region-end))
    (outline-back-to-heading t)
    (let* ((head (match-string-no-properties 0))
	   (level (save-match-data (funcall outline-level)))
	   (up-head (make-string (1- level) ?*)))
      (if (= level 1) (error "Cannot promote to level 0. UNDO to recover."))
      (replace-match up-head nil t)
      (if org-adapt-indentation
          (org-fixup-indentation "^ " "" "^ ?\\S-"))
      (when children
	(org-map-tree 'org-promote level)))))

(defun org-demote (&optional children)
  "Demote the current heading lower down the tree.
If prefix argument CHILDREN is given, demote also all the children.
If the region is active in transient-mark-mode, demote all headings
in the region."
  (interactive "P")
  (if (and transient-mark-mode mark-active)
      (org-map-region 'org-demote (region-beginning) (region-end))
    (outline-back-to-heading t)
    (let* ((head (match-string-no-properties 0))
	   (level (save-match-data (funcall outline-level)))
	   (down-head (make-string (1+ level) ?*)))
      (replace-match down-head nil t)
      (if org-adapt-indentation
          (org-fixup-indentation "^ " "  " "^\\S-"))
      (when children
	(org-map-tree 'org-demote level)))))

(defun org-map-tree (fun level)
  "Call FUN for every heading underneath the current one."
  (save-excursion
    (while (and (progn
		  (outline-next-heading)
		  (> (funcall outline-level) level))
		(not (eobp)))
      (funcall fun))))

(defun org-map-region (fun beg end)
  "Call FUN for every heading between BEG and END."
  (save-excursion
    (setq end (copy-marker end))
    (goto-char beg)
    (deactivate-mark)
    (if (and (re-search-forward (concat "^" outline-regexp) nil t)
	     (< (point) end))
	(funcall fun))
    (while (and (progn
		  (outline-next-heading)
		  (< (point) end))
		(not (eobp)))
      (funcall fun))))

(defun org-fixup-indentation (from to prohibit)
  "Change the indentation in the current entry by re-replacing FROM with TO.
However, if the regexp PROHIBIT matches at all, don't do anything.
This is being used to change indentation along with the length of the
heading marker.  But if there are any lines which are not indented, nothing
is changed at all."
  (save-excursion
    (let ((beg (point))
          (end (save-excursion (outline-next-heading)
                               (point-marker))))
      (unless (save-excursion (re-search-forward prohibit end t))
        (while (re-search-forward from end t)
          (replace-match to)
          (beginning-of-line 2)))
      (move-marker end nil))))

;;; TODO stuff

(defvar org-mouse-map (make-sparse-keymap))
(define-key org-mouse-map [mouse-2] 'org-open-at-mouse)

(require 'font-lock)

(defun org-activate-links (limit)
  (if (re-search-forward "\\(https?\\|ftp\\|mailto\\|file\\|news\\):[^ \t\n\r>]+"
                         limit t)
      (progn
        (add-text-properties (match-beginning 0) (match-end 0)
                             (list 'mouse-face 'highlight
                                   'keymap org-mouse-map))
        t)))

(font-lock-add-keywords 'org-mode
  '((org-activate-links (0 font-lock-keyword-face))
    ("\\<TODO\\>" (0 font-lock-warning-face t))
    ("\\<DEADLINE\\>.*" (0 font-lock-warning-face t))
    ("\\<FIXME\\>" (0 font-lock-warning-face t))
    ("\\<COMMENT\\>" (0 font-lock-warning-face t))
    ("^[*]+ +\\<\\(DONE\\)\\>\\(.*\\)"
     (1 font-lock-type-face t) (2 font-lock-string-face t)))
  'append)

(defun org-unfontify-region (beg end)
  (font-lock-default-unfontify-region beg end)
  (let* ((modified (buffer-modified-p))
         (buffer-undo-list t)
         (inhibit-read-only t) (inhibit-point-motion-hooks t)
         (inhibit-modification-hooks t)
         deactivate-mark buffer-file-name buffer-file-truename)
    (remove-text-properties beg end '(mouse-face nil keymap nil))))

(defun org-mark-todo ()
  (interactive)
  (save-excursion
    (outline-back-to-heading)
    (if (looking-at outline-regexp)
        (save-excursion
          (goto-char (match-end 0))
          (if (looking-at " +\\(TODO\\|DONE\\)\\>")
              (replace-match " TODO" t t nil)
            (insert (concat " TODO")))))))
(defun org-mark-done ()
  (interactive)
  (save-excursion
    (outline-back-to-heading)
    (if (looking-at outline-regexp)
        (save-excursion
          (goto-char (match-end 0))
          (if (looking-at " +\\(TODO\\|DONE\\)\\>")
              (replace-match " DONE" t t nil)
            (insert (concat " DONE")))))))
(defun org-unmark-todo ()
  (interactive)
  (save-excursion
    (outline-back-to-heading)
    (if (looking-at outline-regexp)
        (save-excursion
          (goto-char (match-end 0))
          (if (looking-at " +\\(TODO\\|DONE\\)\\>")
              (replace-match "" t t nil))))))

(defun org-todo ()
  "Change the TODO state of an item.
When the item starts with TODO, it is changed to DONE.  When it starts with
DONE, the DONE is removed.  And when neither TODO nor DONE are present, add
TODO at the beginning of the heading.  So this function rotates throught the
three different TODO states."
  (interactive)
  (save-excursion
    (outline-back-to-heading)
    (cond
     ((looking-at (concat outline-regexp " +TODO")) (org-mark-done))
     ((looking-at (concat outline-regexp " +DONE")) (org-unmark-todo))
     (t (org-mark-todo)))))

(defun org-deadline ()
  "Insert the DEADLINE: string to make a deadline.
A timestap is also inserted - use \\[org-timestamp-up] and \\[org-timestamp-down]
to modify it to the correct date."
  (interactive)
  (let ((case-fold-search nil))
    (cond
     ((save-excursion
        (beginning-of-line 1)
        (looking-at ".*\\(\\<DEADLINE\\>\\)"))
      (replace-match "deadline" t t nil 1))
     ((save-excursion
        (beginning-of-line 1)
        (looking-at ".*\\(\\<deadline\\>\\)"))
      (replace-match "DEADLINE" t t nil 1))
     (t
      (insert "DEADLINE: ")
      (org-time-stamp nil)
      (message (substitute-command-keys
                "Use \\[org-timestamp-up] and \\[org-timestamp-down] to set date."))))))

(defun org-toggle-comment ()
  "Change the COMMENT state of an entry."
  (interactive)
  (save-excursion
    (outline-back-to-heading)
    (if (looking-at (concat outline-regexp "\\( +\\<COMMENT\\>\\)"))
        (replace-match "" t t nil 1)
      (if (looking-at outline-regexp)
          (progn
            (goto-char (match-end 0))
            (insert " COMMENT"))))))

(defun org-check-deadlines (ndays)
  "Check if there are any deadlines due or past due.
A deadline is considered due if it happens within `org-deadline-warning-days'
days from todays date.  The prefix arg NDAYS can be used to test that many
days.  If the prefix are is a raw C-u prefix, all deadlines are shown."
  (interactive "P")
  (let* ((org-warn-days
          (cond
           ((equal ndays '(4)) 100000)
           (ndays (prefix-numeric-value ndays))
           (t org-deadline-warning-days)))
         (case-fold-search nil)
         (tsr (org-make-ts-regexp))
         (regexp (concat "\\<DEADLINE\\>: *" tsr))
         (callback
          (lambda ()
            (let ((d1 (time-to-days (current-time)))
                  (d2 (time-to-days
                       (apply 'encode-time
                              (append '(0 0 0)
                                      (nthcdr 3 (parse-time-string
                                                 (match-string 1))))))))
              (< (- d2 d1) org-warn-days)))))
    (message "%d deadlines past-due or due within %d days"
             (org-occur regexp callback)
             org-warn-days)))

(defun org-move-line-down (arg)
  "Move the current line up."
  (interactive "p")
  (let ((col (current-column))
	beg end pos)
    (beginning-of-line 1) (setq beg (point))
    (beginning-of-line 2) (setq end (point))
    (beginning-of-line (+ 1 arg))
    (setq pos (move-marker (make-marker) (point)))
    (insert (delete-and-extract-region beg end))
    (goto-char pos)
    (move-to-column col)))

(defun org-move-line-up (arg)
  "Move the current line up."
  (interactive "p")
  (let ((col (current-column))
	beg end pos)
    (beginning-of-line 1) (setq beg (point))
    (beginning-of-line 2) (setq end (point))
    (beginning-of-line (+ -2 arg))
    (setq pos (move-marker (make-marker) (point)))
    (insert (delete-and-extract-region beg end))
    (goto-char pos)
    (move-to-column col)))


;;; Link Stuff


(defun org-open-at-mouse (ev)
  "Open file link or URL at mouse."
  (interactive "e")
  (mouse-set-point ev)
  (org-open-at-point))

(defun org-open-at-point (&optional force)
  "Open file link or URL at or after point.
If there is no link at point, this function will search forward up to
the end of the current subtree."
  (interactive "P")
  (let (url type path)
    (save-excursion
      (skip-chars-backward "^ \t\n\r<")
      (if (re-search-forward
           "\\<\\(https?\\|ftp\\|mailto\\|file\\|news\\):\\([^ \t\n\r>]+\\)"
           (save-excursion
             (condition-case nil
                 (progn (outline-end-of-subtree) (point))
               (error (end-of-line 1) (point))))
           t)
          (setq url (match-string 0)
                type (match-string 1)
                path (match-string 2)))
      (cond
       ((string= type "file")
        (org-open-file path force))
       ((string= type "news")
        (gnus-fetch-group path))
       (t
        (browse-url-at-point))))))

(defun org-open-file (path &optional force)
  "Open the file at PATH.
First, this expands any special file name abbreviations.  Then the
configuration variable `org-file-apps' is checked if it contains an
entry for this file type, and if yes, the corresponding command is launched.
If no application is found, Emacs simply visits the file.
If the file does not exist, throw an error unless FORCE is non-nil."
  (let* ((file (org-expand-file-name path))
         (dfile (downcase file))
         ext cmd nofile)
    (if (and (not (file-exists-p file))
             (not force))
        (error "No such file: %s" file))
    (if (string-match "^.*\\.\\([a-zA-Z0-9]+\\.gz\\)$" dfile)
        (setq ext (match-string 1 dfile))
      (if (string-match "^.*\\.\\([a-zA-Z0-9]+\\)$" dfile)
          (setq ext (match-string 1 dfile))))
    (setq cmd (cdr (assoc ext (append org-file-apps
                                      (org-default-apps)))))
    (if (and cmd (not (string-match "^\\s-*$" cmd)))
        (progn (setq cmd (format cmd file))
               (save-window-excursion
                 (shell-command (concat cmd " & &"))))
      (find-file-other-window file))))

(defun org-default-apps ()
  "Return the default applications for this operating system.
There is no windows support."
  (cond
   ((eq system-type 'darwin)
    org-file-apps-defaults-macosx)
   ((eq system-type 'linux)
    org-file-apps-defaults-linux)
   (t org-file-apps-defaults-linux)))

(defun org-expand-file-name (path)
  "Replace special path abbreviations and expand the file name."
  (expand-file-name path))

(defun org-insert-link (&optional absolute)
  (interactive "P")
  "Insert a link to a file, with file name completion.
The path to the file will be relative to the current directory if the file
is in the current directory or a subdirectory.  Otherwise, the link will
be as completed in the minibuffer (i.e. normally relative to the users
HOME directory).
With optional prefix ABSOLUTE, insert the absolute path."
  (let ((link (read-file-name "File or URL: ")))
    (if (string-match "^\\(https?\\|ftp\\|mailto\\|news\\)" link)
        (insert link)
      (let ((pwd (file-name-as-directory (expand-file-name "."))))
        (cond
         (absolute
          (insert "file:" (expand-file-name link)))
         ((string-match (concat "^" (regexp-quote pwd) "\\(.+\\)")
                        (expand-file-name link))
          (insert "file:" (match-string 1 (expand-file-name link))))
         (t (insert "file:" link)))))))

(defconst org-wchars "^ \t\n\r~!@#$%^&*()[]{}/\\\\|:;")
(defun org-bbdb-name ()
  (interactive)
  (save-excursion
    (let* ((beg (save-excursion (skip-chars-backward org-wchars) (point)))
           (end (save-excursion (skip-chars-forward org-wchars) (point)))
           (word (buffer-substring-no-properties beg end)))
      (bbdb word nil))))

(defun org-time-stamp (arg)
  "Insert or update a time stamp.
If there is already a time stamp in the current line, update it.
Otherwise, insert a new one.  This function detects time stamps by
looking for <...> with the same number of characters as the current time
string would give."
  (interactive "P")
  (let ((tsr (org-make-ts-regexp))
        (fmt (if arg (cdr org-time-stamp-formats)
               (car org-time-stamp-formats))))
    (if (save-excursion
          (beginning-of-line 1)
          (looking-at (concat ".*\\(" tsr "\\)")))
        (progn
          (replace-match "" nil nil nil 1)
          (insert (format-time-string fmt))
          (message "Timestamp updated"))
      (insert (format-time-string fmt)))))

(defun org-timestamp-up ()
  "Increase the date in the time stamp by one day."
  (interactive)
  (org-timestamp-change 1 'day))

(defun org-timestamp-down ()
  "Increase the date in the time stamp by one day."
  (interactive)
  (org-timestamp-change -1 'day))

(defun org-timestamp-up-month ()
  "Increase the date in the time stamp by one day."
  (interactive)
  (org-timestamp-change 1 'month))

(defun org-timestamp-down-month ()
  "Increase the date in the time stamp by one day."
  (interactive)
  (org-timestamp-change -1 'month))

(defun org-timestamp-change (n what)
  "Increase the date in the time stamp by one day."
  (let ((tsr (org-make-ts-regexp))
        (fmt (car org-time-stamp-formats))
        time)
    (if (save-excursion
          (beginning-of-line 1)
          (looking-at (concat ".*\\(" tsr "\\)")))
        (progn
          (setq time (parse-time-string (substring (match-string 1) 1 -1)))
          (setq time
                (apply 'encode-time
                       (append '(0 0 0)
                               (list (+ (if (eq what 'day) n 0) (nth 3 time)))
                               (list (+ (if (eq what 'month) n 0) (nth 4 time)))
                               (list (+ (if (eq what 'year) n 0) (nth 5 time)))
                               (nthcdr 6 time))))
          (replace-match "" nil nil nil 1)
          (insert (format-time-string fmt time)))
      (insert (format-time-string fmt)))))

(defun org-make-ts-regexp ()
  "Make a regular expression matching a time string.
The regexp will match a string <...> which has the right length of either
of the two formats in the variable `org-time-stamp-formats'."
  (let* ((ts1 (format-time-string (car org-time-stamp-formats)))
         (ts2 (format-time-string (cdr org-time-stamp-formats)))
         (l1 (length ts1)) (l2 (length ts2))
         (n1 (- (min l1 l2) 2)) (n2 (abs (- l1 l2))))
    (concat "<\\(" (make-string n1 ?.) "\\(" (make-string n2 ?.) "\\)?\\)>")))

;;; Key bindings

(define-key org-mode-map "\C-a" 'org-beginning-of-line)
(define-key org-mode-map [(tab)] 'org-cycle)
(define-key org-mode-map "\C-c\C-c" 'org-edit-picture)
(define-key org-mode-map [(meta shift left)] 'org-promote-subtree)
(define-key org-mode-map [(meta left)] 'org-do-promote)
(define-key org-mode-map [(meta shift right)] 'org-demote-subtree)
(define-key org-mode-map [(meta right)] 'org-do-demote)
(define-key org-mode-map [(meta up)] 'org-move-subtree-up)
(define-key org-mode-map [(meta down)] 'org-move-subtree-down)
(define-key org-mode-map "\C-c\C-t" 'org-todo)
(define-key org-mode-map "\C-c\C-d" 'org-deadline)
(define-key org-mode-map "\C-c;"    'org-toggle-comment)
(define-key org-mode-map "\C-c\C-v" 'org-show-todo-tree)
(define-key org-mode-map "\C-c\C-w" 'org-check-deadlines)
(define-key org-mode-map "\C-c/"    'org-occur)
(define-key org-mode-map "\C-c\C-m" 'org-insert-heading)
(define-key org-mode-map "\M-\C-m"  'org-insert-heading)
(define-key org-mode-map "\C-c\C-l" 'org-insert-link)
(define-key org-mode-map "\C-c\C-o" 'org-open-at-point)
(define-key org-mode-map "\C-c:"    'org-bbdb-name)
(define-key org-mode-map "\C-c."    'org-time-stamp)
(define-key org-mode-map [(shift up)]     'org-timestamp-up)
(define-key org-mode-map [(shift down)]   'org-timestamp-down)
(define-key org-mode-map [(shift left)]   'org-timestamp-down-month)
(define-key org-mode-map [(shift right)]  'org-timestamp-up-month)
(define-key org-mode-map [(control up)]   'org-move-line-up)
(define-key org-mode-map [(control down)] 'org-move-line-down)
(define-key org-mode-map "\C-c\C-xa" 'org-export-as-ascii)
(define-key org-mode-map "\C-c\C-xt" 'org-insert-html-options-template)
(define-key org-mode-map "\C-c\C-xh" 'org-export-as-html)
(define-key org-mode-map "\C-c\C-x\C-h" 'org-export-as-html-and-open)

;;; Menu entries

(defvar org-mode-menu-bar-map
  (let ((map (make-sparse-keymap)))

    ;; Define the menu heads
;    (define-key map [headings]
;      (cons "Headings" (make-sparse-keymap "Headings")))
    (define-key map [org]
      (cons "Org" (make-sparse-keymap "Org")))
    (define-key map [struct]
      (cons "Structure" (make-sparse-keymap "Struct")))

    ;; Now fill the menus

    (mapcar
     (lambda (x) (define-key map (vector 'org (car x)) (cdr x)))
     (nreverse
      '(
        (todo . ("TODO/DONE/-" . org-todo))
        (show-todo . ("Show TODO Tree" . org-show-todo-tree))
        (occur . ("Occur" . org-occur))

        (s1 . (menu-item "--"))

        (dead . ("Deadline" . org-deadline))
        (show-dead . ("Check Deadlines" . org-check-deadlines))

        (s2 . (menu-item "--"))

        (time . ("Timestamp" . org-time-stamp))
        (time1 . ("1 day later" . org-timestamp-up))
        (time2 . ("1 day earlier" . org-timestamp-down))
        (time3 . ("1 month later" . org-timestamp-up-month))
        (time4 . ("1 month earlier" . org-timestamp-down-month))

        (s3 . (menu-item "--"))

        (link . ("Insert Link" . org-insert-link))
        (open . ("Follow Link" . org-open-at-point))
        (bbdb . ("BBDB" . org-bbdb-name))

        (s4 . (menu-item "--"))

        (picture . ("Picture Mode" . org-edit-picture))

        (s5 . (menu-item "--"))
        
        (exASCII    . ("Export region as ASCII" . org-export-as-ascii))
        (extmpl . ("Option template HTML" . org-insert-html-options-template))
        (exHTML . ("Export region as HTML" . org-export-as-html))
        (exopHTML . ("Export HTML and open" . org-export-as-html-and-open))
        )))

    (mapcar
     (lambda (x) (define-key map (vector 'struct (car x)) (cdr x)))
     (nreverse
      '(
        (cycle . (menu-item (if (and (bobp) (not (outline-on-heading-p)))
                                 "Cycle Global" "Cycle Subtree")
                            org-cycle
                             :enable (or (bobp) (outline-on-heading-p))))
        (show-all . ("Show All" . show-all))

        (s1 . (menu-item "--"))

        (new  . ("New Heading" . org-insert-heading))
        (s3   . (menu-item "--"))
        (oup  . ("Up" . outline-up-heading))
        (onvh . ("Next" . outline-next-visible-heading))
        (opvh . ("Previous" . outline-previous-visible-heading))
        (ofsl . ("Next Same Level" . outline-forward-same-level))
        (obsl . ("Previous Same Level" . outline-backward-same-level))

        (s4   . (menu-item "--"))
        (move-up         . ("Move subtree up" . org-move-subtree-up))
        (move-down       . ("Move subtree down" . org-move-subtree-down))

        (s5 . (menu-item "--"))
        (promote-heading . ("Promote Heading" . org-promote))
        (promote-tree    . ("Promote Subtree" . org-promote-subtree))
        (demote-heading  . ("Demote Heading"  . org-demote))
        (demote-tree     . ("Demote Subtree"  . org-demote-subtree))
        )))
    map))

;;; Modified picture mode with extra functions and bindings
;;; BUGS:  The mouse stuff does not really work reliable

(eval-after-load "picture"
  ' (progn
      (define-key picture-mode-map [(meta left)] (lambda (arg) (interactive "p") (org-picture-draw 4 arg)))
      (define-key picture-mode-map [(meta right)] (lambda (arg) (interactive "p") (org-picture-draw 6 arg)))
      (define-key picture-mode-map [(meta up)] (lambda (arg) (interactive "p") (org-picture-draw 8 arg)))
      (define-key picture-mode-map [(meta down)] (lambda (arg) (interactive "p") (org-picture-draw 2 arg)))
      (define-key picture-mode-map [(meta shift left)] (lambda (arg) (interactive "p") (org-picture-draw 7 arg)))
      (define-key picture-mode-map [(meta shift right)] (lambda (arg) (interactive "p") (org-picture-draw 3 arg)))
      (define-key picture-mode-map [(meta shift up)] (lambda (arg) (interactive "p") (org-picture-draw 9 arg)))
      (define-key picture-mode-map [(meta shift down)] (lambda (arg) (interactive "p") (org-picture-draw 1 arg)))

      (define-key picture-mode-map [(meta ?j)] (lambda (arg) (interactive "p") (org-picture-draw 1 arg)))
      (define-key picture-mode-map [(meta ?k)] (lambda (arg) (interactive "p") (org-picture-draw 2 arg)))
      (define-key picture-mode-map [(meta ?l)] (lambda (arg) (interactive "p") (org-picture-draw 3 arg)))
      (define-key picture-mode-map [(meta ?u)] (lambda (arg) (interactive "p") (org-picture-draw 4 arg)))
      (define-key picture-mode-map [(meta ?o)] (lambda (arg) (interactive "p") (org-picture-draw 6 arg)))
      (define-key picture-mode-map [(meta ?7)] (lambda (arg) (interactive "p") (org-picture-draw 7 arg)))
      (define-key picture-mode-map [(meta ?8)] (lambda (arg) (interactive "p") (org-picture-draw 8 arg)))
      (define-key picture-mode-map [(meta ?9)] (lambda (arg) (interactive "p") (org-picture-draw 9 arg)))
      (define-key picture-mode-map [(meta ?-)] 'org-picture-draw-line)
      (define-key picture-mode-map [mouse-2] 'org-picture-mouse-line-to-here)
      (define-key picture-mode-map [mouse-1] 'org-picture-mouse-set-point)
      (define-key picture-mode-map [(shift down-mouse-1)] 'org-picture-draw-with-mouse)
      (define-key picture-mode-map "\C-c\C-c" 'org-picture-mode-exit)))

(defun org-edit-picture ()
  "Switch to picture mode and save the value of `transient-mark-mode'.
Turn transient-mark-mode off while in picture-mode."
  (interactive)
  (if (not org-enable-picture-mode)
      (error "Set variable `org-enable-picture-mode' to allow picture-mode."))
  (set (make-local-variable 'org-transient-mark-mode)
       transient-mark-mode)
  (set (make-local-variable 'transient-mark-mode) nil)
  (picture-mode))

(defun org-picture-mode-exit (&optional arg)
  "Turn off picture mode and restore `transient-mark-mode'."
  (interactive "P")
  (if (local-variable-p 'org-transient-mark-mode)
      (setq transient-mark-mode org-transient-mark-mode))
  (deactivate-mark)
  (picture-mode-exit))

(defun org-picture-draw (dir arg)
  (cond
   ((equal dir 1)
    (picture-movement-sw)
    (setq last-command-event ?/)  (picture-self-insert arg))
   ((equal dir 2)
    (picture-movement-down)
    (setq last-command-event ?|)  (picture-self-insert arg))
   ((equal dir 3)
    (picture-movement-se)
    (setq last-command-event ?\\) (picture-self-insert arg))
   ((equal dir 4)
    (picture-movement-left)
    (setq last-command-event ?-)  (picture-self-insert arg))
   ((equal dir 5))
   ((equal dir 6)
    (picture-movement-right)
    (setq last-command-event ?-)  (picture-self-insert arg))
   ((equal dir 7)
    (picture-movement-nw)
    (setq last-command-event ?\\) (picture-self-insert arg))
   ((equal dir 8)
    (picture-movement-up)
    (setq last-command-event ?|)  (picture-self-insert arg))
   ((equal dir 9)
    (picture-movement-ne)
    (setq last-command-event ?/)  (picture-self-insert arg)))
  (picture-movement-right))

(defun org-picture-draw-line (&optional beg end)
  "Draw a line from mark to point."
  (interactive)
  (unless (and beg end)
    (setq beg (mark 'force)
          end (point)))
  (let (x1 x2 y1 y2 n i Dx Dy dx dy char lp x y x1a y1a lastx lasty)
    (goto-char beg)
    (setq x1 (current-column) y1 (count-lines (point-min) (point)))
    (if (bolp) (setq y1 (1+ y1)))
    (goto-char end)
    (setq x2 (current-column) y2 (count-lines (point-min) (point)))
    (if (bolp) (setq y2 (1+ y2)))
    (setq Dx (- x2 x1) Dy (- y2 y1)
	  n (+ (abs Dx) (abs Dy))
	  n (sqrt (+ (* Dx Dx) (* Dy Dy)))
	  n (max (abs Dx) (abs Dy))
	  n (max (abs Dx) (abs Dy))
	  dx (/ (float Dx) (float n)) dy (/ (float Dy) (float n)))
    (setq x1a (floor (+ x1 (* 1. dx) .5))
          y1a (floor (+ y1 (* 1. dy) .5)))
    ;; Do the loop
    (setq i -1)
    (setq lastx x1a lasty y1a)
    (while (< i n)
      (setq i (1+ i)
	    x (floor (+ x1 (* (float i) dx) .5))
	    y (floor (+ y1 (* (float i) dy) .5)))
      (setq char (cond ((= lastx x) ?|) ((= lasty y) ?-)
                       ((> (* (- x lastx) (- y lasty)) 0) ?\\)
                       (t ?/))
            lastx x lasty y)
      (goto-line y)
      (move-to-column x t)
      (setq last-command-event char)
      (setq lp (point))
      (picture-self-insert 1))
    (goto-char lp)
    (set-mark lp)))

(defun org-picture-mouse-line-to-here (ev)
  "Draw a line from point to the click position."
  (interactive "e")
  (let* ((beg (move-marker (make-marker) (point))))
    (org-picture-mouse-set-point ev)
    (org-picture-draw-line beg (point))
    (move-marker beg nil)))

;; Draw with the mouse
(defun org-picture-mouse-set-point (ev)
  "Mouse-set-point, but force position."
  (interactive "e")
  (let* ((colrow (posn-col-row (event-end ev)))
	 (col (car colrow)) (line (cdr colrow))
	 (realline (1+ (+ (count-lines (point-min) (window-start)) line))))
    (goto-line realline)
    (while (and (eobp)
		(not (> (count-lines (point-min) (point-max)) realline)))
      (newline))
    (goto-line realline)
    (move-to-column col t)))

(defun org-picture-draw-with-mouse (ev)
  "Use the mouse like a brush and paint stars where it goes."
  (interactive "e")
  (let (lastcr cr)
    (track-mouse
      (catch 'exit
	(while t
	  (setq e (read-event))
	  (if (not (eq (car e) 'mouse-movement)) (throw 'exit nil))
	  (setq cr (posn-col-row (event-end e)))
	  (when (not (equal cr lastcr))
	    (setq lastcr cr)
	    (org-picture-mouse-set-point e)
	    (setq last-command-event ?*)
	    (save-excursion
	      (picture-self-insert 1))))))))

(defun org-occur (regexp &optional callback)
  "Make a compact tree which shows all matches of REGEXP.
The tree will show the lines where the regexp matches, and all higher
headlines above the match.  It will also show the heading after the match,
to make sure editing the matching entry is easy.
if CALLBACK is non-nil, it is a function which is called to confirm
that the match should indeed be shown."
  (interactive "sRegexp: ")
  (let ((cnt 0))
    (save-excursion
      (goto-char (point-min))
      (hide-sublevels 1)
      (while (re-search-forward regexp nil t)
        (when (or (not callback)
                  (funcall callback))
          (setq cnt (1+ cnt))
          (if (outline-on-heading-p 'invisible-ok)
              (org-flag-heading nil)    ; only show the heading
            (show-entry))               ; show entire entry
          (save-excursion
            (and (outline-next-heading)
                 (org-flag-heading nil)))  ; show the next heading
          (save-excursion                  ; show all higher headings
            (while (condition-case nil
                       (progn (outline-up-heading-all 1) t)
                     (error nil))
              (org-flag-heading nil))))))
    (if (interactive-p)
        (message "%d match(es) for regexp %s" cnt regexp))
    cnt))

(defun org-show-todo-tree (arg)
  "Make a compact tree which shows all headlines marked with TODO.
The tree will show the lines where the regexp matches, and all higher
headlines above the match."
  (interactive "P")
  (message "%d TODO entries found"
           (org-occur (concat "^" outline-regexp " +\\<TODO\\>"))))

(defun org-flag-heading (flag &optional entry)
  "Flag the current heading. FLAG non-nil means make invisible."
  (save-excursion
    (outline-back-to-heading 'invisible)
    (if entry
        (show-entry)
      (outline-flag-region (max 1 (1- (point)))
                           (save-excursion (outline-end-of-heading) (point))
                           flag))))

;; Exporting

(defconst org-level-max 20)

(defun org-export-find-first-heading-line (list)
  (let ((re (concat "^" outline-regexp)))
    (while (not (string-match re (car list)))
      (pop list))
    list))

(defun org-skip-comments (lines)
  "Skip lines startin with \"#\" and subtrees starting with COMMENT."
  (let ((re1 "^\\(\\*+\\)[ \t]+COMMENT")
        (re2 "^\\(\\*+\\)[ \t\n\r]")
        rtn line level)
    (while (setq line (pop lines))
      (cond
       ((and (string-match re1 line)
             (setq level (- (match-end 1) (match-beginning 1))))
        ;; Beginning of a COMMENT subtree.  Skip it.
        (while (and (setq line (pop lines))
                    (or (not (string-match re2 line))
                        (> (- (match-end 1) (match-beginning 1)) level))))
        (setq lines (cons line lines)))
       ((string-match "^#" line)
        ;; an ordinary commet line
        )
       (t (setq rtn (cons line rtn)))))
    (nreverse rtn)))

;; ASCII

(defconst org-ascii-underline '(?\$ ?\# ?^ ?\~ ?\= ?\-))

(defun org-export-as-ascii (arg)
  "Export the outline as a pretty ASCII file.
If there is an active region, export only the region.
The prefix ARG specifies how many levels of the outline should become
underlined headlines.  The default is 3."
  (interactive "P")
  (let* ((region
          (buffer-substring
           (if (org-region-active-p) (region-beginning) (point-min))
           (if (org-region-active-p) (region-end) (point-max))))
         (lines (org-export-find-first-heading-line
                 (org-skip-comments (split-string region "\n"))))
         (level 0) line macro txt
         (umax (if arg (prefix-numeric-value arg)
                 org-export-headline-levels))
         (bufname (concat (buffer-name) ".txt"))
         (levels-open (make-vector org-level-max nil)))
    (org-init-section-numbers)

    (if org-export-ascii-show-new-buffer
	(switch-to-buffer-other-window (get-buffer-create bufname)))
    (set-buffer (get-buffer-create bufname))
    (erase-buffer)
    (fundamental-mode)
    (while (setq line (pop lines))
      (cond
       ((string-match "^\\(\\*+\\)[ \t]*\\(.*\\)" line)
        ;; a Headline
        (setq level (- (match-end 1) (match-beginning 1))
              txt (match-string 2 line))
        (org-ascii-level-start level txt umax))
       (t (insert line "\n"))))
    (setq buffer-file-name (expand-file-name bufname))
    (normal-mode)
    (save-buffer)
    (goto-char (point-min))))

(defun org-ascii-level-start (level title umax)
  (let (char)
    (if (> level umax)
;        (insert (make-string (* 2 (- level umax)) ?\ ) title "\n")
        (insert (make-string (* 2 (- level umax 1)) ?\ ) "* " title "\n")
      (setq char (nth (- umax level) (reverse org-ascii-underline)))
      (if org-export-with-section-numbers
          (setq title (concat (org-section-number level) " " title)))
      (insert title "\n" (make-string (length title) char) "\n\n"))))

;; HTML

(defun org-insert-html-options-template ()
  (interactive)
  (if (not (bolp)) (newline))
  (let ((pos (point)))
(insert (format
"+TITLE:     %s
+AUTHOR:    %s
+EMAIL:     %s
+TEXT:      Some descriptive text to be emitted.  Several lines may 
+TEXT:      be given. Precede HTML tags with \"@\", like @<b>here@</b>. 
" (buffer-name) (user-full-name) user-mail-address))))

(defun org-export-as-html-and-open (arg)
  "Export the outline as HTML and immediately open it with a browser.
If there is an active region, export only the region.
The prefix ARG specifies how many levels of the outline should become
headlines.  The default is 3.  Lower levels will become bulleted lists."
  (interactive "P")
  (org-export-as-html arg 'hidden)
  (org-open-file (buffer-file-name) ".html"))

(defun org-export-as-html-batch ()
  "Call org-export-as-html, may be used in batch processing as
emacs 	--batch
	--load=$HOME/lib/emacs/org.el
	--eval \"(setq org-export-headline-levels 2)\"
	--visit=MyFile --funcall org-export-as-html-batch"
  (org-export-as-html org-export-headline-levels 'hidden)
)

(defun org-export-as-html (arg &optional hidden)
  "Export the outline as a pretty HTML file.
If there is an active region, export only the region.
The prefix ARG specifies how many levels of the outline should become
headlines.  The default is 3.  Lower levels will become bulleted lists."
  (interactive "P")
  (let* ((region-p (org-region-active-p))
         (region
          (buffer-substring
           (if region-p (region-beginning) (point-min))
           (if region-p (region-end) (point-max))))
         (all_lines 
          (org-skip-comments (split-string region "\n")))
         (lines (org-export-find-first-heading-line all_lines))
         (level 0) line macro txt todo
	 (last-level 1)
         (umax (if arg (prefix-numeric-value arg)
                 org-export-headline-levels))
         (bufname (concat (buffer-name) ".html"))
         (levels-open (make-vector org-level-max nil))
	 (date  (format-time-string "%Y/%m/%d" (current-time)))
	 (time  (format-time-string "%X" (current-time)))
         (author      user-full-name)
	 (title       (buffer-name))
	 (email       user-mail-address)
	 (text        nil)
	 (with-date   nil)
	 (with-toc    org-export-html-with-toc)
	 (head-count  0) cnt
         (opt-re (concat "^[ \t]*\\+[ \t]*\\("
                         "TITLE\\|AUTHOR\\|EMAIL\\|TEXT"
                         "\\):[ \t]*\\(.*\\)"))
	 )
    (org-init-section-numbers)

    ;; Search for the HTML key lines
    (save-excursion
      (goto-char 0)
      (let ((re opt-re)
            key)
        (while (re-search-forward re nil t)
          (setq key (match-string 1))
          (cond ((string-equal key "TITLE") 
                 (setq title (org-html-expand (match-string 2))))
                ((string-equal key "AUTHOR") 
                 (setq author (org-html-expand (match-string 2))))
                ((string-equal key "EMAIL")
                 (setq email (org-html-expand (match-string 2))))
                ((string-equal key "TEXT") 
                 (setq text (concat text "\n" (org-html-expand
                                               (match-string 2)))))))))

    ;; Switch to the output buffer
    (if (or hidden (not org-export-html-show-new-buffer))
        (set-buffer (get-buffer-create bufname))
      (switch-to-buffer-other-window (get-buffer-create bufname)))
    (erase-buffer)
    (fundamental-mode)

    ;; File header
    (insert (format
             "<html lang=\"en\"><head>
<title>%s</title>
<meta http-equiv=\"Content-Type\" content=\"text/html\">
<meta name=generator content=\"org-mode\">
<meta name=generated content=\"%s %s\">
<meta name=author content=\"%s\">
</head><body>
"
             title date time author))
    (if title     (insert (concat "<H1 align=\"center\">" title "</H1>\n")))
    (if author	  (insert (concat "Author: " author "\n")))
    (if email	  (insert (concat "<a href=\"mailto:" email "\">&lt;" email "&gt;</a>\n")))
    (if (or author email) (insert "<br>\n"))
    (if (and date time) (insert (concat "Date: " date " " time "<br>\n")))
    (if text      (insert (concat "<p>\n" text)))
    (if with-toc
        (progn
	  (insert "<H2>Table of Contents</H2>\n")
          (insert "<ul>\n")
          (mapcar '(lambda (line)
		     (if (string-match "^\\(\\*+\\)[ \t]*\\(TODO\\|DONE\\)?\\(.*\\)" line)
			 ;; This is a headline
			 (progn
			   (setq level (- (match-end 1) (match-beginning 1))
                                 todo (and (match-beginning 2)
                                           (equal (match-string 2 line) "TODO"))
				 txt (org-html-expand 
                                      (match-string 3 line)))
			   (if (<= level umax)
			       (progn
				 (setq head-count (+ head-count 1))
				 (if (> level last-level) (insert "<ul>\n"))
				 (if (< level last-level)
                                     (progn
                                       (setq cnt (- last-level level))
                                       (while (>= (setq cnt (1- cnt)) 0)
                                         (insert "</ul>\n"))))
				 (insert
				  (format
                                   (if todo
                                       "<li><a href=\"#sec-%d\"><span style='color:red'>%s</span></a></li>\n"
                                     "<li><a href=\"#sec-%d\">%s</a></li>\n")
				   head-count txt))
				 (setq last-level level))
			     ))))
		  lines)
          (while (> last-level 0)
            (setq last-level (1- last-level))
            (insert "</ul>\n"))
          ))
    (setq head-count 0)
    
    (while (setq line (pop lines))
      ;; replace "<" and ">" by "&lt;" and "&gt;"
      ;; handle @<..> HTML tags (replace "@&gt;..&lt;" by "<..>")
      (setq line (org-html-expand line))

      ;; Verbatim lines
      (if (and org-export-html-with-fixed-width
               (string-match "^[ \t]+|\\(.*\\)" line))
          (progn
            (let ((l (match-string 1 line)))
              (while (string-match " " l)
                (setq l (replace-match "&nbsp;" t t l)))
              (insert "\n<br><span style='font-family:Courier'>"
                      l "</span>"
                      (if (and lines 
                               (not (string-match "^[ \t]+\\(|.*\\)" (car lines))))
                          "<br>\n" "\n"))))
      
        ;; Web links
        (if (string-match "\\<\\(https?\\|ftp\\|mailto\\|news\\):\\([^ \t\r\n>]+\\)"
                          line)
            (setq line (replace-match "<a href=\"\\1:\\2\">&lt;\\2&gt;</a>"
                                      nil nil line)))
        ;; Links to files
        (if (string-match "\\<\\(file\\):\\([^ \t\r\n>]+\\)"
                          line)
            (setq line (replace-match
;;                      ;; FIXME: Can we do without expand-file-name???
;;                      (concat "<a href=\"file://"
;;                              (expand-file-name (match-string 2 line))
;;                              "\">\\&</a>")
                        "<a href=\"\\2\">\\2</a>"
                        nil nil line)))
        
        ;; TODO items
        (if (string-match "^\\(\\*+\\)[ \t]*\\(TODO\\)\\>" line)
            (setq line (replace-match "<span style='color:red'>TODO</span>"
                                      nil nil line 2)))
        ;; DONE items
        (if (string-match "^\\(\\*+\\)[ \t]*\\(DONE\\)\\>" line)
            (setq line (replace-match
                        "<span style='color:green'>DONE</span>"
                        nil nil line 2)))
        ;; DEADLINES
        (if (string-match "\\<DEADLINE.*" line)
            (setq line (replace-match "<span style='color:red'>\\&</span>"
                                      nil nil line)))
        
        (if (string-match "^\\(\\*+\\)[ \t]*\\(.*\\)" line)
            ;; This is a headline
            (progn
              (setq level (- (match-end 1) (match-beginning 1))
                    txt (match-string 2 line))
              (if (<= level umax) (setq head-count (+ head-count 1)))
              (org-html-level-start level txt umax
                                    (and with-toc (<= level umax))
                                    head-count)
              ;; Normal lines
              ;; Lines starting with "-", and empty lines make new paragraph.
              (if (string-match "^ *-\\|^[ \t]*$" line) (insert "<p>")))
          (insert line (if org-export-html-preserve-breaks "<br>\n" "\n")))))
    (insert "</body>\n</html>\n")
    (setq buffer-file-name (expand-file-name bufname))
    (normal-mode)
    (save-buffer)
    (goto-char (point-min))))

(defun org-html-expand (s)
  ;; convert < to &lt; and > to &gt;
  (while (string-match "<" s)
    (setq s (replace-match "&lt;" nil nil s)))
  (while (string-match ">" s)
    (setq s (replace-match "&gt;" nil nil s)))
  (if org-export-html-expand
      (while (string-match "@&lt;\\([^&]*\\)&gt;" s)
        (setq s (replace-match "<\\1>" nil nil s))))
  (if org-export-html-with-sub-superscripts
      (setq s (org-export-html-convert-sub-super s)))
  (if org-export-html-with-emphasize
      (setq s (org-export-html-convert-emphasize s)))
  (if org-export-html-with-TeX-macros
      (while (string-match "\\\\\\([a-zA-Z]+\\)" s)
        (setq s (replace-match (concat "&" (match-string 1 s) ";") t t s))))
  s)

(defun org-html-level-start (level title umax with-toc head-count)
  (let (char
        (l (1+ (max level umax))))
    (while (<= l org-level-max)
      (if (aref levels-open (1- l))
          (progn
            (org-html-level-close l)
            (aset levels-open (1- l) nil)))
      (setq l (1+ l)))
    (if (> level umax)
        (progn
          (if (aref levels-open (1- level))
              (insert "<li>" title "<p>\n")
            (aset levels-open (1- level) t)
            (insert "<ul><li>" title "<p>\n")))
      (if org-export-with-section-numbers
          (setq title (concat (org-section-number level) " " title)))
      (setq level (+ level 1))
      (if with-toc
	  (insert (format "\n<H%d><a name=\"sec-%d\">%s</a></H%d>\n"
			  level head-count title level))
	(insert (format "\n<H%d>%s</H%d>\n" level title level))))))

(defun org-html-level-close (level)
  (insert "</ul>"))

(defun org-export-html-convert-sub-super (string)
  (let (key c)
    (while (string-match "\\([_^]\\)\\({\\([^}]+\\)}\\|\\(.\\)\\)" string)
      (setq key (if (string= (match-string 1 string) "_") "sub" "sup"))
      (setq c (match-string (if (match-beginning 4) 4 3) string))
      (setq string (replace-match
                    (concat "<" key ">" c "</" key ">")
                    t t string))))
  string)

(defun org-export-html-convert-emphasize (string)
  (while (string-match
          "\\(\\s-\\|^\\)\\(\\*\\([a-zA-Z]+\\)\\*\\)\\([^a-zA-Z*]\\|$\\)"
          string)
    (setq string (replace-match
                  (concat "<i>" (match-string 3 string) "</i>")
                  t t string 2)))
  string)

;; Variable holding the vector with section numbers
(defvar org-section-numbers (make-vector org-level-max 0))

(defun org-init-section-numbers ()
  (let* ((level  -1)
         (numbers (nreverse (split-string "" "\\.")))
         (depth (1- (length org-section-numbers)))
         (i depth) number-string)
    (while (>= i 0)
      (if (> i level)
          (aset org-section-numbers i 0)
        (setq number-string (or (car numbers) "0"))
        (if (string-match "\\`[A-Z]\\'" number-string)
            (aset org-section-numbers i
                  (- (string-to-char number-string) ?A -1))
            (aset org-section-numbers i (string-to-int number-string)))
        (pop numbers))
      (setq i (1- i)))))

(defun org-section-number (&optional level)
  ;; Return a string with the current section number.
  ;; When LEVEL is non-nil, increase section numbers on that level.
  (let* ((depth (1- (length org-section-numbers))) idx n (string ""))
    (when level
      (when (> level -1)
        (aset org-section-numbers
              level (1+ (aref org-section-numbers level))))
      (setq idx (1+ level))
      (while (<= idx depth)
        (if (not (= idx 1))
            (aset org-section-numbers idx 0))
        (setq idx (1+ idx))))
    (setq idx 0)
    (while (<= idx depth)
      (setq n (aref org-section-numbers idx))
      (setq string (concat string (if (not (string= string "")) "." "")
                           (int-to-string n)))
      (setq idx (1+ idx)))
    (save-match-data
      (if (string-match "\\`\\([@0]\\.\\)+" string)
          (setq string (replace-match "" nil nil string)))
      (if (string-match "\\(\\.0\\)+\\'" string)
          (setq string (replace-match "" nil nil string))))
    string))

(provide 'org)

;;; org.el ends here


