;;; randomsig.el --- insert a randomly selected signature

;; Copyright (C) 2001 Hans-Jrgen Ficker

;; Emacs Lisp Archive Entry
;; Author: Hans-Juergen Ficker <hjficker@informatik.uni-bremen.de>
;; Version: 0.1
;; X-CVS-Version: $Id: randomsig.el,v 1.4 2001/04/12 12:31:09 hjficker Exp $
;; Keywords: mail random signature

;; This file is not currently part of GNU Emacs.

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2, or (at
;; your option) any later version.

;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program ; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; This is yet another implementation to insert a randomly choosen 
;; signature into a mail.

;; It is only tested with gnus.

;; To make it work, put the following lines into your ~/.gnus:

;; (require 'randomsig)
;; (define-key message-mode-map "\C-c\C-s" 'randomsig-replace-sig)
;; (define-key gnus-summary-save-map "-" 'gnus/randomsig-summary-read-sig)
;; (setq randomsig-dir "/some/directory")
;; (setq randomsig-files '("some" "files"))

;; `randomsig-files' must be a list of (existing) files. If these
;; don't have absolute paths, they are located in `randomsig-dir'.

;; File format: Each file must contain at least one signature. Each
;; signature must begin with "-- ". The rest of that line is ignored
;; (for compatibility with an implemation of Dirk Meyer
;; <dmeyer@informatik.uni-bremen.de>).  If there is only one signature
;; in the file, the "-- " can be omitted, so real .signature-files can
;; be used.

;; The *-read-sig functions read the signature of a message, or use
;; the marked text, and write it to a signature-file, for which the
;; name is asked. If the file does not exist, it will be generated.

;; if `randomsig-replace-sig' is called with any prefix, it will ask
;; for a file to get the signature from.

;; Completion will only work for files in `randomsig-files', though
;; others files will work, too.

;;; Changelog:

;; 2001/04/12   0.1   Initial release



(defvar randomsig-dir "~/.signatures"
  "Directory for signature-files. See also `randomsig-files'")


;; FIXME: perhaps search through `randomsig-dir'.
(defvar randomsig-files '("default")
  "Files with random signatures.
The files are searched in `randomsig-dir', if they don't have absolute paths.
The signatures have to be separated by lines containing
\"-- \" at the beginning.")



(defvar randomsig-history nil)


(defun randomsig-prompt (&optional prompt)
  (completing-read (if prompt
		       prompt
		     "signature: ")
		   (mapcar 
		    (lambda (x) (list x))
		    randomsig-files)
		   nil
		   nil
		   nil
		   randomsig-history))


(defun randomsig-signature ()
  "Read the files in `randomsig-files', which are searched in `randomsig-path',
and return one random signature from the files"
  (save-excursion

    ;; Make new buffer,
    ;; save possibly local variables `randomsig-files' and `randomsig-path'
    (let '(sigfiles randomsig-files) 
      (let '(sigdir randomsig-dir)
	 (get-buffer-create "*Signatures*")
	 (set-buffer "*Signatures*")
	 (set (make-local-variable 'randomsig-files) sigfiles)
	 (set (make-local-variable 'randomsig-dir) sigdir)))
	 

    ;; Insert files into buffer
    (if (listp randomsig-files)

	(mapcar 
	 (lambda (fname)
	   (insert-file-contents 
	    (expand-file-name fname randomsig-dir)))
	 randomsig-files)
	
      (insert-file-contents 
       (expand-file-name randomsig-files randomsig-dir)))
    

    (goto-char (point-min))
    (let '(count 0) 'selected 
	      
	 ;; Count number of signatures
	 (while (search-forward-regexp "^-- .*$" nil t) 
	   (setq count (1+ count)))

	 ;; Select random signature out out these
	 (setq selected (1+ (random count)))
	 (goto-char (point-min))
	 (if (search-forward-regexp "^-- .*$" nil t selected)
	     (forward-char))

	 ;; Cut signature and return it
	 (let '(here (point)) 'signature-string
	      
	      (if (not (search-forward-regexp "^-- .*$" nil t))
		  (goto-char (point-max))
		(beginning-of-line))
	      (setq signature-string (buffer-substring here (point)))
	      (kill-buffer "*Signatures*")
	      signature-string))))



(setq message-signature 'randomsig-signature)


(defun randomsig-replace-sig (arg)
  "Replace the actual signature with a new one. When called with prefix, 
read the filename of the signature-file that should be used"
  (interactive "P")
  (save-excursion

    ;; FIXME: This is too early. The old signature is lost before the
    ;; new is known to work.
    (if (message-goto-signature)
	(progn
	  (forward-line -1)
	  (backward-char)
	  (delete-region (point) (point-max))))

    (if arg
	(let `(filename (randomsig-prompt "read frm signature-lib: "))

	  (make-local-variable 'randomsig-files)
	  (setq randomsig-files filename)
	  (message-insert-signature)
	  ;; FIXME: perhaps `randomsig-files' or `randomsig-path' were 
	  ;; local before...
	  (kill-local-variable 'randomsig-files))
      (message-insert-signature))))



(defun randomsig-message-read-sig ()
  "Get the signature of current message and copy it to a file"
  (interactive)
  (save-excursion
    (let '(signature-string
	   (if mark-active

	       (buffer-substring (point) (mark))

	     (if (message-goto-signature)
		 (let `(here (point))
		   (goto-char (point-max))
		   (while (char-equal (char-before) 10)
		     (backward-char))
		   (buffer-substring here (point)))
	       nil)))
    
      (if signature-string
	  (progn
	    (set-buffer (find-file-noselect
			 (expand-file-name 
			  (randomsig-prompt "Write to signature-lib: ")
			  randomsig-dir)
			 nil nil nil))
	       
	    (goto-char (point-max))
	    (insert "-- \n") ;; FIXME: more infos?
	    (insert signature-string)
	    (insert "\n")
	    (save-buffer))))))


(defun gnus/randomsig-summary-read-sig ()
  "Get the signature of current message and copy it to a file"
  (interactive)
  (save-excursion
    ;; FIXME: Doesn't return to summary buffer (save-excursion should do this)
    (gnus-summary-select-article-buffer)
    (randomsig-message-read-sig)))
  

(provide 'randomsig)

