;; remember -- Teaching Emacs to act as an extension of your memory
;;
;; Copyright (C) 1999 John Wiegley
;;
;; Author: John Wiegley <johnw@oneworld.new-era.com>
;; Created: 29 Mar 1999
;; Keywords: data memory todo pim schedule storage retrieval
;; X-URL: http://oneworld.new-era.com/johnw/emacs.html

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.


;;; Commentary:
;;
;; @ The idea
;;
;; Todo lists, schedules, phone databases... everything we use
;; databases for is really just a way to extend the power of our
;; memory.  To be able to remember what our conscious mind may not
;; currently have access to.
;;
;; There are many different databases out there -- and good ones --
;; which this mode is not trying to replace.  Rather, it's how that
;; data gets there that's the question.  Most of the time, we just
;; want to say "Remember so-and-so's phone number, or that I have to
;; buy dinner for the cats tonight."  That's the FACT.  How it's
;; stored is really the computer's problem.  But at this point in
;; time, it's most definitely also the user's problem, and sometimes
;; so laboriously so that people just let data slip, rather than
;; expend the effort to record it.
;;
;; "Remember" is a mode for remembering data.  It uses whatever
;; back-end is appropriate to record and correlate the data, but it's
;; main intention is to allow you to express as _little_ structure as
;; possible up front.  If you later want to express more powerful
;; relationships between your data, or state assumptions that were at
;; first too implicit to be recognized, you can "study" the data later
;; and rearrange it.  But the initial "just remember this" impulse
;; should be as close to simply throwing the data at Emacs as
;; possible.
;;
;; @ Implementation
;;
;; Hyperbole, as a data presentation tool, always struck me as being
;; very powerful, but it seemed to require a lot of "front-end" work
;; before that data was really available.  The problem with BBDB, or
;; keeping up a Bibl-mode file, is that you have to use different
;; functions to record the data, and it always takes time to stop what
;; you're doing, format the data in the manner expected by that
;; particular data interface, and then resume your work.
;;
;; With "remember", you just hit `M-x remember' (you'd probably want
;; to bind this to an easily accessible keystroke, like C-x M-r), slam
;; in your text however you like, and then hit C-c C-c.  It will file
;; the data away for later retrieval, and possibly indexing.
;;
;; Indexing is to data what "studying" is in the real world.  What you
;; do when you study (or lucubrate, for some of us) is to realize
;; certain relationships implicit in the data, so that you can make
;; use of those relationships.  Expressing that a certain quote you
;; remembered was a religious quote, and that you want the ability to
;; pull up all quotes of a religious nature, is what studying does.
;; This is a more labor intensive task than the original remembering
;; of the data, and it's typical in real life to set aside a special
;; period of time for doing this work.
;;
;; "Remember" works in the same way.  When you enter data, either by
;; typing it into a buffer, or using the contents of the selected
;; region, it will store that data -- unindexed, uninterpreted -- in a
;; data pool.  It will also try to remember as much context
;; information as possible (any text properties that were set, where
;; you copied it from, when, how, etc).  Later, you can walk through
;; your accumulated set of data (both organized, and unorganized) and
;; easily begin moving things around, and making annotations that will
;; express the full meaning of that data, as far as you know it.
;;
;; Obviously this latter stage is more user-interface intensive, and
;; it would be nice if "remember" could do it as elegantly as
;; possible, rather than requiring a billion keystrokes to reorganize
;; your hierarchy.  Well, as the future arrives, hopefully experience
;; and user feedback will help to make this as intuitive a tool as
;; possible.
;;
;; @ Future Goals
;;
;; This tool hopes to track (and by doing it with as little new
;; code as possible):
;;
;; * The raw data that gets entered
;;
;; * The relationships between that data (either determined
;;   implicitly by parsing the input, or explicitly by the user's
;;   studying the data).
;;
;; * Revisioning of the data
;;
;; * Where it came from, and any context information that can be
;;   programmatically determined.
;;
;; * Allowing particular views of the initially amorphous data pool
;;   (ala the Xanadu concept).
;;
;; * Storage of the data in a manner most appopriate to that data,
;;   such as keeping address-book type information in BBDB, etc.
;;
;; @ Using "remember"
;;
;; As a rough beginning, what I do is to keep my .notes file in
;; outline-mode format, with a final entry called "* Raw data".  Then,
;; at intervals, I can move the data that gets appended there into
;; other places.  But certainly this should evolve into an intuitive
;; mechanism for shuffling data off to its appropriate corner of the
;; universe.
;;
;; Mapping the remember functions to very accessible keystrokes
;; facilities using the mode:
;;
;;  (autoload 'remember "remember" nil t)
;;  (autoload 'remember-region "remember" nil t)
;;
;;  (define-key global-map [f8] 'remember)
;;  (define-key global-map [f9] 'remember)
;;
;; @ Feedback
;;
;; Please send me your comments and ideas.  If Emacs could become a
;; more intelligent data store, where brainstorming would focus on the
;; IDEAS involved -- rather than the structuring and format of those
;; ideas, or having to stop your current flow of work in order to
;; record them -- it would map much more closely to how the mind
;; (well, at least mine) works, and hence would eliminate that very
;; manual-ness which computers from the very beginning have been
;; championed as being able to reduce.
;;
;; Have you ever noticed that having a laptop to write on doesn't
;; _actually_ increase the amount of quality material that you turn
;; out, in the long run?  Perhaps its because the time we save
;; electronically in one way, we're losing electronically in another;
;; the tool should never dominate one's focus.  As the mystic
;; Faridu'd-Din `Attar wrote: "Be occupied as little as possible with
;; things of the outer world but much with things of the inner world;
;; then right action will overcome inaction."


;;; Header:

(provide 'remember)

(defconst remember-version "0.1"
  "This version of remember.")

(defgroup remember nil
  "A mode to remember information."
  :group 'data)


;;; User Variables:

(defcustom remember-data-file "~/.notes"
  "*The file in which to store unprocessed data."
  :type 'file
  :group 'remember)

(defcustom remember-register ?R
  "The register in which the window configuration is stored."
  :type 'character
  :group 'remember)

(defcustom remember-mode-hook nil
  "*A series of function to be run upon entering remember-mode."
  :type 'hook
  :group 'remember)


;;; Internal Variables:

(defvar remember-buffer "*Remember*"
  "The name of the remember data entry buffer.")


;;; User Functions:

;;;###autoload
(defun remember ()
  "Remember an arbitrary piece of data."
  (interactive)
  (window-configuration-to-register remember-register)
  (let ((buf (get-buffer-create remember-buffer)))
    (switch-to-buffer-other-window buf)
    (remember-mode)
    (message "Use C-c C-c to remember the data.")))

;;;###autoload
(defun remember-region (&optional beg end)
  "Remember the data from BEG to END.
If called from within the remember buffer, BEG and END are ignored,
and the entire buffer will be remembered.  If called from any other
buffer, that region, plus any context information specific to that
region, will be remembered."
  (interactive)
  (let ((orig-buf (current-buffer))
        (mem-buf  (get-buffer-create remember-buffer))
        from-other)
    (if (eq orig-buf mem-buf)
        t
      (setq from-other t)
      (let ((b (or beg (min (point) (mark))))
            (e (or end (max (point) (mark)))))
        (copy-to-buffer mem-buf b e)))
    (save-excursion
      (set-buffer mem-buf)
      (goto-char (point-min))
      (insert "!---begin---!\n")
      (goto-char (point-max))
      (insert "\n")
      (insert "!---variables---!\n")
      (insert "Created: " (current-time-string) "\n")
      (insert "!---end---!\n")
      (append-to-file (point-min) (point-max) remember-data-file))
    (kill-buffer mem-buf)
    (if (not from-other)
        (jump-to-register remember-register))))


;;; Internal Functions:

(defvar remember-mode-map ())
(if remember-mode-map
    ()
  (setq remember-mode-map (make-sparse-keymap))
  (define-key remember-mode-map "\C-c\C-c" 'remember-region))

(defun remember-mode ()
  "Major mode for output from \\[remember].
\\<remember-mode-map>This buffer is used to collect data that you want
remember.  Just hit \\[remember-mode-remember] when your done
entering, and it will go ahead and file the data for latter retrieval,
and possible indexing.
\\{remember-mode-map}"
  (kill-all-local-variables)
  (use-local-map remember-mode-map)
  (setq major-mode 'remember-mode)
  (setq mode-name "Remember")
  (run-hooks 'remember-mode-hook))


;;; remember.el ends here

