;;; rfc-util.el --- RFC-util interface for emacs.

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: rfc
;; Version: 1.0.2

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; An interface to rfc-util to download RFC's and view them.  Keeps RFCs on disk
;; so that you can view them in the future.  You will need to install rfc-util
;; in order to use this.

;; You can download rfc-util from http://www.dewn.com/rfc/

;;; TODO:

;; - add a new function rfc-search for searching your local RFC index.  This
;; should then popup a new buffer for you to select a result.

;;; History:
;; 
;; - Tue Apr 24 21:10:06 2001 (burton@relativity.yi.org): added the ability to
;; search the RFC index.
;;
;; - Tue Apr 24 17:27:51 2001 (burton@relativity.yi.org): added an
;; rfc-local-index function which allows you to quickly look at what RFCs you
;; have already downloaded and jump to them.  This is really helpful as it helps
;; keep a set of 'bookmarks' of RFCs which you find interesting.  It is also
;; very nice when you are on a laptop without an Internet connection and you
;; want to see what RFCs you have local if you need to lookup something.

;;; Code:
(defvar rfc-util-directory "~/.rfc-util/" 
  "Destination to store downloaded RFCs.")

(defvar rfc-search-buffer "*rfc-search-buffer*" 
  "Buffer used to show the index of buffered RFCs.")


(defvar rfc-local-index-buffer "*rfc-local-index-buffer*" 
  "Buffer used to show the index of buffered RFCs.")

;;FIXME: make it a defvar
(defvar rfc-util-mode-font-lock-keywords
  '(("^[0-9]+"
     (0 'font-lock-variable-name-face append))
    ("^---------------------------------------------------------------------------------$"
     (0 'font-lock-comment-face append)))
  "Font lock information.")
  
(defun rfc-search(search-string)
  (interactive
   (list 
    (read-string "Search for: ")))

  (set-buffer (get-buffer-create rfc-search-buffer))

  (toggle-read-only -1)
  (erase-buffer)
  
  (insert (shell-command-to-string (concat "rfc -s " search-string)))

  (goto-char (point-min))

  (rfc-util-mode)
  
  (pop-to-buffer rfc-search-buffer))

(defun rfc()
  "Prompt for an rfc number and display it in a new buffer"
  (interactive)

  (if (not (file-exists-p rfc-util-directory))
      (make-directory rfc-util-directory))
  
  (setq rfc-number (read-string "What RFC number: "))

  ;;no reason to download if the file already exists
  (if (not (file-readable-p (concat rfc-util-directory "/rfc" rfc-number ".txt")))
    (rfc-util-fetch rfc-number))

  (rfc-util-open rfc-number))

(defun rfc-local-index-open-at-point()
  "Open the RFC at the point"
  (interactive)
  
  (save-excursion
    (let(rfc-number)
      (goto-char (point-at-eol))

      (if (re-search-backward "^[0-9]+")
          (progn
            
            (setq rfc-number (string-to-number (buffer-substring (match-beginning 0) 
                                                                 (match-end 0))))
            (rfc-util-display rfc-number))

        (error "No RFC on the current line")))))

(defun rfc-local-index()
  "Show an index of the RFCs available locally."
  (interactive)

  (message "Generating rfc index...")
  
  (set-buffer (get-buffer-create rfc-local-index-buffer))

  (toggle-read-only -1)
  
  (erase-buffer)
  
  (let((i 0)
       files filename rfc-number)
    (setq files (directory-files rfc-util-directory))

    (while (< i (length files))
      
      (setq filename (elt files i))

      (if (string-match ".*rfc[0-9]+\\.txt$" filename)
          (progn
            ;;(insert filename)

            ;;pull out the rfc number from the filename

            (string-match "[0-9]+" filename)

            (setq rfc-number (substring filename
                                        (match-beginning 0)
                                        (match-end 0)))
            
            (insert (shell-command-to-string (concat "rfc " rfc-number)))))

      (setq i (1+ i))))


  ;;clean up the buffer
  (goto-char (point-min))

  ;;remove the result identifiers
  (save-excursion
    (while (re-search-forward "^The Result:$" nil t)
      (replace-match "---------------------------------------------------------------------------------")))
  
  (pop-to-buffer rfc-local-index-buffer)

  (rfc-util-mode)
  
  (message "Generating rfc index...done" ))

(defun rfc-util-open(rfc-number)
  "Open the given RFC by rfc nymber.  It just be already available on the
system."
  
  (find-file (rfc-util-filename rfc-number))
  (goto-char (point-min)))

(defun rfc-util-filename(rfc-number)
  "Get a correct filename from the given rfc-number."

  (let(filename)

    (setq filename (concat rfc-util-directory "/rfc" rfc-number ".txt"))

    ;;if the file does not exist it is probably remote.
    (if (and (not (file-exists-p filename))
             (yes-or-no-p (format "RFC %s is not on disk.  Download? "
                                  rfc-number)))
        (rfc-util-fetch rfc-number))
        
    filename))

(defun rfc-util-display(rfc-number)
  "Display the RFC but in a second window."

  (display-buffer (find-file-noselect 
                   (rfc-util-filename rfc-number))))

(defun rfc-util-fetch(rfc-number)

  (message "Fetching RFC... please wait.")

  (let(command)

    (setq command (concat "rfc -d "
                          rfc-number
                          " "
                          rfc-util-directory
                          " > /dev/null"))

    (shell-command command))
  
  (message "DONE fetching RFC"))

(define-derived-mode rfc-util-mode fundamental-mode "RFCIndex"
  "Major mode for Java property files."

  (toggle-read-only 1)

  ;;(make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(rfc-util-mode-font-lock-keywords))
  
  (font-lock-mode 1))

;;setup key bindings for the major mode.
(define-key rfc-util-mode-map  [return] 'rfc-local-index-open-at-point)

(provide 'rfc-util)

;;; rfc-util.el ends here
