;;; setnu.el --- vi-style line number mode for Emacs

;;; Copyright (C) 1994, 1995, 1997 Kyle E. Jones
;;; Copyright (C) 2002 Kevin A. Burton (burton@peerfear.org)
;;;
;;; This program is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by the Free
;;; Software Foundation; either version 2, or (at your option) any later
;;; version.
;;;
;;; This program is distributed in the hope that it will be useful, but WITHOUT
;;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;;; more details.
;;;
;;; A copy of the GNU General Public License can be obtained from this program's
;;; author (send electronic mail to kyle@uunet.uu.net) or from the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;;
;;; Send bug reports to burton@peerfear.org

;;; Commentary: 

;; M-x setnu-mode toggles the line number mode on and off.
;;
;; turn-on-setnu-mode is useful for adding to a major-mode hook
;; variable.
;;
;; Example:
;;     (add-hook 'text-mode-hook 'turn-on-setnu-mode)
;;
;; to automatically turn on line numbering when enterting text-mode."

;;; TODO (Kevin Burton)
;;
;; - remove all references to glyphs becasue this is confusing.
;;
;; - BUG: if something comes in and modifies an open buffer (without us
;; knowning) we wont execute any before|after-change-functions.  Example: we
;; have .emacs open, go into customization, change some variables, save and then
;; the open .emacs file is bonked.
;;
;;    - NOTE: it doesn't look like this is a problem with insert or
;;    anything.. maybe it is 'pp or 'prin1
;;
;; - play with using permanent-local properties on symbols so that
;; kill-all-local-variables still keeps setnu functioning
;;
;; - Why does it break on multiline intangible text?

;;; History:
;;
;; - Thu Dec 19 2002 01:28 PM (burton@universe): added after-revert-hooks so
;; that the buffer isn't destroyed with excessive overlays.q
;;
;; - Thu Dec 19 2002 01:28 PM (burton@universe): refactored code to be cleaner
;; and easier to work with.

;;; Code:

(defconst setnu-mode-version "1.06"
  "Version number for this release of setnu-mode.")

(defvar setnu-mode nil
  "Non-nil if setnu-mode is active in the current buffer.")
(make-variable-buffer-local 'setnu-mode)
(put 'setnu-mode 'permanent-local t)

(defvar setnu-start-extent nil
  "First extent of a chain of extents used by setnu-mode.
Each line has its own extent.  Each line extent has a
`setnu-next-extent' property that points to the next extent in
the chain, which is the extent for the next line in the buffer.
There is also a `setnu-prev-extent' that points at the previous
extent in the chain.  To distinguish them from other extents the
setnu-mode extents all have a non-nil `setnu' property.")
(make-variable-buffer-local 'setnu-start-extent)
(put 'setnu-start-extent 'permanent-local t)

(defvar setnu-glyph-obarray (make-vector 401 0)
  "Obarray of symbols whose values are line number glyphs.
Each symbol name is the string represnetation of a number, perhaps
passed with spaces.  The value of the symbol is a glyph that can
be made the begin glyph of an extent to display as a line number.")

(defvar setnu-begin-glyph-property 'before-string
  "Property name to use to set the begin glyph of an extent.")

(defcustom setnu-line-number-format "%6d  "
  "String suitable for `format' that will generate a line number string.
`format' will be called with this string and one other argument
which will be an integer, the line number."
  :group 'setnu
  :type 'string)

(defcustom setnu-automatic-for-open-files nil
  "If true `setnu-mode' is enabled for all files opened with `find-file'"
  :group 'setnu
  :type 'boolean)

(defface setnu-line-number-face '((t (:foreground "medium aquamarine" :background "gray10" :bold t)))
  "*Face used to display the line numbers."
  :group 'setnu)

(defun setnu-mode (&optional arg)
  "Toggle setnu-mode.
With prefix argument, turn setnu-mode on if argument is positive.
When setnu-mode is enabled, a line number will appear at the left
margin of each line."
  (interactive "P")
  (let ((oldmode (not (not setnu-mode)))
        (inhibit-quit t))
    (setq setnu-mode (or (and arg (> (prefix-numeric-value arg) 0))
                         (and (null arg) (null setnu-mode))))
    (if (not (eq oldmode setnu-mode))
        (if setnu-mode
            (setnu-mode-on)
          (setnu-mode-off)))))

(defun turn-on-setnu-mode ()
  "Turn on setnu-mode.
Useful for adding to a major-mode hook variable.
Example:
    (add-hook 'text-mode-hook 'turn-on-setnu-mode)
to automatically turn on line numbering when enterting text-mode."
  (setnu-mode 1))

;;; Internal functions

;;; The program is written using XEmacs terminology,
;;; e.g. extents, glyphs, etc.  Functions are defined to twist
;;; the FSF Emacs overlay API into the XEmacs model.

(defun setnu-set-extent-begin-glyph (overlay glyph)
  "Set the before string on this overlay `overlay' with the glyph `glyph'."
  (overlay-put overlay setnu-begin-glyph-property glyph))

;;FIXME: I don't know what to do about this under FSF Emacs
(fset 'setnu-make-glyph 'identity)

(defun setnu-set-glyph-face (g face)
  "Set the face for the given glyph."
  
  (put-text-property 0 (length g) 'face face g))

(defun setnu-mode-off ()
  "Internal shutdown of setnu-mode.
Deletes the extents associated with setnu-mode."

  (if setnu-start-extent
      (let (e ee)
        (setq e setnu-start-extent)
        (while e
          (setq ee e)
          (setq e (overlay-get e 'setnu-next-extent))
          (delete-overlay ee))
        (setq setnu-start-extent nil))))

(defun setnu-mode-on ()
  "Internal startup of setnu-mode.
Sets up the extents associated with setnu-mode."

  ;;FIXME: make sure that we have the hooks within our X-change-functions and if
  ;;not add them.

  (when (not (member 'setnu-before-change-function before-change-functions))
    (message "Warning - no setnu-before-change-function"))
  
  (let ((done nil)
        (curr-e nil)
        (n 1)
        (match-data (match-data))
        e start numstr)
    (unwind-protect
        (save-excursion
          (save-restriction
            (widen)
            (goto-char (point-min))
            (setq start (point))
            (while (not done)
              (setq done (null (search-forward "\n" nil 0)))
              (setq e (setnu-make-setnu-extent start (point)))
              (if (null setnu-start-extent)
                  (setq setnu-start-extent e
                        curr-e e)
                (overlay-put curr-e 'setnu-next-extent e)
                (overlay-put e 'setnu-prev-extent curr-e)
                (setq curr-e e))
              (setq numstr (format setnu-line-number-format n))
              (overlay-put e 'line-number numstr)
              (setnu-set-extent-begin-glyph e (setnu-number-glyph numstr))
              (setq n (1+ n)
                    start (point)))))
      (store-match-data match-data))))

(defun setnu-before-change-function (start end)
  "Before change function for setnu-mode.
Notices when a delete is about to delete some lines and adjusts
the line number extents accordingly."
  (if (or (not setnu-mode) (= start end))
      () ;; not in setnu-mode or this is an insertion
    (let ((inhibit-quit t)
          (start-e nil)
          (match-data (match-data))
          end-e saved-next e ee)
      (unwind-protect
          (save-excursion
            (save-restriction
              (widen)
              (goto-char start)
              (if (search-forward "\n" end t)
                  (progn
                    (setq start-e (setnu-extent-at-create start nil)
                          saved-next (overlay-get
                                      start-e
                                      'setnu-next-extent))
                    (setq end-e (setnu-extent-at-create end nil))
                    (move-overlay
                     start-e
                     (overlay-start start-e)
                     (overlay-end end-e))
                    (overlay-put
                     start-e 'setnu-next-extent
                     (overlay-get end-e 'setnu-next-extent))))
              (if start-e
                  (progn
                    (setq e (overlay-get start-e 'setnu-next-extent)
                          ee saved-next)
                    (while (and e (overlay-get e 'setnu-next-extent))
                      (setq e (overlay-get e 'setnu-next-extent)
                            ee (overlay-get ee 'setnu-next-extent)))
                    (while (and e (not (eq ee start-e)))
                      (setnu-set-extent-begin-glyph
                       e (overlay-get ee setnu-begin-glyph-property))
                      (overlay-put e
                                   'line-number
                                   (overlay-get ee 'line-number))

                      (setq e (overlay-get e 'setnu-prev-extent)
                            ee (overlay-get ee 'setnu-prev-extent)))
                    (setq end-e (overlay-get start-e
                                             'setnu-next-extent))
                    (and end-e
                         (overlay-put end-e
                                                    'setnu-prev-extent
                                                    start-e))
                    (setq e saved-next)
                    (while (not (eq e end-e))
                      (setq ee e
                            e (overlay-get e 'setnu-next-extent))
                      (delete-overlay ee))))))
        (store-match-data match-data)))))

(defun setnu-after-change-function (start end length)
  "After change function for setnu-mode.
Notices when an insert has added some lines and adjusts
the line number extents accordingly."
  (if (or (not setnu-mode) (= start end))
      ()                               ; not in setnu-mode or this is a deletion
    (let ((inhibit-quit t)
          (ee nil)
          (match-data (match-data))
          (new-lines 0)
          start-e e saved-end saved-next n numstr)
      (unwind-protect
          (save-excursion
            (save-restriction
              (widen)
              (setq start-e (setnu-extent-at-create start nil))
              (if (< (overlay-end start-e) (point))
                  ;; bogus!  insertion didn't put the text into
                  ;; the extent because,
                  ;; a. the extent was zero length or
                  ;; b. this is FSF Emacs which means chars
                  ;;    inserted at the end position of an extent
                  ;;    are not inserted into the extent.
                  (move-overlay
                   start-e
                   (overlay-start start-e)
                   end))
              (setq saved-next (overlay-get start-e
                                            'setnu-next-extent)
                    saved-end (overlay-end start-e)
                    e start-e)
              (goto-char start)
              (while (search-forward "\n" end 0)
                (move-overlay e
                                            (overlay-start e)
                                            (point))
                (setq ee (setnu-make-setnu-extent (point) (point)))
                (overlay-put e 'setnu-next-extent ee)
                (overlay-put ee 'setnu-prev-extent e)
                (setq e ee new-lines (1+ new-lines)))
              (if ee
                  (progn
                    (move-overlay
                     e (overlay-start e) saved-end)
                    (overlay-put e 'setnu-next-extent saved-next)
                    (and saved-next
                         (overlay-put
                          saved-next 'setnu-prev-extent e))
                    (setq e (overlay-get start-e 'setnu-next-extent)
                          ee saved-next)
                    (while ee
                      (setnu-set-extent-begin-glyph
                       e (overlay-get ee setnu-begin-glyph-property))
                      (overlay-put
                       e 'line-number (overlay-get ee 'line-number))
                      (setq e (overlay-get e 'setnu-next-extent)
                            ee (overlay-get ee 'setnu-next-extent)))
                    (setq n (1+ (string-to-int
                                 (overlay-get
                                  (overlay-get e 'setnu-prev-extent)
                                  'line-number))))
                    (while e
                      (setq numstr (format setnu-line-number-format n))
                      (overlay-put e 'line-number numstr)
                      (setnu-set-extent-begin-glyph
                       e (setnu-number-glyph numstr))
                      (setq e (overlay-get e 'setnu-next-extent)
                            n (1+ n)))))))
        (store-match-data match-data)))))

(defun setnu-number-glyph (number-string)
  "FIXME: need documentation."
  
  (let ((sym (intern number-string setnu-glyph-obarray)))
    (if (boundp sym)
        (symbol-value sym)
      (let ((g (setnu-make-glyph number-string)))
        (set sym g)
        (setnu-set-glyph-face g 'setnu-line-number-face)
        g ))))

(defun setnu-make-setnu-extent (beg end)
  "Create an extent and set some properties that all setnu extents have."
  (let ((e (make-overlay beg end)))
    (overlay-put e 'setnu t)
    ;;    (overlay-put e 'begin-glyph-layout 'outside-margin)
    (overlay-put e 'detachable nil)
    (overlay-put e 'evaporate nil)
    e ))

(defun setnu-extent-at (pos buf)
  "Finds the setnu extent at the position POS in the buffer BUF."
  (catch 'done
    (save-excursion
      (and buf (set-buffer buf))
      (let ((o-list (overlays-in pos (1+ pos))))
        (while o-list
          (if (overlay-get (car o-list) 'setnu)
              (throw 'done (car o-list)))
          (setq o-list (cdr o-list)))
        nil ))))

(defun setnu-extent-at-create (pos buf)
  "Like `setnu-extent-at' except if an extent isn't found, then
it is created based on where the extent failed to be found."
  (let ((e (setnu-extent-at pos buf)) ee beg numstr)
    (if e
        e
      ;; no extent found so one must be created.
      (save-excursion
        (goto-char pos)
        (goto-char (point-at-bol))
        (setq e (setnu-extent-at (point) buf))
        (cond (e
               ;; found one.  extend it to cover this whole line.
               ;; this takes care of zero length extents that
               ;; might exist at bob or eob that can't be
               ;; inserted into.
               (setq beg (point))
               (forward-line 1)
               (move-overlay e beg (point))
               e )
              ((bobp)
               ;; we are at bob and there's no extent.
               ;;
               ;; this is because the extent that was there got
               ;; detached because all the text in the buffer was
               ;; deleted.  so we create a new extent and make it
               ;; contain the whole buffer, since there can be no
               ;; other attached extents.
               (setq e (setnu-make-setnu-extent (point-min) (point-max))
                     numstr (format setnu-line-number-format 1))
               (overlay-put e 'line-number numstr)
               (setnu-set-extent-begin-glyph e (setnu-number-glyph numstr))
               (setq setnu-start-extent e)
               e )
              (t
               ;; we must be at eob and there's no extent.
               ;; 
               ;; this is because the extent that was there
               ;; shrank to zero length and was detached.  create
               ;; a new extent that contains all text from point
               ;; to pos.
               (setq e (setnu-make-setnu-extent (point) pos))
               (setq ee (setnu-extent-at (1- (point)) buf))
               (overlay-put e 'setnu-prev-extent ee)
               (overlay-put ee 'setnu-next-extent e)
               (setq numstr
                     (format setnu-line-number-format
                             (1+ (string-to-int
                                  (overlay-get ee 'line-number)))))
               (overlay-put e 'line-number numstr)
               (setnu-set-extent-begin-glyph e (setnu-number-glyph numstr))
               e ))))))

(defun setnu-find-file-hook()
   "If `setnu-automatic-for-open-files' is enabled we will start `setnu-mode'
   for all files opened with `find-file'."

   (when setnu-automatic-for-open-files
     (setnu-mode 1)))

(defun setnu-change-major-mode-hook()
  "Hook to run before we change a major mode.  This is required because if we
don't turn *off* setnu it will break."

  (when setnu-mode
    ;;is there a way to turn setnu back on?  Maybe with a post-command-hook?

    (setnu-mode-off)))

(defun setnu-after-revert-hook()
  "Hook to run after a buffer is reverted.  We must run this or we will
*totally* destroy the buffer with invalid overlays."

  (when setnu-mode
    (setnu-mode -1)
    (setnu-mode 1)))

(add-hook 'after-revert-hook 'setnu-after-revert-hook)

;;This is probably necessary because we are about to kill off local variables.
(add-hook 'change-major-mode-hook 'setnu-change-major-mode-hook)

(add-hook 'before-change-functions 'setnu-before-change-function)
(add-hook 'after-change-functions 'setnu-after-change-function)

(add-hook 'find-file-hooks 'setnu-find-file-hook)

(provide 'setnu)

;;; setnu.el ends here