;;; timelog.el ---  log and summarize your working times in useful ways

;; Copyright (C) 2000, 2001 Thomas Gehrlein <thomas.gehrlein@t-online.de>

;; Emacs Lisp Archive Entry
;; Filename: timelog.el
;; Version: 1.2
;; Keywords: calendar data time
;; Author: Thomas Gehrlein <thomas.gehrlein@t-online.de>
;; Maintainer: Thomas Gehrlein <thomas.gehrlein@t-online.de>
;; Description: Keep track of the time you work
;; URL: sorry, no URL
;; Compatibility: Emacs20

;; This file is not part of GNU Emacs.
;;
;; This is free software; you can redistribute it and/or modify it under the
;; terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or (at your option) any later
;; version.
;;
;; This is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:

;; This file helps you to keep track of what you do.  It writes information
;; into a file.  Each line in this file consists of a code letter (i or o - "i"
;; means "in", i.e. starting an activity, "o" means "out", i.e. finishing an
;; activity), a date and time stamp, and information supplied by the user.
;; Example (w/o the ";;", of course):

;; i 2001/02/21 11:07:00 timelog
;; o 2001/02/21 11:53:23 code writing
;; i 2001/02/21 13:46:34 timelog
;; o 2001/02/21 14:12:12 debugging
;; i 2001/02/21 14:24:23 timelog
;; o 2001/02/21 14:33:56 debugging

;; The data in this file can then be summarized in amazing ways.

;; timelog.el is very much a work-alike of John Wiegley's timeclock.el.  If you
;; want more advanced features, try timeclock.el.  I'd like to thank John for
;; writing timeclock.el.
;;
;; I wrote timelog.el for fun and to learn a bit about elisp and programming.
;; Beware, I pretty much make up my elisp as I go along.
;;
;; Comments are welcome.  Especially appreciated: "Your code works, but this is
;; not the best/canonical way to do it ..." "Have you considered using
;; some-wonderful-function-that-is-absolutely-perfect-for-what-you-need."
;; "Your code violates ancient elisp coding traditions, like ..."


;;; RESTRICTIONS/PROBLEMS/WORKAROUNDS
;; To enter a SPC in the completion-minibuffer use C-q SPC or (slightly less
;; annoying) M-SPC (bound to just-one-space).  (Thanks to
;; <Kai.Grossjohann@CS.Uni-Dortmund.DE>)
;;
;; The time format in the timelog file cannot be changed.  Project and
;; subproject entries cannot include newlines.
;;
;; Blank lines in the timelog file confuse timelog.el.  So better avoid them
;; when you edit the file manually.
;;
;; timelog.el supports only one timelog file.  Support for multiple files is
;; not planned.  (Workaround: set the variable timelog-file to a new value and
;; then timelog-reread-logfile).  Note that it is possible to have several
;; projects in your timelog-file
;;
;; timelog-summarize-project doesn't work properly if you log out and forget to
;; enter a subproject.  It cannot handle "" as subproject string.  If your
;; timelog-file has missing projects or subprojects, use
;; timelog-add-omitted-projects first.  (Maybe check for it?)


;;; TODO (POSSIBLE IMPROVEMENTS/RANDOM THOUGHTS)
;;
;; timelog.el is feature complete, as far as I am concerned.  I do not plan do
;; add more functions in the future.  (Except for someone special of course.)
;;
;; Add-function timelog-remove-blank-lines-in-timelog-file?
;;
;; timelog-make-subproject-list and timelog-read-subprojects could be one
;; function ("Make a list of all subprojects in TIMELOG-FILE.  If optional arg
;; PROJECT, make a list of subprojects of PROJECT only.")
;;
;; Add defun for summarizing the work times of all days in ~/.timelog.  Useful
;; for finding out how much you really worked, if you work on more than one
;; projects.

 
;;; SETUP AND KEY-BINDINGS
;; Put timelog where your load-path finds it.
;;
;; Key-bindings as suggested by John Wiegley in timeclock.el.  Add something
;; like the following lines to your .emacs.  (Remove the comments (;;) first.)
;; Type C-x C-e at the end of each line.  Enjoy.
;; (require 'timelog)
;; (define-key ctl-x-map "ti" 'timelog-in)
;; (define-key ctl-x-map "to" 'timelog-out)
;; (define-key ctl-x-map "tc" 'timelog-change)
;; (define-key ctl-x-map "ts" 'timelog-make-summary)
;; (define-key ctl-x-map "tr" 'timelog-reread-logfile)
;;
;; If your timelog-file contains entries created by timelog.el v1.1 or earlier,
;; do M-x timelog-add-seconds.

;; Here are the settings in my .emacs:
;; (setq timelog-hourly-rate 50
;;       timelog-currency-string "DM"
;;       timelog-currency-separator-string ","
;;       timelog-currency-decimal-string "."
;;       timelog-summary-function 'timelog-my-summarize-project)


;;; Commentary:
;; 

;;; HISTORY
;; timelog 0.1
;; First draft.  No good.
;;
;; timelog 0.2
;; Initial revision.  It works.  Tested by some people.
;;
;; timelog 0.3
;; Minor bugfixes and code cleaning.  All functions read their args with completion.
;; (Except where it doesn't make sense).  The summary layout can be customized
;; in amazing ways.
;;
;; timelog 0.4
;; Uses defcustom instead of defvars.  checkdoc'd.
;;
;; timelog 1.0
;; Added timelog-make-summary.  timelog know does what I want it to do.
;;
;; timelog 1.1
;; code cleaning.  Posted to gnu.emacs.sources
;;
;; timelog 1.1a
;; Prepared for change to new timelog-file format.
;;
;; timelog 1.2
;; Changed timelog-file format (hh:mm -> hh:mm:ss).  timelog is now 60 times
;; more precise than before!!.  Fixed bug in timelog-time-of-subproject.  (See
;; comments).


;;; Code:
;; User Variables:
(defgroup timelog nil
  "Logging and evaluating time spent on projects."
  :group 'data)

(defcustom timelog-file "~/.timelog"
  "*File to store timelog information."
  :type 'file
  :group 'timelog)

(defcustom timelog-summary-include-subprojects t
  "*Non-nil means:  Include time spent on subprojects in the summary."
  :type 'boolean
  :group 'timelog)

(defcustom timelog-summary-include-logfile t
  "*Non-nil means:  Include relevant lines from `timelog-file' in the summary."
  :type 'boolean
  :group 'timelog)

(defcustom timelog-summary-include-days t
  "*Non-nil means:  Include time spent on each day in the summary."
  :type 'boolean
  :group 'timelog)

(defcustom timelog-summary-include-rates t
  "*Non-nil means: Include your rates in the summary.

Adds for overall-time and for every subproject a string that indicates how much
you charge for this service.  The rate is computed using TIMELOG-HOURLY-RATE.
The currency is determined by TIMELOG-CURRENCY."
  :type 'boolean
  :group 'timelog)

(defcustom timelog-hourly-rate 100
  "*Number of currency units you charge for one hour of your time."
  :type 'integer
  :group 'timelog)

(defcustom timelog-currency-string "US-$"
  "*The currency you charge your rates in."
  :type 'string
  :group 'timelog)

(defcustom timelog-currency-separator-string "."
  "*String that separates big currency units from small currency units.

Like dollars and cents.  They use \".\".  We use \",\"."
  :type 'string
  :group 'timelog)

(defcustom timelog-currency-decimal-string ","
  "*String that separates big currency units in groups of three.

Like 1,234,567.  They use \",\".  We use \".\"."
  :type 'string
  :group 'timelog)

(defcustom timelog-summary-function 'timelog-summarize-project
  "*Function for summarizing a project.

If you don't like the default summary by `timelog-summarize-project' write your
own function.  See `timelog-my-summarize-project' for an example."
  :type 'function
  :group 'timelog)
 
(defcustom timelog-out-with-message t
  "*Non-nil means: When logging out, indicate how much time you worked."
  :type 'boolean
  :group 'timelog)

;;; Internal Variables:
(defvar timelog-version "1.2"
  "This version of timelog.")

(defvar timelog-last-event nil
  "Last logging event: in or out.  \"i\" is \"in\" and \"o\" is \"out\".")

;; You don't always get the last project with (car timelog-project-list)
(defvar timelog-last-project nil
  "The project you specified when you last logged in.")

(defvar timelog-last-subproject nil
  "The subproject you specified when you last logged out.")

(defvar timelog-project-list nil
  "List of timelog projects, is read from the `timelog-file'.
Used for completion (and maybe other things as well).")

(defvar timelog-subproject-list nil
  "List of things that are parts of projects.  Used for completion and so.
There is only one list for all projects.")

(defvar timelog-time-string "%Y/%m/%d %H:%M:%S"
  "String used by `format-time-string'.
Don't change it.  There's so much that depends on it.")

 
;;; File conversion
;; timeclock.el by John Wiegley <johnw@gnu.org> uses a _slightly_ different
;; format: it records the current time as hh:mm:ss (timelog-2.el uses hh:mm)
;; (Note: This has changes as of version 1.2.)

(defun timelog-remove-seconds ()
  "This function is obsolete as of version 1.2.

Remove the seconds from a timeclock file to convert it into a timelog file:
hh:mm:ss -> hh:mm"

  (interactive)
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))
    ;; search for seconds and remove them
    (while
	(re-search-forward
;;	 "^[io] [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9] [0-9][0-9]:[0-9][0-9]"
	 "^[io] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+"
					;"i 2000/10/17 12:34" or
					;"o 2000/10/17 12:34" at BOL
	 nil t)
      (delete-char 3))
    ;;(bury-buffer)))
    ))


(defun timelog-add-seconds ()
  "Convert timelog file from old format to new (hh:mm to hh:mm:ss)."
  (interactive)
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))
    ;; search for where seconds should be and insert them
    (while
	(re-search-forward
;;	 "^[io] [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9] [0-9][0-9]:[0-9][0-9]"
	 "^[io] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+"
					;"i 2000/10/17 12:34" or
					;"o 2000/10/17 12:34" at BOL
	 nil t)
      (insert ":00"))
    ;;(bury-buffer)))
    ))


(defun timelog-add-omitted-projects ()
  "Find lines with date and time but no project or subproject in `timelog-file'.

Enter some string.  `timelog-summarize-project' doesn't work properly with
missing project and subproject names."
  (interactive)
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    (while
	(re-search-forward
;; 	 "^[io] [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9] \
;; [0-9][0-9]:[0-9][0-9]$" nil t)
;;	 "^[io] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+$" nil t)
	 "^[ioO] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]$" nil t)
      (insert " nn"))
    ;;(bury-buffer)))
    ))


;;; reading information from the timelog-file

(defun timelog-make-project-list ()
  "Make a list of projects from the TIMELOG-FILE.  Set `timelog-last-project'.

A project is everything you specify, when you log in."
  (interactive)
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))
    ;; make the list (search for anything that comes after a time string at BOL
    ;; and before EOL or EOF)
    (while
	(re-search-forward
;;;	 "^i [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9] [0-9][0-9]:[0-9][0-9] "
	 "^i [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]+ "
					;"i 2001/03/22 12:34:56 " at BOL


	 nil t)
      (add-to-list 'timelog-project-list
		   (buffer-substring (match-end 0) (search-forward-regexp
						    "$"))))
    (setq timelog-last-project (car timelog-project-list))
    ;;(bury-buffer)))
    ))


(defun timelog-make-subproject-list ()
  "Make a list of subprojects from the `timelog-file'.

Sets `timelog-last-subproject'.  A subproject is everything you specify, when
you log out, i.e. what you've been working on."
  (interactive)
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))
    ;; make the list (search for anything that comes after a time string at BOL
    ;; and before EOL or EOF)
    (while
	(re-search-forward
;; "^o [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9] [0-9][0-9]:[0-9][0-9] "
 	  "^[oO] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]+ "
	   					;"o 2000/10/17 12:34 " at BOL

	 nil t)
      (add-to-list 'timelog-subproject-list
		   (buffer-substring (match-end 0) (search-forward-regexp
						    "$"))))
    (setq timelog-last-subproject (car timelog-subproject-list))
    ;;(bury-buffer)))
    ))


(defun timelog-reread-logfile ()
  "Read the `timelog-file' again.

Make new `timelog-project-list' and `timelog-subproject-list'."
  (interactive)
  (setq timelog-project-list ()
	timelog-subproject-list ())
  (timelog-make-project-list)
  (timelog-make-subproject-list))


(defun timelog-read-project (project)
  "Make a list of all entries for PROJECT and return it.

PROJECT should be an existing project in `timelog-file'.  Each entry consists of
2 lines."
  (interactive (list
		(completing-read "Project to read: " (mapcar 'list timelog-project-list)
				 nil	; no predicate
				 t)))	; require-match
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))
    ;; Search for project, then search backward for "^i", then search forward
    ;; for "^[oO] .*$" (end of a line that starts with "o " or "O " and take
    ;; the buffer-substring between these two points.  Append it to a list.

    (let ((project-list ()))
      (while
	  (re-search-forward project nil t) ; search for the project
	(setq project-list
	      (append			; we append (list + new-element) ->
					; don't need nreverse
	       project-list
	       (list (buffer-substring
		      (search-backward-regexp "^i")
		      ;;(search-forward-regexp "^o .*$"))))))
		      (search-forward-regexp "^[oO] .*$"))))))

      ;;(bury-buffer)

      ;; return what we came here for (Thanks to Jeff <jeff@dvns.com> for
      ;; explaining nreverse and for the save-excursion.)
      project-list)))


(defun timelog-read-subprojects (project)
  "Make a list of all subprojects of PROJECT.  Return the list.

PROJECT should be an existing project in `timelog-file'."
  (interactive (list
		(completing-read "Read subprojects of which project: "
				 (mapcar 'list timelog-project-list)
				 nil	; no predicate
				 t)))	; require-match
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    ;; Search for a project, then search forward for sth. like "o 2000/12/12
    ;; 12:23:45 " with "o" or "O" as first letter, and then search forward for
    ;; EOL.  Add the string between these two points to a list.

    (let ((subproject-list ()))
      (while
	  (re-search-forward project nil t) ; search for project
	(add-to-list 'subproject-list
		     (buffer-substring
		      (search-forward-regexp

		       ;; "^o [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+ ")
		       "^[oO] [0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]+ ")

		      (search-forward-regexp "$"))))

      ;;(bury-buffer)

      ;; return what we came her for
      (nreverse subproject-list))))


;; This is for you, Jeff <jeff@dvns.com>
(defun timelog-read-days (project)
  "Make a list of all days worked on PROJECT.  Return the list.

PROJECT should be an existing project in `timelog-file'."
  (interactive (list
		(completing-read "Read days worked on which project: "
				 (mapcar 'list timelog-project-list)
				 nil	; no predicate
				 t)))	; require-match
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    ;; Search for a project, then search forward for sth. like "2000/12/12".
    ;; Assume this is a data and add it to our list.

    (let ((days-list ()))
      (while
	  (re-search-forward project nil t) ; search for project
	(beginning-of-line)
	(search-forward-regexp
	 "[0-9]+/[0-9]+/[0-9]+")
	(add-to-list 'days-list
		     (match-string 0))
	(forward-line 1))

      ;;(bury-buffer)
      
      ;; return what we came her for
      (nreverse days-list))))


(defun timelog-delta-time ()
  "Return the time spent on the subproject point is on.

Time is given in seconds.  If dates for log-in and log-out are different, assume
that sb.  worked after midnight and adjust the result.  Do not assume that
anybody works more than 24 hours.  Return the time as a number."
  (interactive)
  
  ;; find the date and time strings
  (search-backward-regexp "^i ")
  
  (let*
      ((date-1 (progn
		 (search-forward-regexp
		  ;;		  "[0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9]")
		  "[0-9]+/[0-9]+/[0-9]+")
		 (match-string 0)))
       (time-1 (progn
		 (search-forward-regexp
		  ;;		  "[0-9][0-9]:[0-9][0-9]")
		  "[0-9]+:[0-9]+:[0-9]+")
		 (match-string 0)))
       (date-2 (progn
		 (search-forward-regexp
		  ;;		  "[0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9]")
		  "[0-9]+/[0-9]+/[0-9]+")
		 (match-string 0)))
       (time-2 (progn
		 (search-forward-regexp
		  ;;		  "[0-9][0-9]:[0-9][0-9]")
		  "[0-9]+:[0-9]+:[0-9]+")
		 (match-string 0)))
       (delta (-
	       (timelog-hh:mm:ss-to-seconds time-2)
	       (timelog-hh:mm:ss-to-seconds time-1))))

    ;; return delta.  if date-1 and date-2 are not string=, add (* 24 60 60) 86400
    ;; first.

    (if (not (string= date-1 date-2))
	(+ delta 86400)
      delta)))


(defun timelog-time-of-subproject (project subproject)
  "Return the overall time spent on PROJECT's SUBPROJECT in seconds."
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    ;; Search for project on one and subproject on the next line.  Extract the
    ;; date strings from either line and compare.  (If they are not string= we
    ;; assume that sb. worked until after midnight

    (let ((time 0))
      (while
	  (re-search-forward
	   ;;(concat "^i .*" project "\no .*" subproject)
	   ;; BUGFIX: Working 1h on "project" and
	   ;; 1h on project-a resulted in 2h work on "project".  This happened
	   ;; only when "project" is a substring of "other-project".
	   ;;(concat "^i .*" project "\n[oO] .*" subproject)
	   (concat "^i .*" project "\n[oO] .*" subproject "$")
	   nil t)
	(setq time
	      (+ time
		 (timelog-delta-time))))

      ;;(bury-buffer)

      time)))


(defun timelog-time-of-project (project)
  "Return the overall time spent on PROJECT in seconds."
  ;; Create a list of all subprojects for project, then a list of times for
  ;; these subprojects.  Then add the times in this list.  Not elegant, but it works.
  (let*
      ((subprojects (timelog-read-subprojects project))
       (subproject-times
	(mapcar
	 '(lambda (arg) (timelog-time-of-subproject project arg))
	 subprojects))			;list of times for each subproject
       ;; there must be some other way to add a list of numbers
       (project-time
	(apply '+ subproject-times)))	; NEVER use eval, Thomas!

    ;; return what we came for
    project-time))


;; The name of this function is not very good
(defun timelog-time-of-day (project day)
  "Return the overall time spent on PROJECT on DAY in seconds."
  
  ;;(save-excursion
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    ;; Search for a line with date and project.
    (let ((time 0))
      (while
	  (re-search-forward
	   (concat "^i " day ".*" project)
	   nil t)
	(setq time
	      (+ time
		 (timelog-delta-time))))

      ;;(bury-buffer)
      time)))


(defun timelog-hourly-rate-of-project (project amount)
  "Return the hourly rate for PROJECT if you get AMOUNT for the whole thing.

This could be useful, if you don't get paid by the hour."
  (interactive
   (list
    (completing-read (format "Project (default is %s): "
			     timelog-last-project)
		     (mapcar 'list timelog-project-list)
		     nil		; no predicat
		     t			; require-match
		     nil		; no initial input
		     nil		; no history
		     timelog-last-project) ; default
       (string-to-number (read-string "Amount: "))))

  ;; error check
  ;; timelog cannot summarize a project if you're logged in to it
  (if (and (string= project timelog-last-project)
	   (string= "i" timelog-last-event))
      (error "You must log out first"))

  (/					; divide
   amount				; amount by
   ;;(/ (timelog-time-of-project project) 60.0))) ; hours worked on project
   (/ (timelog-time-of-project project) 3600.0))) ; hours worked on project


;;; SUMMARY FUNCTIONS
 
;; timelog-summary-include-subprojects, timelog-summary-include-logfile,
;; timelog-summary-include-days, and timelog-summary-include-rates control what
;; functions are called from timelog-summarize-project.

(defun timelog-summary-insert-project-time (project)
  "Insert the time worked on PROJECT."

  (let ((project-time (timelog-time-of-project project)))
    (insert (timelog-seconds-to-hh:mm:ss project-time)) ; insert project-time
    (when timelog-summary-include-rates	; insert rates
      (insert " (" timelog-currency-string " "
	      (timelog-number-to-currency-string
	       (/
		(* timelog-hourly-rate project-time)
		;;60.0))
		3600.0))
	      ")"))))


(defun timelog-summary-insert-subprojects (project)
  "Insert a summary of the subprojects of PROJECT in the timelog summary buffer."
  (let* ((subprojects (timelog-read-subprojects project))
	 (subproject-times
	  (mapcar
	   '(lambda (arg) (timelog-time-of-subproject project arg))
	   subprojects))		;list of times for each subproject
	 ;; there must be some other way to add a list of numbers

	 ;; find the longest string in subprojects (compute length of each
	 ;; subproject, sort the list and get the car
	 (longest-string		; lenght of long. str. in subproj. + 2
	  (+ 2
	     (car (sort
		   (mapcar 'length subprojects)
		   '>)))))

    (while subprojects
      (insert
       (format (concat "%-" (number-to-string longest-string) "s")
	       (car subprojects))	; subproject + a lot of spaces
       ;;(format "%5s" (timelog-number-to-hh:mm (car subproject-times))))	; " 2:34"
       (format "%5s" (timelog-seconds-to-hh:mm:ss (car subproject-times))))
					; " 2:34:56"

      ;; this is for you, montrealf1@hotmail.com
      (when timelog-summary-include-rates
	(insert " (" timelog-currency-string " " ; (US-$ 123.45)
		(timelog-number-to-currency-string
		 (/
		  (* timelog-hourly-rate (car subproject-times))
		  ;;60.0))
		  3600.0))
		")"))

      (insert "\n")

      (setq subprojects (cdr subprojects)
	    subproject-times (cdr subproject-times)))))


(defun timelog-summary-insert-days (project)
  "Insert a summary of the days you worked on PROJECT."
  (let* ((days (timelog-read-days project))
	 (days-times			; sounds silly
	  (mapcar
	   '(lambda (arg) (timelog-time-of-day project arg))
	   days)))			;list of times for each day

    (while days
      (insert
       (car days) " "
       ;;(format "%6s" (timelog-number-to-hh:mm (car days-times))))
       (format "%6s" (timelog-seconds-to-hh:mm:ss (car days-times))))

      (when timelog-summary-include-rates
	(insert " (" timelog-currency-string " " ; (US-$ 123.45)
		(timelog-number-to-currency-string
		 (/
		  (* timelog-hourly-rate (car days-times))
		  ;;60.0))
		  3600.0))
		")"))

      (insert "\n")

      (setq days (cdr days)
	    days-times (cdr days-times)))))


(defun timelog-summary-insert-logfile (project)
  "Insert all in `timelog-file' lines for PROJECT."
      (let ((all-entries (timelog-read-project project)))

	(while all-entries
	  (insert (car all-entries) "\n")
	  (setq all-entries (cdr all-entries)))))



(defun timelog-summarize-project (project)
  "Create a buffer with a summary of PROJECT.

The buffer contains nicely formatted information on how much time was spent on
PROJECT and its subprojects."
  ;; This function mainly calls other functions.  The called functions
  ;; evaluate the timelog-file to compute everything they need.  Some things
  ;; may be computed more than once.  This is absolutely acceptable.  I want to
  ;; keep it as simple as possible and timelog-summary-insert-* functions
  ;; should require no additional args besides PROJECT.
  (interactive
   (list
    (completing-read (format "Project to summarize (default is %s): "
			     timelog-last-project)
		     (mapcar 'list timelog-project-list)
		     nil		; no predicat
		     t			; require-match
		     nil		; no initial input
		     nil		; no history
		     timelog-last-project))) ; default
  ;; timelog cannot summarize a project if you're logged in to it
  (if (and (string= project timelog-last-project)
	   (string= "i" timelog-last-event))
      (error "You must log out before you summarize this project"))

  ;; By default we only insert: "Timelog summary: Project \n Overall time:
  ;; 12:34"

  ;; prepare the the buffer
  (switch-to-buffer (get-buffer-create
		     (concat "Timelog Summary: " project)))
  (erase-buffer)
  (text-mode)

  ;; insert default
  (insert
   "Timelog Summary of \"" project "\"\n\n"
   "Overall Time: ")
  (timelog-summary-insert-project-time project) ; "12:23:45"
  (insert " \n")

  ;; insert subprojects
  (when timelog-summary-include-subprojects
    (insert "\nSubprojects:\n")
    (timelog-summary-insert-subprojects project))

  ;; insert day-list
  (when timelog-summary-include-days
    (insert "\nDays worked on project:\n")
    (timelog-summary-insert-days project))

  ;; insert lines from log-file
  (when timelog-summary-include-logfile
    (insert "\nLines from log-file\n")
    (timelog-summary-insert-logfile project))

  (goto-char (point-min)))


(defun timelog-make-summary (ask-for-amount)
  "Call `timelog-summary-function' interactively.

When called with prefix arg ASK-FOR-AMOUNT, bind `timelog-hourly-rate' to the
value returned by `timelog-hourly-rate-of-project' before calling
`timelog-summary-function'.  This is meant to be useful if you are not paid by
the hour."

;; The argument reading is strange, but I couldn't think of a better way.  The
;; argument list should be (project &optional amount prefix-arg).  When called
;; interactively with prefix arg, prompt for project and amount, otherwise
;; prompt for project only.

  (interactive "P")
;    (list (interactive "P")
; 	 (completing-read (format "Project to summarize (default is %s): "
; 				  timelog-last-project)
; 			  (mapcar 'list timelog-project-list)
; 			  nil		; no predicat
; 			  t			; require-match
; 			  nil		; no initial input
; 			  nil		; no history
; 			  timelog-last-project))) ; default

  (let ((project
	 (completing-read (format "Project to summarize (default is %s): "
				  timelog-last-project)
			  (mapcar 'list timelog-project-list)
			  nil		; no predicat
			  t			; require-match
			  nil		; no initial input
			  nil		; no history
			  timelog-last-project))) ; default
    (if ask-for-amount
	;; call with new value for hourly-rate
	(let ((timelog-hourly-rate
	       (funcall 'timelog-hourly-rate-of-project
			project
			(string-to-number (read-string "Amount: ")))))
	  (funcall timelog-summary-function project)) ; is funcall the right
					; thing?
      ;; call with nothing special
      (funcall timelog-summary-function project))))


;; customized-summary
(defun timelog-my-summarize-project (project)
  "Create a buffer with a summary of PROJECT.

Personal version of Thomas Gehrlein."
;;FIXME: add sample output to doc-string.

  (interactive
   (list
    (completing-read (format "Projekt (%s): " ; with default
			     timelog-last-project)
		     (mapcar 'list timelog-project-list)
		     nil		; no predicat
		     t			; require-match
		     nil		; no initial input
		     nil		; no history
		     timelog-last-project))) ; default

  ;; timelog cannot summarize a project if you're logged in to it
  (if (and (string= project timelog-last-project)
	   (string= "i" timelog-last-event))
      (error "You must log out before you summarize this project"))

  ;; prepare buffer
  (switch-to-buffer (get-buffer-create
		     (concat "Timelog Summary: " project)))
  (erase-buffer)
  (text-mode)

  ;; insert default
  (insert
   project "\n\n"
   "Gesamtzeit:       ")
  (let ((timelog-summary-include-rates nil))
    (timelog-summary-insert-project-time project))
  (insert
   " Stunden\n"
   "Stundensatz:      "
   (timelog-number-to-currency-string timelog-hourly-rate) ; 12,95
   " " timelog-currency-string "/Stunde\n"	; " DM/Stunde" DM/h
   "Zeilenzahl:       \n"
   "Zeilenpreis:      DM/Zeile\n"
   "Gesamtbetrag:     "			; overall amount
   (timelog-number-to-currency-string (* timelog-hourly-rate
					 (/ (timelog-time-of-project project)
					    ;;60.0))) ; min -> hours
					    3600.0))) ; sec -> hours
					 
   " "
   timelog-currency-string
   "\n"
   "Anmerkungen:\n\n")

  ;; insert subprojects
  (when timelog-summary-include-subprojects
    (insert "\nTeilprojekte:\n")
    (timelog-summary-insert-subprojects project))

  ;; insert day-list
  (when timelog-summary-include-days
    (insert "\nArbeitstage:\n")
    (timelog-summary-insert-days project))

  ;; insert lines from log-file
  (when timelog-summary-include-logfile
    (insert "\nEintrge aus der Log-Datei\n")
    (timelog-summary-insert-logfile project))

  (goto-char (point-min)))



;;; adding information to the timelog-file

(defun timelog-in (project)
  "Log in, recording the current time and the PROJECT you intend to work on."
  ;;read project.  Use timelog-project-list ist used for completion.
  (interactive (progn
		 ;;avoid logging in twice
		 (if (string= timelog-last-event "i")
		     (error "You're already logged in"))
		 (list
		  (completing-read
		   (format "Project to work on (default is \"%s\"): "
			   timelog-last-project)
		   (mapcar 'list timelog-project-list) ;list for
					;completion
		   nil			; no predicte
		   nil			; no require-match
		   nil			; no initial input
		   nil			; no history
		   timelog-last-project)))) ; default

  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-max))
    ;; insert sth. like "i 2000/10/17 12:34:56 some project"
    (insert "i "			; "i" means "in"
	    (format-time-string timelog-time-string)
	    " "
	    project
	    "\n")
    
    ;; update variables: timelog-last-event, timelog-project-list
    (setq timelog-last-event "i"
	  timelog-last-project project
	  timelog-project-list (add-to-list 'timelog-project-list project))
    ;;(bury-buffer)))
    (write-file timelog-file)))		; as suggested by Jeff Bowman
					; <jeff@dvns.com>


(defun timelog-out (subproject)
  "Log out, recording the current time and the SUBPROJECT you worked on."
  (interactive (progn
		 ;;avoid logging out, if you're not logged in
		 (if (string= timelog-last-event "o")
		     (error "You're not logged in"))

		 (list
		  (completing-read
		   (format "%s: Subproject you worked on (default is \"%s\"): "
			   timelog-last-project
			   timelog-last-subproject) ;prompt
		   (mapcar 'list timelog-subproject-list) ;list for completion
		   nil			; no predicte
		   nil			; no require-match
		   nil			; no initial input
		   nil			; no history
		   timelog-last-subproject)))) ;default

  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-max))
    ;; insert sth. like "o 2000/10/17 12:34:56 some subproject"
    (insert "o "			; "o" means "out"
	    (format-time-string timelog-time-string)
	    " "
	    subproject
	    "\n")

    ;;(bury-buffer)))
    (write-file timelog-file)		; as suggested by Jeff Bowman
					; <jeff@dvns.com>

    ;; message
    (when timelog-out-with-message
      (message "You worked %s hours."
	       (timelog-seconds-to-hh:mm:ss (timelog-delta-time))))

    ;; update variables: timelog-last-event, timelog-subproject-list
    (setq timelog-last-event "o"
	  timelog-last-subproject subproject
	  timelog-subproject-list (add-to-list 'timelog-subproject-list
					       subproject))))


;; stolen from timeclock.el
;; Add something to save timelog-file
(defun timelog-query-out ()
  "Ask the user before clocking out.

This is a useful function for adding to `kill-emacs-hook'."
  (when (and (string= timelog-last-event "i")
	     (y-or-n-p "You're currently clocking time, clock out? "))
    (timelog-out
     ;; read the arg for timelog-out
     (completing-read
      (format "%s: Subproject you worked on (default is \"%s\"): "
	      timelog-last-project
	      timelog-last-subproject)	;prompt
      (mapcar 'list timelog-subproject-list) ;list for completion
      nil				; no predicte
      nil				; no require-match
      nil				; no initial input
      nil				; no history
      timelog-last-subproject))))	; default


(defun timelog-change (subproject project)
  "Log out and then log in again.

Useful when you want to start on a new SUBPROJECT or PROJECT."
  (interactive (progn

		 ;; make sure, you're logged in
		 (if (string= timelog-last-event "o")
		     (error "You're not logged in"))

		 (list
		  ;; read subproject
		  (completing-read
		   (format "%s: Subproject you worked on (default is \"%s\"): "
			   timelog-last-project
			   timelog-last-subproject) ;prompt
		   (mapcar 'list timelog-subproject-list) ;list for completion
		   nil			; no predicte
		   nil			; no require-match
		   nil			; no initial input
		   nil			; no history
		   timelog-last-subproject) ;default
		  ;; read project
		  (completing-read
		   (format "Project to work on (default is \"%s\"): "
			   timelog-last-project)
		   (mapcar 'list timelog-project-list) ;list for
					;completion
		   nil			; no predicte
		   nil			; no require-match
		   nil			; no initial input
		   nil			; no history
		   timelog-last-project)))) ; default

  ;; The log-in time may be one minute after the log-out time, since
  ;; format-time-string is called twice.  If that is a real problem, something
  ;; should be done about it.  (It happened to me the other day.)
  (timelog-out subproject)
  (timelog-in project))



;;; Deleting information from the timelog-file
(defun timelog-delete-project (project)
  "Delete a PROJECT from the timelog file.

Use with care.  After a project is deleted, it is gone.  So better make sure to
summarize the project before you delete it."
  (interactive (list
		(completing-read "Project to delete: "
				 (mapcar 'list timelog-project-list)
				 nil	; no predicat
				 t)))	; require-match
  (save-window-excursion
    (find-file timelog-file)
    (goto-char (point-min))

    (while
	(re-search-forward project
			   nil
			   t)
      (delete-region
       ;;       (search-backward-regexp "^i [0-9][0-9][0-9][0-9]/[0-9][0-9]")
       ;;(search-backward-regexp "^i [0-9]+/[0-9]+")
       (search-backward-regexp "^i [0-9]+/[0-9]+/[0-9]+")
       (+ 1
	  ;;	  (search-forward-regexp "^o [0-9][0-9][0-9][0-9]/.*$"))))
	  ;;(search-forward-regexp "^o [0-9]+/.*$"))))
	  (search-forward-regexp "^[Oo] [0-9]+/.*$"))))
    
    ;; update
    (timelog-reread-logfile)
    (write-file timelog-file)
    ;; (bury-buffer)))
    ))

;;; timelog utility functions

;; obsolete, but renamed it anyway
(defun timelog-hh:mm-to-minutes (time)
  "Return the number of minutes of a TIME string like \"12:34\"."
  (interactive "sTime string (hh:mm): ")

  (let* ((hh
	  (string-to-int (substring time 0 2)))	;hh
	 (mm
	  (string-to-int (substring time 3 5)))) ;mm

    (+ mm				; mm + (hh * 60)
       (* 60 hh))))

(defun timelog-hh:mm:ss-to-seconds (time)
  "Return the number of seconds of TIME.  The format of TIME is \"12:34:56\"."
  (let* ((hh
	  (string-to-int (substring time 0 2)))	;hh
	 (mm
	  (string-to-int (substring time 3 5))) ;mm
	 (ss
	  (string-to-int (substring time 6 8)))) ;ss

    (+ ss				; ss + (mm * 60) + (hh * 3600)
       (* 60 mm)
       (* 3600 hh))))

(defun timelog-minutes-to-hh:mm (number)
  "Return NUMBER minutes as a string like \"12:23 \" or \"1:23\".
Hours are not zero-padded."
  (interactive "nHow many minutes: ")

  (let*
      ((hours (int-to-string (/ number 60)))
       ;; hh (since we're using ints, an int is returned)
       (mins (int-to-string (mod number 60)))
       (mins (if (= (length mins) 1)	;pad a "0", if length = 1
		 (concat "0" mins)
	       mins)))
    (concat hours ":" mins)))

(defun timelog-seconds-to-hh:mm:ss (number)
  "Return NUMBER seconds as a string like \"12:23:33\" or \"1:23:33\".

Hours are not zero-padded."
  (let*
      ((hours (int-to-string (/ number 3600)))
       ;; hh (since we're using ints, an int is returned)
       (mins (int-to-string (/ (mod number 3600) 60)))
       (mins (if (= (length mins) 1)	;pad a "0", if length = 1
		 (concat "0" mins)
	       mins))
       (secs (int-to-string (mod number 60)))
       (secs (if (= (length secs) 1)
		 (concat "0" secs)
	       secs)))
    (concat hours ":" mins ":" secs)))

(defun timelog-number-to-currency-string (amount)
  "Format AMOUNT as \"1,234.44\" or \"12,00\".

Uses decimal commas or whatever you call them, i.e. 1,234.12 instead of
1234.56, and 1.234,56 instead of 1234,56."

  (let* ((amount-string
	  (number-to-string (round (* 100 amount)))) ; 123456.7812 ->12345678
	 (dollars (substring amount-string ; 123456
			     0
			     (- (length amount-string) 2)))
	 (dollars
	  (with-temp-buffer
	    (insert-string dollars)
	    (goto-char (point-max))
	    (while (> (- (point) 4) 0)	; there must be 4 or more chars before
					; point
	      (backward-char 3)
	      (insert timelog-currency-decimal-string) ;","
	      (backward-char 1))	; move back over the inserted char
	    (buffer-string)))
	 (cents (substring amount-string ; 78
			   (- (length amount-string) 2))))
    (concat dollars
	    timelog-currency-separator-string
	    cents)))			; 123,456.78


;;; Prepare everything for use
(timelog-make-project-list)
(timelog-make-subproject-list)
(add-hook 'kill-emacs-hook 'timelog-query-out) ; don't kill emacs before you
					       ; logged out


(provide 'timelog)

;;; timelog.el ends here

