;;; tramp-ff.el --- Smarter file opening using Tramp and find-file

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: Tramp, find-file, root, su
;; Version: 1.0

;; This file is [not yet] part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2, or (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
;; A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License along with
;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; Basically if the current user doesn't have rights to edit the given file,
;; either because you are not root or it is owned by another user account, you
;; will be prompted with a user/password and then the operation will proceed
;; normally.  This should allow seamless operation within emacs.
;;
;; Also, if the user just enters an SSH path, tramp-ff will detect this and
;; morph it into a correct tramp path.  This helps out a little in the
;; "transparent" role.

;;; TODO:
;;
;; - If the user does NOT have write but *IS* the owner, just do a chmod u+w on
;;   the file.
;;
;; - should I make this a generic grant-access package or something?

;;; History:
;; 
;; Sun Dec 09 2001 10:57 PM (burton@openprivacy.org): We now support a
;; tramp-ff-default-method for local operations (usually 'su') and a
;; tramp-ff-method for remote (normal) operations.
;; 
;; Thu Dec 06 2001 12:48 AM (burton@openprivacy.org): now support numeric IP
;; addresses
;; 

(require 'tramp)

;;; Code:
(defcustom tramp-ff-method "rsync" "Method to use to transfer files with Tramp."
  :group 'tramp-ff
  :type 'string)

(defvar tramp-ff-default-method "sum" "Default method (used locally).")

(defvar tramp-ff-default-user "root" "Default user.")

(defvar tramp-ff-default-host "localhost" "Default host.")

(defvar tramp-ff-user nil "User to connect to local system with.")

(defun tramp-ff-find-file(filename)
  "Same as the `find-file' function.  Browse the filesystem and if you don't
   have access to this file use Tramp to prompt for user information"
  (interactive
   (list (read-file-name "Find file: ")))

  ;;SSH detection.  Allow the user to quickly enter an SSH path without having
  ;;to worry about adding the /r@sm: method.  This should allow the user to just
  ;;enter an SSH path as raw data.  Example:
  ;; M-x tramp-ff-find-file root@www.emacs.org:/etc/password

  (if (tramp-ff-is-ssh filename)
      (let((hostname nil)
           (username nil)
           (tramp-filename nil))

        (string-match "\\([a-z]+\\)@\\([^:]+\\):\\(.*\\)" filename)

        (setq username (match-string 1 filename))

        (setq hostname (match-string 2 filename))

        (setq filename (match-string 3 filename))

        ;;this needs to work with SSH!!!

        (setq tramp-filename (concat "/[" tramp-ff-method "/" username "@" hostname "]" filename))

        (message "Opening file via SSH: %s" tramp-filename)

        (find-file tramp-filename))

    ;;else assume normal operation
  
    ;; if the user can't read or write the file, and it exists, use tramp.
    (if (and (file-exists-p filename)
             (or (not (file-writable-p filename))
                 (not (file-readable-p filename))))
        ;;ok.  the current user can't edit this.  use Tramp.
        (progn

          (setq filename (tramp-ff-get-path filename))
          
          (if (find-file filename)
              (message "Opened file as '%s' using tramp." tramp-ff-user)))
    
      ;;else no need to use Tramp
      (find-file filename))))

(defun tramp-ff-get-path(filename)
  "Given a filename, prompt for the correct user and then use tramp to open the file"
  
  (if (null tramp-ff-user)
      (let(message)

        ;;don't know whether or not we should keep this in.  It theoretically
        ;;breaks emacs UI conventions, beeps should only occur on error.  The
        ;;difference here is that normally find-file doesn't prompt for input. 
        ;;(beep)
        
        ;;determine what username to use
        (setq message (concat "'" (user-login-name) "' doesn't have access.  Specify new user: "))

        ;;read the username from the user.
        (setq tramp-ff-user (read-string message tramp-ff-default-user))))
    
  ;;now use tramp to open the file.
  (concat "/[" tramp-ff-default-method "/" tramp-ff-user "@" tramp-ff-default-host "]" filename))

(defun tramp-ff-is-ssh(filename)
  "Return true if the given filename is an ssh path"

  ;;if the method is SSH and the regexp matches then we can return true.

  ;;HACK: this regexp might not match all UNIX users and hostname characters.  I
  ;;just added what I knew was valid.  Some characters migh t be missing.

  (and (or (string-equal tramp-ff-method "rsync")
           (string-equal tramp-ff-method "sm")
           (string-equal tramp-ff-method "su"))
       (string-match "^[a-z]+@[a-z0-9._-]+:.*" filename)))

;;setup the correct key-binding to replace the default one bound to find-file
(global-set-key [?\C-x?\C-f] 'tramp-ff-find-file)

(provide 'tramp-ff)

;;; tramp-ff.el ends here

