;;; transient.el --- create temporary transient message in a buffer which will
;;; disappear.  

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; TODO
;;
;;
;; if the bottom frame is less than min-window-height then we will get an
;; error... we will need to resize this window first and *then* split... and
;; then restore the window to its correct size.

;; if (keyboard-escape-quit) is ran from within the transient buffer window it
;; should destroy this window and select the previous one.  
;;
;;

(require 'electric)

(defgroup transient nil
  "Display transient messages in Emacs."
  :prefix "transient-"
  :group 'environment)

(defcustom transient-volatile t 
  "Use non-nil to erase previous output"
  :options '(nil t)
  :group 'transient)


(defcustom transient-audible nil 
  "Use non-nil to produce a bell when some output is displayed"
  :options '(nil t)
  :group 'transient)

(defcustom transient-raise nil 
  "Use non-nil to raise current frame when some output is displayed (could be *very* annoying)"
  :options '(nil t)
  :group 'transient)

(defcustom transient-hide-delay 5 
  "Time in seconds before a transient window is deleted"
  :type 'integer
  :group 'transient)

(defcustom transient-max-size 5 
  "Maximum size of the window"
  :type 'integer
  :group 'transient)

(defcustom transient-buffer "*transient*"
  "Buffer used for storing the messages output from the transient package."
  :type 'string
  :group 'transient)

(defun transient-message(&optional msg)
  "Output a transient message.  See `message'."
  (interactive)

  (if (null msg)
      (setq msg (read-string "Message: ")))
  
  ;;display the given message
  (transient-disp-window transient-buffer msg))


(defun transient-disp-window( transient-buffer msg )
  "Display some content specified by ``msg'' inside a temporary buffer.  Create
this buffer if necessary and put it inside a newly created window on the lowest
side of the frame."

  
  ;; Make sure we're not in the minibuffer
  ;; before splitting the window.

  (if (window-minibuffer-p)
      (other-frame -1))
    
      
  (let* ((this-buffer (current-buffer))
         (this-window (selected-window))
         (transient-disp-buf (set-buffer (get-buffer-create transient-buffer))))
    


    (if (cdr (assq 'unsplittable (frame-parameters)))
        ;; In an unsplittable frame, use something somewhere else.
        (display-buffer transient-disp-buf)
      (unless (or (special-display-p (buffer-name transient-disp-buf))
                  (same-window-p (buffer-name transient-disp-buf))
                  (get-buffer-window transient-buffer))
        
        ;; By default, split the bottom window and use the lower part.
        (transient-select-lowest-window)
        
        ;;if the current window is a mini-buffer .. we can't split it so go back
        ;;one window.
        (if (window-minibuffer-p)
            (other-window -1))

        ;; if the split would make a frame less than `window-min-height' then we
        ;; need to enlarge the current window.
        ;;
        ;; The algorithm is that we need to save the current window height then
        ;; enlarge it to twice `window-min-height'.  Then we make the bottom
        ;; window the size of `transient-max-size' and then resize the top to
        ;; its original value.
        ;; 
         (if (< (/ (window-height) 2)
                window-min-height)
             (progn
               (message "FIXME: window-height: %s" (window-height))
               ;;(let((window-height (window-height)))

               (enlarge-window (+ (window-height) transient-max-size))))
        

        (let((window-min-height 1))
          (split-window))

        (pop-to-buffer transient-disp-buf))
    
    (toggle-read-only 0)

    (if transient-volatile
        (erase-buffer))

    (insert-string msg)
    (toggle-read-only 1)


    (shrink-window-if-larger-than-buffer (get-buffer-window transient-disp-buf t))

    (if (> (window-height (get-buffer-window transient-disp-buf t)) transient-max-size)
        (shrink-window (- (window-height (get-buffer-window transient-disp-buf t)) transient-max-size)))
    (set-buffer-modified-p nil)
    (if transient-raise
        (raise-frame (selected-frame)))
    (select-window this-window)
    (if transient-audible
        (beep 1))
    (if transient-hide-delay
        (run-with-timer transient-hide-delay nil 'transient-hide-window transient-buffer)))))

(defun transient-hide-window (buffer)   
  (delete-window (get-buffer-window buffer t)))    ; TODO: cancel timer when some output comes during that time

(defun transient-select-lowest-window ()
  "Select the lowest window on the frame."
  (let* ((lowest-window (selected-window))
     (bottom-edge (car (cdr (cdr (cdr (window-edges))))))
         (last-window (previous-window))
         (window-search t))
    (while window-search
      (let* ((this-window (next-window))
         (next-bottom-edge (car (cdr (cdr (cdr 
                           (window-edges this-window)))))))
    (if (< bottom-edge next-bottom-edge)
        (progn
          (setq bottom-edge next-bottom-edge)
          (setq lowest-window this-window)))

    (select-window this-window)
    (if (eq last-window this-window)
        (progn
          (select-window lowest-window)
          (setq window-search nil)))))))


(provide 'transient)
