;;; u-appt.el --- Appointment parser
;;
;;  Copyright (C) 2002 by Ulf Jasper
;;
;;  Author:     Ulf Jasper <ulf.jasper@web.de>
;;  Keywords:   diary calendar outlook lotus
;;  Time-stamp: "14. November 2002, 22:05:52 (ulf)"
;;  Version:    0.1
;;  CVS:        $Id: u-appt.el,v 2.2 2002/11/14 21:06:06 ulf Exp $
;;
;;
;;  This program is free software; you can redistribute it and/or modify it
;;  under the terms of the GNU General Public License as published by the
;;  Free Software Foundation; either version 2 of the License, or (at your
;;  option) any later version.
;;
;;  This program is distributed in the hope that it will be useful, but
;;  WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;  General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License along
;;  with this program; if not, write to the Free Software Foundation, Inc.,
;;  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;;
;;; Commentary:
;; 
;;  This package contains functions for extracting appointments from a
;;  buffer and adding/converting them to an emacs diary file. If an
;;  appointment is found, you will be asked, whether you want to add
;;  the appointment to your diary file. Currently it will recognize
;;  appointments that were sent by
;;
;;  - MS Outlo*k (English, german)
;;  - L*tus N*tes (German only)
;;  - a very proprietary spanish appointment transmitter
;;
;;  In order to use, add the following to your .emacs file:
;;
;;  (autoload 'u-appt-check-outlook "u-appt" "Check for outlook invitations" t)
;;  (autoload 'u-appt-check-notes "u-appt" "Check for notes invitations" t)
;;
;;  or, to keep your .emacs smaller, in case you have quota problems:
;;
;;   (require 'u-appt)
;;
;;  Say `M-x u-appt-check-outlook' whenever you run across an
;;  appointment which you want to add to your diary. VM users want to
;;  put this into their vm config file:
;;
;;   (add-hook 'vm-select-new-message-hook 'u-appt-check-outlook)
;;   (add-hook 'vm-select-new-message-hook 'u-appt-check-notes)
;;
;;  Unfortunately this does not work for mime-encoded messages. They
;;  are decoded *after* that hook is run. So you have to call these
;;  functions by hand, or you could use
;;
;; (defadvice vm-decode-mime-message (after u-appt activate) 
;;         (u-appt-check-outlook))
;; (defadvice vm-decode-mime-message (after u-appt activate) 
;;         (u-appt-check-notes))
;;
;;  but that would call the functions each time you look at a
;;  message. Bad idea.
;;
;;; History:
;; 
;;  No history so far...
;;
;;; To do:
;;  
;;  Add the spanish outlook pattern
;;  Fixe the european style calendar problem!?!

(require 'appt)

;;; Code:
(require 'appt)

(defun u-appt-handle (subject string)
  "Asks user whether to add an appointment.
SUBJECT is the appointment-subject
STRING is the formatted diary entry"
  (if (y-or-n-p (format "Add appointment for `%s' to diary? " subject))
      (save-window-excursion
	(make-diary-entry string)
	(save-excursion
	  (set-buffer (find-buffer-visiting diary-file))
	  (save-buffer))
	(appt-initialize)
	(message "Addded %s to diary" string)))
  ;;(when (y-or-n-p "Appointment: Send an answer? ")
  ;;(vm-reply-include-text 1)
  ;;(u-header "Subject" (format "Re: %s" subject))
  ;;(mail-text)
  ;;(raise-frame))
  )

(defun u-appt-outlook (&rest args)
  "Search a buffer for an Outlook-style appointment and add a diary entry.
Optional argument ARGS is unused!"
  (interactive)
  (let (subject day month year time am-pm string type header-list
		(appt-found nil))
    (save-excursion
      (goto-char (point-min))
      (setq header-list (mail-header-extract))
      (setq subject (mail-header 'subject header-list))
      (setq type    (mail-header 'content-type header-list))
      (if (or (not type) (not (string-match "message" type)))
	  (progn
	    (cond (;; german
		   ;; example:
		   ;; Zeit: Freitag, 6. Dezember 2002 00:07 Stiefel rausstellen!
		   (re-search-forward
		    (concat "^\\(Zeit\\): [^ ]+, +\\([0-9]+\\)\. +"
			    "\\([A-Z][a-z][a-z]\\)[^ ]* +\\([0-9]+\\) +"
			    "\\([^ ]+\\)\\s-+\\(.*\\)$") nil t)
		   (setq day   (match-string 2))
		   (setq month (match-string 3))
		   (setq year  (match-string 4))
		   (setq time  (match-string 5))
		   (if (> (length (match-string 6)) 0)
		       (setq subject (match-string 6)))
		   ;; FIXME: this is european style!!!
		   ;; FIXME: must translate month names/abbrevs
		   (setq string (format "%s %s %s %s %s"
					day month year time subject))
		   (setq appt-found t))
		  (;; english
		   ;; example: FIXME
		   (re-search-forward
		    (concat "^\\(Start Date\\):\\s-+\\([0-9]+\\)/\\([0-9]+\\)/"
			    "\\([0-9]+\\)\\s-+"
			    "\\([0-9]+:[0-9]+\\)\\s-*\\([ap]m\\)"
			    "\\(.*\\)$") nil t)
		   (setq month (string-to-number (match-string 2)))
		   (setq day   (string-to-number (match-string 3)))
		   (setq year  (string-to-number (match-string 4)))
		   (setq time  (match-string 5))
		   (setq am-pm (match-string 6))
		   (if (> (length (match-string 7)) 0)
		       (setq subject (match-string 7)))
		   (setq string (format "%s %s %s%s"
					(calendar-date-string
					 (list month day year) t t)
					time am-pm subject))
		   (setq appt-found t))
		  (;; spanish -- not really outlook but similar
		   ;; example:
		   ;; Fecha: mircoles 16 de octubre 2002 10:00am  convocatoria
		   (re-search-forward
		    (concat "^\\(Fecha\\): [^ ]+ +\\([0-9]+\\) +[^ ]+ +"
			    "\\([A-Za-z][a-z][a-z]\\)[^ ]* +\\([0-9]+\\) +"
			    "\\([0-9]+:[0-9]+\\)\\s-*\\([ap]m\\)\\s-+"
			    "\\(.*\\)$") nil t)
		   (setq day   (match-string 2))
		   (setq month (capitalize (match-string 3)))
		   (setq year  (match-string 4))
		   (setq time  (match-string 5))
		   (setq am-pm (match-string 6))
		   (if (> (length (match-string 7)) 0)
		       (setq subject (match-string 7)))
		   ;; FIXME: this is european style!!!
		   ;; FIXME: must translate month names/abbrevs
		   (setq string (format "%s %s %s %s%s %s"
					day month year time am-pm subject))
		   (setq appt-found t))
		  ))))
    (when appt-found
      (u-appt-handle subject string))))

(defun u-appt-check-notes (&rest args)
  "Searches a (mail) buffer for a Lotus-Notes-style appointment and adds a diary entry.
Optional argument ARGS is unused!"
  (interactive)
  (let (subject start-day start-month start-year start-time end-day
		end-month end-year end-time string type header-list
		(appt-found nil)
		(has-end nil))
    (save-excursion
      (goto-char (point-min))
      (setq header-list (mail-header-extract))
      (setq subject (mail-header 'subject header-list))
      (setq type    (mail-header 'content-type header-list))
      (when (or (not type) (not (string-match "message" type)))
	(when (re-search-forward "^\\s-*Calendar Entry:\\s-*" nil t)
	  (when (re-search-forward
		 (concat "^\\s-*Begins:\\s-+\\([0-9]+\\).\\([0-9]+\\)."
			 "\\([0-9]+\\)\\s-+\\([0-9]+:[0-9]+\\)\\s-*"
			 ".*$") nil t)
	    (setq start-day   (string-to-number (match-string 1)))
	    (setq start-month (string-to-number (match-string 2)))
	    (setq start-year  (string-to-number (match-string 3)))
	    (setq start-time  (match-string 4))
	    (setq string (format "%s %s %s"
				 (calendar-date-string
				  (list start-month start-day start-year)
				  t t)
				 start-time subject))
	    (setq appt-found t))
	  (when (re-search-forward
		 (concat "^\\s-*Ends:\\s-+\\([0-9]+\\).\\([0-9]+\\)."
			 "\\([0-9]+\\)\\s-+\\([0-9]+:[0-9]+\\)\\s-*"
			 ".*$") nil t)
	    (setq end-day   (string-to-number (match-string 1)))
	    (setq end-month (string-to-number (match-string 2)))
	    (setq end-year  (string-to-number (match-string 3)))
	    (setq end-time  (match-string 4))
	    (if (and (eq start-day end-day)
		       (eq start-month end-month)
		       (eq start-year end-year))
		(setq string (format "%s %s-%s %s"
				     (calendar-date-string
				      (list start-month start-day start-year)
				      t t)
				     start-time end-time subject))
	      (setq string (format "%%%%(diary-block %s %s) %s"
				   (calendar-date-string
				    (list start-month start-day start-year)
				    t t)
				   (calendar-date-string
				    (list end-month end-day end-year)
				    t t)
				   subject)))
	    (setq has-end t)))))
    (when appt-found
      (u-appt-handle subject string))))


(provide 'u-appt)
;;; u-appt.el ends here
