;;; url-font-lock.el --- underline URLs within the current buffer with font-lock 

;; $Id: url-font-lock.el,v 1.2 2002/12/26 04:07:38 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; This package provides the ability to underline URLs within a buffer.

;;; TODO:
;;
;; - turn this into a minor-mode
;;
;;;- change package name to url-font-lock

(defface url-font-lock-face '((t (:underline t)))
  "Face used for a vizualizing a URL."
  :group 'codestack)

(defcustom url-font-lock-major-modes (list 'lisp-interaction-mode
                                           'lisp-mode
                                           'jde-run-mode
                                           'eshell-mode
                                           'fundamental-mode
                                           'text-mode
                                           'gnus-article-mode
                                           'message-mode)
  "Major modes where it is acceptable to run URL vizualization."
  :group 'url-font-lock
  :type '(repeat (symbol :tag "Major mode")))

(defcustom url-font-lock-exclude-special-buffers t
  "When true we will not enable in special buffers."
  :group 'url-font-lock
  :type 'boolean)

(defvar url-font-lock--generic-regexp
  (concat "\\("                                 ;;beginning of regexp paren
          "[a-z-]+:"                            ;;the beginning of the scheme and as in http: or file:
          "//?"                                 ;;beginning of the URL data.  The second / is optional
          "[0-9a-zA-Z:\\.]?"                    ;;URL authentication for some schemes
          "[0-9a-zA-Z\\.\\+/?,=%~_$&:#;{}@!-]+" ;; All valid URL characters
          "\\)")
  "Generic regular expression for all URL types.")

(defvar url-font-lock--generic-regexp-fentry
  (list url-font-lock--generic-regexp
        (list 1 ''url-font-lock-face 'append))
  "Entry to use within `font-lock-keywords'")

(defvar url-font-lock--urn-regexp
  (concat "\\("
          "urn:"              ;;beginning of a URN match
          "[^\"'?&\n ]+"      ;;all acceptable URN chars
          "\\)")
  "Regular expression for URNs.")

(defvar url-font-lock--urn-regexp-fentry
  (list url-font-lock--urn-regexp
        (list 0 ''url-font-lock-face 'append))
  "Entry to use within `font-lock-keywords'")

(defvar url-font-lock--file-regexp
  (concat "\\("
          "file:/"                       ;;beginning of file URL
          "+[0-9a-zA-Z\\./~#()_:-]+"     ;;all valid characters that can be in a file URL
          "\\)")
  "Regular expression for URNs.")

(defvar url-font-lock--file-regexp-fentry
  (list url-font-lock--file-regexp
        (list 1 ''url-font-lock-face 'append))
  "Entry to use within `font-lock-keywords'")

(defvar url-font-lock--mailto-regexp
  (concat "\\("
          "mailto:"             ;;required mailto prefix
          "[0-9a-zA-Z\\.-]+"    ;;valid username characters 
          "@"                   ;;@ character as in user@server.com
          "[0-9a-zA-Z.?=&-]+"   ;;all valid hostname characters.
          "\\)")
  "Regular expression for mailto URLs.")

(defvar url-font-lock--mailto-regexp-fentry
  (list url-font-lock--mailto-regexp
        (list 1 ''url-font-lock-face 'append))
  "Entry to use within `font-lock-keywords'")

(defvar url-font-lock--email-regexp
  (concat " \\("
          "[0-9a-zA-Z\\._-]+"     ;;valid username characters 
          "@"                     ;;@ character as in user@server.com
          "[0-9a-zA-Z.?=&-]+"     ;;all valid hostname characters.
          "\\)")
  "Regular expression for email addresses.  We need a space before this so that
a CVSROOT doesn't get used.")

(defvar url-font-lock--email-regexp-fentry
  (list url-font-lock--email-regexp
        (list 1 ''url-font-lock-face 'append))
  "Entry to use within `font-lock-keywords'")

(defun turn-on-url-font-lock-mode()
  "Highlight all URLs/URIs and e-mail addresses in the current buffer if necessary."
  (interactive)
  
  ;;it is important to restrict this operation to certain buffers.  Some buffer
  ;;types have complex file format and it is important that we do not use the
  ;;fontlocks in order to avoid confusion.
  (if (or (and (not (string-match "\\*" (buffer-name)))
               (not (equal major-mode 'dired-mode)))
          (member major-mode url-font-lock-major-modes))

      (url-font-lock-mode-on)))

(defun url-font-lock-mode-on()
  "Turn on `url-font-lock-mode'"
  (interactive)

  ;; We should modify font-lock-keywords directly
        
  ;; (t
  ;;  (" \\([0-9a-zA-Z\\._-]+@[0-9a-zA-Z.?=&-]+\\)"
  ;;   (1 'underline append))
  ;;  ("\\(mailto:[0-9a-zA-Z\\.-]+@[0-9a-zA-Z.?=&-]+\\)"
  ;;   (1 'underline append))
  ;;  ("\\(file:/+[0-9a-zA-Z\\./~#()_:-]+\\)"
  ;;   (1 'underline append))
  ;;  ("urn:[^\"'?&\n ]+"
  ;;   (0 'underline append))
  ;;  ("\\([a-z-]+:/\\(/\\)?[0-9a-zA-Z:\\.]?[0-9a-zA-Z\\.\\+/?,=%~_$&:#;{}@!-]+\\)"
  ;;   (1 'underline append))

   (add-to-list 'font-lock-keywords url-font-lock--generic-regexp-fentry t)
   (add-to-list 'font-lock-keywords url-font-lock--urn-regexp-fentry t)
   (add-to-list 'font-lock-keywords url-font-lock--file-regexp-fentry t)
   (add-to-list 'font-lock-keywords url-font-lock--mailto-regexp-fentry t)
   (add-to-list 'font-lock-keywords url-font-lock--email-regexp-fentry t)

  (font-lock-fontify-buffer))

(defun url-font-lock-mode-off()
  "Turn off `url-font-lock-mode'"
  (interactive)

  ;;remove my font lock entries
  (setq font-lock-keywords (delete url-font-lock--generic-regexp-fentry font-lock-keywords))
  (setq font-lock-keywords (delete url-font-lock--urn-regexp-fentry font-lock-keywords))
  (setq font-lock-keywords (delete url-font-lock--file-regexp-fentry font-lock-keywords))
  (setq font-lock-keywords (delete url-font-lock--mailto-regexp-fentry font-lock-keywords))
  (setq font-lock-keywords (delete url-font-lock--email-regexp-fentry font-lock-keywords))
  
  (font-lock-fontify-buffer))

(add-hook 'font-lock-mode-hook 'turn-on-url-font-lock-mode)

(provide 'url-font-lock)
